/*
   This file is part of the MoCA video library.
   Name: FileListTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#include <IImageStream.h>
#include <OImageStream.h>
#include <Image.h>
#include <FileList.h>
#include <CropFilter.h>

int main(int argc, char* argv[])
{
	
	string directory = "define directory, e.g.: D:/test/";

	string oldExt = "ppm";
	string newExt = "jpg";

	vector<FileInfo> files;
	FileList fl(directory, true);    
	fl.process("*." + oldExt, files);



	// open and save files
    vector<FileInfo>::iterator it;
	for (it = files.begin(); it != files.end(); it++) {

		cout << "old file: "<< (*it).name << endl;

		Image<uint8> img;
		
		try {
			// open input filestream and read image
			IImageStream<uint8> iis((*it).name);
			iis.read(img);
		} catch (MocaException e) {
			cout << "IImageStream " << e.what() << endl;
			exit (-1);
		} // try

		ImageSize size=img.getSize();
		CropFilter<uint8> crf (ImagePosition (5,10), ImagePosition (size[DIM_X]-10, size[DIM_Y]-10));
		crf.process(img);
		

		try {
			// open output filestream and write image
			string filename = (*it).name;
	        int p = filename.find_last_of('.') +1;
		    filename.replace(p, 3, newExt);
			cout << "new file: "<< filename << endl;

			OImageStream<uint8> ois(filename);
			ois.write(img);
		} catch (MocaException e) {
			cout << "OImageStream " << e.what() << endl;
			exit (-1);
		} // try


	} // for
	cout << endl << files.size() << " files found." << endl;

	return 0;
} // main




