/*
   This file is part of the MoCA video library.
   Name: Histogram.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#include <iostream>
#include <Image.h>
#include <IImageStream.h>
#include <GrayscaleFilter.h>
#include <HistogramFeature.h>
#include <ImageStatistic.h>

int main(int argc, char* argv[])
{
    if (argc != 2) {
		cerr << "Usage: HistogramTest file" << endl;
		cerr << endl;
		cerr << "calculate the histogram of an grayscale image and store it as \"hist.txt\" " << endl << endl;
        return 1;
    } // if

    // open input filestream and read image
    Image<uint8> img;
	IImageStream<uint8> iis(argv[1]);
	iis.read(img);
	iis.close();

	// convert to grayscale image
	GrayscaleFilter<uint8> gray;
	gray.process(img);

	int8 msbits = 8;
	bool grayscale=false;
	bool smoothing=false;
	uint8 areas=1;
	HistogramFeature hf (msbits, grayscale, smoothing, areas);
	Signal<uint32> hist;
	hf.process (img, hist);

	// print statistics
	ImageStatisticData data;
	ImageStatistic is;
	is.process(img, data);
	cerr << data << endl;

	// contrast hist
	int32 counter = (int32)(0.1 * img.getSize()[DIM_X] * img.getSize()[DIM_Y]);
	int32 i=0;
	while (counter > 0) {
		counter -= hist(i);
		i++;
	} // for
	int pmin=i-1;
	if (pmin < 0) pmin=0;
	if (pmin > 255) pmin=255;

	counter = (int32)(0.1 * img.getSize()[DIM_X] * img.getSize()[DIM_Y]);
	i=255;
	while (counter > 0) {
		counter -= hist(i);
		i--;
	} // for
	int pmax=i+1;
	if (pmax < 0) pmax=0;
	if (pmax > 255) pmax=255;
	if (pmax < pmin) pmax=pmin;
	cerr << "pmin: " << (int) pmin << endl;
	cerr << "pmax: " << (int) pmax << endl;


	// store histogram values
	ofstream ofs ("hist.txt", std::ios::out | std::ios::binary); 
	for (int32 i=0; i<hist.getSize()[DIM_X]; i++) {
		ofs << (int32)hist(i) << endl;
	} // for
	ofs.close();

    return 0;
} // main



