/*
   This file is part of the MoCA video library.
   Name: MpegReadTest.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/

// C++ includes
#include <iostream>
using namespace std;
#include <iomanip>
#include <sstream>

// Moca includes
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <Video.h>
#include <VideoDecoder.h>


int main(int argc, char* argv[])
{

    if (argc != 5) {
        cerr << "read a mpeg file and save jpg images" << endl << endl;
        //cerr << "read a mpeg file and save png images" << endl << endl;
        cerr << "Usage: " << endl;
		//cerr << argv[0] << " src(mpg)  dest-path(for png's)  startFrame  nrOfFrames" << endl;
		cerr << argv[0] << " src(mpg)  dest-path(for jpg's)  startFrame  nrOfFrames" << endl;
        return 1;
    } // if
	string mpegFileName = argv[1];
	string outPath      = argv[2];
	int startFrame      = atoi(argv[3]);
	int numberOfFrames  = atoi(argv[4]);

	try {
		VideoDecoder video (mpegFileName);
		VideoSize size = video.getSize();
		if (startFrame < 0) {
			startFrame = 0;
		} // if
		if (startFrame + numberOfFrames < size[DIM_Z]) {
			size[DIM_Z] = startFrame + numberOfFrames;
		} // if

		cout << "read and save images (" << startFrame << "..." << size[DIM_Z]-1 << ")..." << endl;
		Image<uint8> img;
		for (int i=startFrame; i<size[DIM_Z]; i++) {
			cerr << "frame: " << i << endl;
			img = video.getImage(i);
			
			// save images
			stringstream oss;
			//oss << outPath << "img" << setw(6) << setfill('0') << (i) << ".png" << ends;
			oss << outPath << "img" << setw(6) << setfill('0') << (i) << ".jpg";
			cout << "save image: " << oss.str() << endl;
			OImageStream<uint8> ois(oss.str());
			ois.write (img);
		} // for
    } catch (MocaException e) {
        cerr << "error: could not save jpeg images." << endl;
        //cerr << "error: could not save png images." << endl;
        cerr << e.what() << endl;
        exit (-1);
    } // try
	cout << endl;

	return 0;
} // main

