/*
   This file is part of the MoCA video library.
   Name: AudioEnergy.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef AUDIO_ENERGY_H
#define AUDIO_ENERGY_H

// C includes
#include <math.h>


// C++ includes
#include <iostream>
using namespace std;


// MoCA includes
#include <MocaSignal.h>
#include <IWaveAudio.h>
#include <OTagStream.h>
#include <WriteMocaData.h>


const static string AUDIO_ENERGY_VERSION  ="Version-0.01";
const static string AUDIO_ENERGY_FILENAME ="audioenergy.idx";

struct AudioEnergyData
{
	AudioEnergyData () 
	{
		version     = AUDIO_ENERGY_VERSION;
		filename    = "";
		blockLengthMilliSeconds = 20;
		size[DIM_X] = 1;
		size[DIM_Y] = 1;
		size[DIM_Z] = 1;
		size[DIM_BAND] = 1;
	} 
    AudioEnergyData& operator=(const AudioEnergyData &other) {
		if (this != &other) {
			version           = other.version;
			filename          = other.filename;
			energy            = other.energy;
			blockLengthMilliSeconds = other.blockLengthMilliSeconds;
			size              = other.size;
		} // if
        return *this;
    } // operator=


	string version;
	string filename;
	int32 blockLengthMilliSeconds;  // in milliseconds
	SignalSize size;
	Signal<int32> energy;
};

std::ostream& operator<< (ostream &, const AudioEnergyData&);
OTagStream& operator<< (OTagStream&, const AudioEnergyData&);
ITagStream& operator>> (ITagStream&, AudioEnergyData&);


/*
	Constructor: initializes variables
	process(AudioEnergyData &energyData): 
*/
class AudioEnergy
{
public:

	// the default block length is 20 milliseconds
	AudioEnergy(const string &filenameWav, int32 blockLengthMilliSeconds = 20)
	{
		pathFilename      = filenameWav;
		this->blockLengthMilliSeconds = blockLengthMilliSeconds;
	}
	~AudioEnergy() {};

	void process (AudioEnergyData &data);

private:
	
	void readValues(AudioEnergyData &data);

	string pathFilename;
	int32 blockLengthMilliSeconds;     // aggregate samples in one block, the default block length is 20 milliseconds
};

#endif
