/*
   This file is part of the MoCA video library.
   Name: ContourEdgeList.h
   Copyright (C) 2004 Oliver Schuster und der kleine Gerald

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster und der kleine Gerald

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: ContourEdgeList.h,v 1.2 2004/03/05 15:21:24 kopf Exp $

*/

#ifndef CONTOUR_EDGE_LIST_H
#define CONTOUR_EDGE_LIST_H

// C includes
#include <math.h>


// C++ includes
// #include <stack>
#include <vector>
#include <algorithm>

// MoCA includes
#include <Image.h>
#include <ImageSize.h>
#include <ImageEdgePosition.h>
#include <MocaExceptions.h>
#include <ForEachAlgos.h>
#include <GrayscaleFilter.h>
#include <OImageStream.h>
#include <BinarizeFilter.h>


class ContourEdgeList
{
public:
    ContourEdgeList (uint8 backgroundColor=255) 
	{
	    background=backgroundColor;
		shiftOffset = 3;
	}

    void process (const Image<uint8>&, vector<ImageEdgePosition>&);

	// get the length of the contour (in pixel)
    ieee64 getLength (vector<ImageEdgePosition>&);


private:
	void convertImage (const Image<uint8> &img1, Image<uint8> &dest);
    bool markOuterEdges (const Image<uint8>& src);

    struct SampleEdges
    {
        bool north, south, east, west;
    };

    Image<SampleEdges> edgeImage;
	uint8 background;
	int32 shiftOffset; // temporary shift of image (border is added)

};

#endif
