/*
    CssDisplayResults.cpp     visualize the results of the CSS approach
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <CssDisplayResult.h>

CssDisplayResult::
CssDisplayResult(vector<FileInfo> &databaseList, vector<FileInfo> &sequenceList, 
				 bool showStatus, bool extended, bool useGlobalDescriptors)
{
    sequenceSize     = 0;
    objectClassSize  = 0;
    databaseSize     = 0;
    this->databaseList = databaseList;
    this->sequenceList = sequenceList;
    this->showStatus   = showStatus;
	this->extended     = extended ;
	this->useGlobalDescriptors = useGlobalDescriptors;
    
    int i;
    for (i=0; i<MAX_SEQUENCE_SIZE; i++) {
        seq[i].fn = "";
    } // for
    for (i=0; i<MAX_DATABASE_SIZE; i++) {
        data[i].fn = "";
        data[i].objectType = "";
        data[i].objectTypeNr = -1;
    } // for       
    for (i=0; i<MAX_OBJECT_CLASS; i++) {
        objectClass[i] = "";
    } // for
    
    readDatabaseFiles();
    readSequenceFiles();
}


void
CssDisplayResult::readDatabaseFiles()
{
    // read filename, object class name, object class number, css-maximum-list:
    // data[i].fn, data[i].objectType, data[i].objectTypeNr, data[i].cm
	vector<FileInfo>::iterator it;
    for (it = databaseList.begin(); it != databaseList.end(); it++) {

		// define filenames
		string pathFile = (*it).name;
		string file = (*it).name;
		int p = file.find_last_of('/')+1;
        file = file.substr(p);
		p = file.length()-8;
        file = file.substr(0,p);

		string path = (*it).name;
		p = path.find_last_of('/')-10;
        path = path.substr(0,p);

		p = path.length()-1;
		string objectClass = path.substr(0,p);
		p = objectClass.find_last_of('/')+1;
        objectClass = objectClass.substr(p);

		string cssFile = (*it).name;
		string xCssFile = path + "result/cssX/" + file + "_CSSX.xtag";

		// filename for geometric features
		string geometricFile = path + "result/geometric/" + file + "_GEOMETRIC.tag";
		

		// read standard css image
        data[databaseSize].fn = pathFile;
        data[databaseSize].objectType = objectClass;
		{
			ITagStream its ((*it).name);
			its >> data[databaseSize].cmS;
		}

		// read geometric features
		if (useGlobalDescriptors) {
			ITagStream its (geometricFile);
			its >> data[databaseSize].geometric;
		}


		// try to read extended css image
		if (extended) {

			// read extended css data
			try {
				ITagStream its (xCssFile);
				its >> data[databaseSize].cmX;
			} catch(...) {
				cerr << "Error: CssDisplayResult::readDatabaseFiles: cannot open extended CSS image data" << endl;
				cerr << "       extended CSS matching is disabled!" << endl;
				extended = false;
			} // catch
		} // if

		while (data[databaseSize].fn.length() < 70) {
			data[databaseSize].fn = data[databaseSize].fn + " ";
		} // while
		// be sure, that all object-type-names have the same length of 22 characters
		while (data[databaseSize].objectType.length() < 22) {
			data[databaseSize].objectType = data[databaseSize].objectType + " ";
		} // while
		databaseSize++;



    } // for    
	if (showStatus) {
		cout << "("<< databaseSize << ")"<< endl;    
	} // if

	// get all object class names:
    // objectClass[i]
	if (showStatus) {
		cout << "read object classes " << flush;
	} // if
    set<string> objectClassSet;
    int i, j;    
    for (i=0; i<databaseSize; i++) {
		objectClassSet.insert(data[i].objectType);
    } // for
	set<string>::iterator its;
    for (its = objectClassSet.begin(); its != objectClassSet.end(); its++) {
		objectClass[objectClassSize] = (*its);
		objectClassSize++;
	} // for
    for (i=0; i<databaseSize; i++) {
		for (j=0; j<objectClassSize; j++) {
			if (data[i].objectType == objectClass[j]) {
				data[i].objectTypeNr = j;
			} // if
		} // for
	} // for
    
    return;
} // :readDatabaseFiles


void
CssDisplayResult::readSequenceFiles()
{
    
    // read all "tag"-files i(sequence) and set filename, sequence-list (difference is calculated in matching): 
    // seq[i].fn, seq[i].diff, seq[i].cm
	vector<FileInfo>::iterator it;
    for (it = sequenceList.begin(); it != sequenceList.end(); it++) {

		// set filenames
		string path = (*it).name;
		int p = path.find_last_of('/')-10;
        path = path.substr(0,p);

		string file = (*it).name;
		p = file.find_last_of('/')+1;
        file = file.substr(p);
		p = file.length()-8;
        file = file.substr(0,p);

		string cssFile = (*it).name;
		string xCssFile = path + "result/cssX/" + file + "_CSSX.xtag";


		// filename for geometric features
		string geometricFile = path + "result/geometric/" + file + "_GEOMETRIC.tag";

		// read css data
        seq[sequenceSize].fn = cssFile;
		{
			ITagStream its (cssFile);
			its >> seq[sequenceSize].cmS;
		}

		// read geometric features
		if (useGlobalDescriptors) {
			ITagStream its (geometricFile);
			its >> seq[sequenceSize].geometric;
		}


		// try to read extended css image
		if (extended) {

			// read extended css data
			try {
				ITagStream its (xCssFile);
				its >> seq[sequenceSize].cmX;
			} catch(...) {
				cerr << "Error: CssDisplayResult::readSequenceFiles: cannot open extended CSS image data" << endl;
				cerr << "       extended CSS matching is disabled!" << endl;
				extended = false;
			} // catch
		} // if


        sequenceSize++;
    } // for    
    return;
} // :readDatabaseFiles


/*!
 */
void
CssDisplayResult::showAll(int32 maxItems)
{

    // show results
    int i;
	ieee64 totalValue = 0;
	CssResults result [MAX_OBJECT_CLASS];
    for (i=0; i<MAX_OBJECT_CLASS; i++) {
	    result[i].topMatch = 0;
	    result[i].top5Match = 0;
	    result[i].top10Match = 0;
	    result[i].top20Match = 0;
	    result[i].f1 = 0;
	    result[i].topValuePercent = 0;
	    result[i].foundAll = 0;
	    result[i].notFoundAll = 0;
	    result[i].found10 = 0;
	    result[i].notFound10 = 0;
	} // for
	int avgSum     = 0;
	int avgCounter = 0;
    

	cout << "show results" << endl << endl;
	list<CssDifference>::iterator itd;
	int32 totalNoHits=0;
    for (i=0; i<sequenceSize; i++) {
		
		CssSequence tempSeq = seq[i];       // 	fn, list:diff, list:cm
		
		tempSeq.diff.sort();
		if (tempSeq.diff.size()==0) {
			totalNoHits++;
		} else {
			// get first element
			itd = tempSeq.diff.begin();
			if ((*itd).distance == INT_MAX) totalNoHits++;
		}

		int c=0;
#if 0
		cout << endl << tempSeq.fn << endl;
#endif
		itd = tempSeq.diff.begin();
		int32 itemNumber=0;
		while ((c < MAX_DATABASE_SIZE) && (itd != tempSeq.diff.end())) {	// itd zeigt auf CssDatabaseResultItem (distance ,dbRef)
			itemNumber++;
			if (itemNumber>maxItems) break;
			
			CssDifference tmpDiff = (*itd);
			int tmpObjectClassId = data[tmpDiff.dbRef].objectTypeNr;

			if (tmpDiff.distance == INT_MAX) {
				result[tmpObjectClassId].notFoundAll++;
				if (c <= 10) {
					result[tmpObjectClassId].notFound10++;
				} // if
			} else {
				
#if 0
				// write filename
				if ((c+1) < 10) {
					cout << " ";
				} // 
				if ((c+1) < 100) {
					cout << " ";
				} // 
				cout << (c+1) << ". " << data[tmpDiff.dbRef].fn << " ";
				
				// write object class
				//cout << data[(tmpDiff).dbRef].objectType << " ";
				cout << objectClass[tmpObjectClassId] << " ";
				
				// write value
				cout << tmpDiff.distance << endl;
#endif
				
				// save top matches in object class
				if (c==0) {
					result[tmpObjectClassId].topMatch++;
					result[tmpObjectClassId].topValuePercent += 1.0/((ieee64)tmpDiff.distance);
					if (tmpDiff.distance > 0) {
						totalValue += 1.0/((ieee64)tmpDiff.distance);
					} // if
					avgSum += tmpDiff.distance;
					avgCounter++;
				} // if
				if (c<=5) {
					result[tmpObjectClassId].top5Match++;
				} // if
				if (c<=10) {
					result[tmpObjectClassId].top10Match++;
				} // if
				if (c<=20) {
					result[tmpObjectClassId].top20Match++;
				} // if
				switch (c) {
					case 0: 
						result[tmpObjectClassId].f1 += 10;
						break;
					case 1: 
						result[tmpObjectClassId].f1 += 6;
						break;
					case 2: 
						result[tmpObjectClassId].f1 += 5;
						break;
					case 3: 
						result[tmpObjectClassId].f1 += 4;
						break;
					case 4: 
						result[tmpObjectClassId].f1 += 3;
						break;
					case 5: 
						result[tmpObjectClassId].f1 += 2;
						break;
					case 6: 
						result[tmpObjectClassId].f1 += 1;
						break;
				} // switch
				result[tmpObjectClassId].foundAll++;
				if (c<=10) {
					result[tmpObjectClassId].found10++;
				} // if
			} // else
			itd++;
			c++;

		} // while
	} // for
	

    
    // show results on class basis
	cout << endl;
    cout << "class                   top  value  F1-method  top-5  top-10  top-20  found-10 " << endl;
    cout << "-------------------------------------------------------------------------------" << endl;

	int sum=0;
    for (i=0; i<objectClassSize; i++) {
		if (result[i].topMatch != 0) {
			sum += result[i].topMatch;
			cout << objectClass[i] << " ";
			// print nr
			if (result[i].topMatch < 100) {
				cout << " ";
			}
			if (result[i].topMatch < 10) {
				cout << " ";
			}
			cout << result[i].topMatch << "   ";
			
			// print value
			if (totalValue <= 0) {
				cout << "xxxxx       ";
			} else {
				ieee64 tmpV = 100 * result[i].topValuePercent / totalValue;
				sum = (int) tmpV;
				if (sum < 100) {
					cout << " ";
				} // if
				if (sum < 10) {
					cout << " ";
				} // if
				cout << sum << "%      ";
			} // else

			// print F1 mothod
			sum = result[i].f1;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "      ";

			// print top 5 match
			sum = result[i].top5Match;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "    ";

			// print top 10 match
			sum = result[i].top10Match;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "     ";

			// print top 20 match
			sum = result[i].top20Match;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "    ";

			// print found10
			sum = result[i].found10;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "/";
			sum = result[i].found10 + result[i].notFound10;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "   ";

#if 0
			// print foundAll
			sum = result[i].foundAll;
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << "/";
			sum = result[i].notFoundAll + result[i].foundAll;
			if (sum < 1000) {
				cout << " ";
			} // if
			if (sum < 100) {
				cout << " ";
			} // if
			if (sum < 10) {
				cout << " ";
			} // if
			cout << sum << " ";
#endif

			cout << endl;

		} // if
	} // for



	//sum = sequenceSize - sum;
	if (totalNoHits > 0) {
		cout << "No matches ";
		if (totalNoHits < 100) {
			cout << " ";
		}
		if (totalNoHits < 10) {
			cout << " ";
		}
		cout << totalNoHits << endl;
	} // if
	if (avgCounter > 0) {
		cout << "Average top 1 value: " << avgSum / avgCounter << endl;
	} // if

	cout << endl << endl;
    
    return ;
} // CssDisplayResult::showAll



/*!
 */
void
CssDisplayResult::printMatrix(string &matrixFile)
{
	ofstream ofs (matrixFile.c_str());

    // print results
    int i;
	ieee64 totalValue = 0;
	CssResults result [MAX_OBJECT_CLASS];
    for (i=0; i<MAX_OBJECT_CLASS; i++) {
	    result[i].topMatch = 0;
	    result[i].top5Match = 0;
	    result[i].top10Match = 0;
	    result[i].top20Match = 0;
	    result[i].f1 = 0;
	    result[i].topValuePercent = 0;
	    result[i].foundAll = 0;
	    result[i].notFoundAll = 0;
	    result[i].found10 = 0;
	    result[i].notFound10 = 0;
	} // for
	int avgSum     = 0;
	int avgCounter = 0;
    

	list<CssDifference>::iterator itd;
	int32 totalNoHits=0;
    for (i=0; i<sequenceSize; i++) {
		
		CssSequence tempSeq = seq[i];       // 	fn, list:diff, list:cm
		
		tempSeq.diff.sort();
		if (tempSeq.diff.size()==0) {
			totalNoHits++;
		} else {
			// get first element
			itd = tempSeq.diff.begin();
			if ((*itd).distance == INT_MAX) totalNoHits++;
		}

		int c=0;
		itd = tempSeq.diff.begin();
		int32 itemNumber=0;
		while ((c < MAX_DATABASE_SIZE) && (itd != tempSeq.diff.end())) {	// itd zeigt auf CssDatabaseResultItem (distance ,dbRef)
			itemNumber++;
			
			CssDifference tmpDiff = (*itd);
			int tmpObjectClassId = data[tmpDiff.dbRef].objectTypeNr;

			if (tmpDiff.distance == INT_MAX) {
				result[tmpObjectClassId].notFoundAll++;
				if (c <= 10) {
					result[tmpObjectClassId].notFound10++;
				} // if
			} else {				
				
				// save top matches in object class
				if (c==0) {
					result[tmpObjectClassId].topMatch++;
					result[tmpObjectClassId].topValuePercent += 1.0/((ieee64)tmpDiff.distance);
					if (tmpDiff.distance > 0) {
						totalValue += 1.0/((ieee64)tmpDiff.distance);
					} // if
					avgSum += tmpDiff.distance;
					avgCounter++;
				} // if
				if (c<=5) {
					result[tmpObjectClassId].top5Match++;
				} // if
				if (c<=10) {
					result[tmpObjectClassId].top10Match++;
				} // if
				if (c<=20) {
					result[tmpObjectClassId].top20Match++;
				} // if
				switch (c) {
					case 0: 
						result[tmpObjectClassId].f1 += 10;
						break;
					case 1: 
						result[tmpObjectClassId].f1 += 6;
						break;
					case 2: 
						result[tmpObjectClassId].f1 += 5;
						break;
					case 3: 
						result[tmpObjectClassId].f1 += 4;
						break;
					case 4: 
						result[tmpObjectClassId].f1 += 3;
						break;
					case 5: 
						result[tmpObjectClassId].f1 += 2;
						break;
					case 6: 
						result[tmpObjectClassId].f1 += 1;
						break;
				} // switch
				result[tmpObjectClassId].foundAll++;
				if (c<=10) {
					result[tmpObjectClassId].found10++;
				} // if
			} // else
			itd++;
			c++;

		} // while
	} // for
	

    
    // show results on class basis
	int sum=0;
    for (i=0; i<objectClassSize; i++) {
		if (result[i].topMatch != 0) {
			sum += result[i].topMatch;
			// print value
			if (totalValue <= 0) {
			} else {
				ieee64 tmpV = 100 * result[i].topValuePercent / totalValue;
				sum = (int) tmpV;
			} // else

			// print F1 mothod
			sum = result[i].f1;

			// print top 5 match
			sum = result[i].top5Match;

			// print top 10 match
			sum = result[i].top10Match;

			// print top 20 match
			sum = result[i].top20Match;

			// print found10
			sum = result[i].found10;
			sum = result[i].found10 + result[i].notFound10;

			// print foundAll
			sum = result[i].foundAll;
			sum = result[i].notFoundAll + result[i].foundAll;
		} // if
	} // for



	sum=0;
    for (i=0; i<objectClassSize; i++) {
		ofs << objectClass[i] << " ";

		for (int32 j=0; j<sequenceSize; j++) {
	
			// get data for frame i of sequence
			CssSequence tempSeq = seq[j];       
			tempSeq.diff.sort();

			// analyse references to database
			itd = tempSeq.diff.begin();
			bool objectFound = false;
			while ((!objectFound) && (itd != tempSeq.diff.end())) {	

				// search most similar object with object nnumber i
				int distance = (*itd).distance;
				if (distance > 10000) distance = 10000;
				int dbRef    = (*itd).dbRef;
				if (data[dbRef].objectTypeNr == i) {
					if (distance < 10000) ofs << " ";
					if (distance < 1000) ofs << " ";
					if (distance < 100) ofs << " ";
					if (distance < 10) ofs << " ";
					ofs << distance << ", ";
					objectFound=true;
				} // if
				itd++;
				/*
				data[dbRef].


			int tmpObjectClassId = data[tmpDiff.dbRef].objectTypeNr;
			if (tmpObjectClassId == i) {
				int32 diffValue = data[tmpDiff.dbRef].objectTypeNr;
			} // if
*/
			} // while


			

		} // for
		ofs << endl;
	} // for
	ofs << endl;
	ofs.close();

    
    return ;
} // CssDisplayResult::printMatrix




/*!
 */
void
CssDisplayResult::show(string &searchString)
{

    // show results
    int i;
	ieee64 totalValue = 0;
	CssResults result [MAX_OBJECT_CLASS];
    for (i=0; i<MAX_OBJECT_CLASS; i++) {
	    result[i].topMatch = 0;
	    result[i].top5Match = 0;
	    result[i].top10Match = 0;
	    result[i].top20Match = 0;
	    result[i].f1 = 0;
	    result[i].topValuePercent = 0;
	    result[i].foundAll = 0;
	    result[i].notFoundAll = 0;
	    result[i].found10 = 0;
	    result[i].notFound10 = 0;
	} // for
	int avgSum     = 0;
	int avgCounter = 0;

    while (searchString.length() < 10) {
         searchString= searchString + " ";
    } // while
    int goodMatches   = 0;
    int badMatches    = 0;
    int missedMatches = 0;
    
	list<CssDifference>::iterator itd;
    for (i=0; i<sequenceSize; i++) {
		
		CssSequence tempSeq = seq[i];       // 	fn, list:diff, list:cm
		
		tempSeq.diff.sort();
		int c=0;
		itd = tempSeq.diff.begin();
		while ((c < MAX_DATABASE_SIZE) && (itd != tempSeq.diff.end())) {	// itd zeigt auf CssDatabaseResultItem (distance ,dbRef)
			
			CssDifference tmpDiff = (*itd);
			int tmpObjectClassId = data[tmpDiff.dbRef].objectTypeNr;

			if (tmpDiff.distance == INT_MAX) {
				result[tmpObjectClassId].notFoundAll++;
				if (c <= 10) {
					result[tmpObjectClassId].notFound10++;
				} // if
			} else {				
				
				// save top matches in object class
				if (c==0) {
					result[tmpObjectClassId].topMatch++;
					result[tmpObjectClassId].topValuePercent += 1.0/((ieee64)tmpDiff.distance);
					if (tmpDiff.distance > 0) {
						totalValue += 1.0/((ieee64)tmpDiff.distance);
					} // if
					avgSum += tmpDiff.distance;
					avgCounter++;
				} // if
				if (c<=5) {
					result[tmpObjectClassId].top5Match++;
				} // if
				if (c<=10) {
					result[tmpObjectClassId].top10Match++;
				} // if
				if (c<=20) {
					result[tmpObjectClassId].top20Match++;
				} // if
				switch (c) {
					case 0: 
						result[tmpObjectClassId].f1 += 10;
						break;
					case 1: 
						result[tmpObjectClassId].f1 += 6;
						break;
					case 2: 
						result[tmpObjectClassId].f1 += 5;
						break;
					case 3: 
						result[tmpObjectClassId].f1 += 4;
						break;
					case 4: 
						result[tmpObjectClassId].f1 += 3;
						break;
					case 5: 
						result[tmpObjectClassId].f1 += 2;
						break;
					case 6: 
						result[tmpObjectClassId].f1 += 1;
						break;
				} // switch
				result[tmpObjectClassId].foundAll++;
				if (c<=10) {
					result[tmpObjectClassId].found10++;
				} // if
			} // else
			itd++;
			c++;

		} // while
	} // for
	
    for (i=0; i<objectClassSize; i++) {
		if (result[i].topMatch != 0) {

            if (searchString == objectClass[i]) {
                goodMatches +=  result[i].topMatch;
            } else {
                badMatches += result[i].topMatch;
            } // else
        } // if
	} // for

    missedMatches = sequenceSize - badMatches - goodMatches;
    goodMatches   = (100*goodMatches) / sequenceSize;
    badMatches    = (100*badMatches) / sequenceSize;
    missedMatches = (100*missedMatches) / sequenceSize;
    
    cout << "       ";
    if (goodMatches < 10) {
        cout << " ";
    } // if
    if (goodMatches < 100) {
        cout << " ";
    } // if
    cout << goodMatches << "     _      ";
    if (badMatches < 10) {
        cout << " ";
    } // if
    if (badMatches < 100) {
        cout << " ";
    } // if
    cout << badMatches << "     _      ";
    if (missedMatches < 10) {
        cout << " ";
    } // if
    if (missedMatches < 100) {
        cout << " ";
    } // if
    cout << missedMatches << "     _" << endl;
    
    return ;
} // CssDisplayResult::show


