/*
   This file is part of the MoCA video library.
   Name: CssMaximaFeature.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: CssMaximaFeature.h,v 1.1 2004/02/17 11:52:58 kopf Exp $

*/

#ifndef CSS_MAXIMA_FEATURE_H
#define CSS_MAXIMA_FEATURE_H

#include <vector>

//#include <OTagStream.h>
#include <ITagStream.h>
#include <Image.h>
#include <ImageSize.h>

const int32 DEFAULT_CSS_PEAK_NUMBER = 10;

struct CssMaximumItem
{
	CssMaximumItem ()
	{
		position   = 0;
		height     = 0;
		width      = 0;
		curveStart = 0;
		curveEnd   = 0;
		isConcave  = false;
		visited    = false;
	}

    int16 position;             // x-position of the maximum
    int16 height;              // height of the maximum
    int16 width;               // width of the curve (at the bottom)
    int16 curveStart;           // start position of the curve the maximum belongs to
    int16 curveEnd;             // end position of the curve the maximum belongs to
	bool isConcave;			    // identifies concave and convex parts of the contour

    bool visited;               // for matching, entry already handled?

    inline bool operator== (const CssMaximumItem&) const;

    bool operator< (const CssMaximumItem& cm) const
    {
        return height < cm.height;
    } // operator<
};

struct CssMaximumData
{
    CssMaximumData () {
		xyRatio = 1.0;
		data.clear();
	}
	ieee64 xyRatio;
	vector<CssMaximumItem> data;


	
	void inverseSort() 
	{
		// copy to list
		list<CssMaximumItem> tmp;
		for (int32 i=0; i<data.size(); i++) {
			tmp.push_back(data[i]);
		}

		// sort
	    tmp.sort();
		tmp.reverse();

		// copy to vector
		list<CssMaximumItem>::iterator it = tmp.begin();
		for (int32 i=0; i<data.size(); i++) {
			data[i] = (*it);
			it++;
		}
	}
};


inline bool
CssMaximumItem::operator== (const CssMaximumItem& m) const
{
    if ((position   == m.position  ) &&
		(height     == m.height    ) &&
		(width      == m.width     ) &&
		(curveStart == m.curveStart) &&
		(curveEnd   == m.curveEnd  ) &&
		(isConcave  == m.isConcave )) {
		return true;
	}
	return false;
} // operator ==


OTagStream& operator<< (OTagStream&, const CssMaximumItem&);
ITagStream& operator>> (ITagStream&, CssMaximumItem&);

OTagStream& operator<< (OTagStream&, const CssMaximumData&);
ITagStream& operator>> (ITagStream&, CssMaximumData&);


class CssMaximaFeature
{
public:
    CssMaximaFeature () {}
    void process (const Image<uint8>&, CssMaximumData&, vector<ImagePosition> &contourPixel);
    void processBottomWidth (const Image<uint8>&, CssMaximumData&);
    void processMaxWidth (const Image<uint8>&, CssMaximumData&);
private:
    // CSS maxima are number from 1 to 254
    // 0 is not used, 255 is reserved for background pixels
    enum { MAX_NUMBER_CSS_MAXIMA = 255, BACKGROUND = 255 };

    void findMaxima (const Image<uint8>&, CssMaximumItem[]);
    void traceCurve (const Image<uint8>&, CssMaximumItem&);
    void traceCurveMaxWidth (const Image<uint8>&, CssMaximumItem&);
};


#endif

