/*
   This file is part of the MoCA video library.
   Name: CssVideoAnalysis.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: CssVideoAnalysis.h,v 1.1 2004/02/17 11:52:58 kopf Exp $

*/

#ifndef CSS_VIDEO_ANALYSIS_H
#define CSS_VIDEO_ANALYSIS_H

#include <list>

#include <OTagStream.h>
#include <ITagStream.h>
#include <Image.h>
#include <ImageSize.h>
#include <WriteMocaData.h>
#include <ReadMocaData.h>
#include <ObjectSegmentationFeature.h>
#include <CssImage.h>
#include <CssMaximaFeature.h>
#include <DifferenceFilter.h>


static const string CSS_VIDEO_ANALYSIS_VERSION  = "Version-0.60";
static const string CSS_VIDEO_ANALYSIS_FILENAME = "cssvideoanalysis.idx";


struct CssVideoAnalysisPeak
{
	CssVideoAnalysisPeak ()
	{
		clear();
	}

    bool operator== (const CssVideoAnalysisPeak& m) const
	{
		if ((position   == m.position  ) &&
			(height     == m.height    ) &&
			(width      == m.width     ) &&
			(isConcave  == m.isConcave )) {
			return true;
		}
		return false;
	} // operator ==


    bool operator< (const CssVideoAnalysisPeak& cm) const
    {
        return height < cm.height;
    } // operator<

	void clear()
	{
		position  = 0;   
		height    = 0;   
		width     = 0;   
		isConcave = true;
		visited   = false;
	}
    
	int16 position;             // x-position of the maximum
    int16 height;               // height of the maximum
    int16 width;                // width of the curve (at the bottom)
	bool isConcave;			    // identifies concave and convex parts of the contour
    bool visited;               // for matching, entry already handled?
};


// data for one frame
const int CSS_MAX_PEAK_NUMBERS = 10;
struct CssVideoAnalysisFrame
{
    CssVideoAnalysisFrame () {
		frameNo = 0;
		xyRatio = 1.0;
	}
	int32 frameNo;
	ieee64 xyRatio;
	CssVideoAnalysisPeak peak[CSS_MAX_PEAK_NUMBERS];
};


// data with all frames of the video
struct CssVideoAnalysisData
{
    CssVideoAnalysisData () {
		version  = CSS_VIDEO_ANALYSIS_VERSION;
		filename = "";
		numberOfFrames = 0;
	}
	string version;
	string filename;
	int32 numberOfFrames;
	list<CssVideoAnalysisFrame> frameData;
};



OTagStream& operator<< (OTagStream&, const CssVideoAnalysisPeak&);
ITagStream& operator>> (ITagStream&, CssVideoAnalysisPeak&);

OTagStream& operator<< (OTagStream&, const CssVideoAnalysisFrame&);
ITagStream& operator>> (ITagStream&, CssVideoAnalysisFrame&);

OTagStream& operator<< (OTagStream&, const CssVideoAnalysisData&);
ITagStream& operator>> (ITagStream&, CssVideoAnalysisData&);


class CssVideoAnalysis
{
public:
    CssVideoAnalysis (string &filename, int16 cropLeft, int16 cropRight, int16 cropTop, int16 cropBottom) 
	{
		pathFilename = filename;
		this->cropLeft   = cropLeft;
		this->cropRight  = cropRight;
		this->cropTop    = cropTop;
		this->cropBottom = cropBottom;
	}
    void process (CssVideoAnalysisData &data);

private:
	void readValues (CssVideoAnalysisData &data);
	void calculateCssData (CssVideoAnalysisData &data);

	string pathFilename;
	int16 cropLeft;
	int16 cropRight;
	int16 cropTop;
	int16 cropBottom;
};


#endif

