/*
   This file is part of the MoCA video library.
   Name: EdgeBasedContrastFeature.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef EDGE_BASED_CONTRAST_FEATURE_H
#define EDGE_BASED_CONTRAST_FEATURE_H

// MoCA includes
#include <MocaTypes.h>
#include <Image.h>
#include <VideoDecoder.h>
#include <ITagStream.h>
#include <MocaSignal.h>

// C++ includes
#include <iostream>
using namespace std;


const static string EDGE_BASED_CONTRAST_VERSION  ="Version-0.1";
const static string EDGE_BASED_CONTRAST_FILENAME ="ebc.idx";

/*
	Constructor: initializes variables
	
	process(): calculate the edge based contrast
	read()   : read values from "Ebc"-file
  */

struct EbcData
{
	EbcData() {
		version         = EDGE_BASED_CONTRAST_VERSION;
		filename        = "";
		frames          = 0;
		data.clear();
		thLow  = 50;
		thHigh = 100;
	} 

	string version;
	string filename;
	int32 frames;
	uint8 thLow;
	uint8 thHigh;
	vector<ieee64> data;
};

std::ostream& operator<< (ostream&,    const EbcData&);
OTagStream&   operator<< (OTagStream&, const EbcData&);
ITagStream&   operator>> (ITagStream&, EbcData&);



class EbcFeature
{
public:

	EbcFeature(const string &pathFilename, uint8 thLow=50, uint8 thHigh=100)
	{
		this->pathFilename = pathFilename;
		this->thLow  = thLow;
		this->thHigh = thHigh;
	}
	~EbcFeature() {}

	// analyse full video
	void process (EbcData &data);

private:
	void readValues (EbcData &data);

	uint8 thLow;
	uint8 thHigh;
	string pathFilename;
};


#endif
