/*
   This file is part of the MoCA video library.
   Name: FaceRecognition.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <FaceRecognition.h>


// *****************************************
// Class: FaceNormalization 
// *****************************************

void 
FaceNormalization ::process (const Image<uint8> &src, Image<uint8> &dest)
{
	


	return;
}





// *****************************************
// Class: AverageFace
// *****************************************

std::ostream& operator<<(ostream &os, const AverageFace &af)	{
	os << af.imageSize[DIM_X] << endl;
	os << af.imageSize[DIM_Y] << endl;

	for(int i=0; i<af.imageSize[DIM_X]*af.imageSize[DIM_Y]; i++)	{
		os << gsl_vector_get(af.averageFace, i) << endl;
	}

	return os;
} // operator<<


std::istream& operator>>(istream &is, AverageFace &af)	{
	is >> af.xDim;
	is >> af.yDim;

	af.averageFace = gsl_vector_alloc(af.xDim*af.yDim);
	double value = 0;

	for(int i=0; i<af.xDim*af.yDim; i++)	{
		is >> value;
		gsl_vector_set(af.averageFace, i, value);
	}

	return is;
} // operator>>


// *****************************************
// Class: AvgFaceCalculation
// *****************************************

void AvgFaceCalculation::calculateAverageImage(const string &srcDir)	{
	// initialize imaall needed values once
	if(true)	{
	    vector<FileInfo> pgmFiles;
	    FileList files(srcDir, true);
    	files.process("*.pgm", pgmFiles);
		vector<FileInfo>::iterator it;
		
		// if no images in srcDir
		if(pgmFiles.size()==0)	{
			if(stop==0)	{
				cerr << endl << "ERROR!" << endl << "No .pgm images found." << endl;
				cerr << "Trying to convert all supported image files into .pgm format ..." << endl << endl;
					
				// convert all supported non-pgm images into pgm images
				convertImages(srcDir, true);
		
				stop++;

				// retry
				calculateAverageImage(srcDir);
			}
			else	{
				cerr << endl << "ERROR!" << endl << "No supported image format found at all." << endl;
				exit (-1);
			}
		} // if(pfmFiles.size()==0)
		
		// vector containing the different imageSizes of the images in the directory
		vector<SizeOccurrence> sov;
		bool firstElement = true;
		
		// read all image sizes and write size occurrence in vector<SizeOccurrence>
		for (it = pgmFiles.begin(); it != pgmFiles.end(); it++) {
			Image<uint8> src;
			ReadImage((*it).name, src);
			SizeOccurrence so(src.getSize());

			// insert first element into empty vector
			if(firstElement)	{
				sov.push_back(so);
				firstElement = false;
			}
			else	{
				/* elements are compared with vector elements 
			 	 * prior to pushing them to the end of the vector
			 	 */
   	 			bool found = false;
   	 			   	 			
    			for(int i=0; i<sov.size(); i++)	{
    				if(so==sov[i])	{
    					sov[i].amount += 1;
    					found = true;
       					break;
       				} // if
    			} // for
   	 			if(!found)	{
   	 				sov.push_back(so);
				}
			} // else
		} // for(it)
		
		// get greatest value and corresponding image size;
		int amount = 0;
		for(int i=0; i<sov.size(); i++)	{
			int buf = sov[i].amount;

			if(buf>amount)	{
				amount = buf;
				imageSize = sov[i].imageSize;
			}
		} // for
		
		// allocate memory
		averageFace = gsl_vector_alloc(imageSize[DIM_X]*imageSize[DIM_Y]);
		
		/* set all values to zero, because there's reading access
		 * to the vector and if it is not initialized with zeros it
		 * is initialized with random values
		 */
		gsl_vector_set_zero(averageFace);
		
		/* convert all supported non-pgm images into pgm 
		 * images of same size
		 * if ... else should not be necessary; image size
		 * should be set until now or program should have
		 * been terminated
		 */
		if(imageSize[DIM_X]==0 && imageSize[DIM_Y]==0)
			convertImages(srcDir, true);
		else
			convertImages(srcDir, false);
	}// if(true)

	vector<FileInfo> pgmFiles;
	FileList files(srcDir, true);
    files.process("*.pgm", pgmFiles);
	vector<FileInfo>::iterator it;
	Image<uint8> src;

	// sum all images
	for (it = pgmFiles.begin(); it != pgmFiles.end(); it++) {
		ReadImage((*it).name, src);
		
		// correct lighting
		LightingCorrectionFilter<uint8> lcf;
		lcf.process(src);

		// sum all images into gsl_vector 'averageFace'
		for(int y=0; y<imageSize[DIM_Y]; y++)	{
			for(int x=0; x<imageSize[DIM_X]; x++)	{
				gsl_vector_set(averageFace, y*imageSize[DIM_X]+x, (double)gsl_vector_get(averageFace, y*imageSize[DIM_X]+x) + (double)src(x, y, 0));
			}
		}
    }

	// divide summed image by amount of images
	for(int i=0; i<imageSize[DIM_X]*imageSize[DIM_Y]; i++)	{
		gsl_vector_set(averageFace, i, gsl_vector_get(averageFace, i)/pgmFiles.size());
	}
} // calculateAverageImage


void AvgFaceCalculation::convertImages(const string &srcDir,  bool getSize)	{
	Image<uint8> src;
	
	/* convert all non-pgm images into pgm images;
	 * extendable with other image types but at
	 * the moment <Image.h> only suports
	 * ".jpg", ".png", ".pnm", ".pgm";
	 * change numberOfExtensions if new file extension
	 * is supported; add new extension to array
	 */
	const int numberOfExtensions = 4;
	string extensions[numberOfExtensions] = {"*.jpg", "*.png", "*.ppm", ".pgm"};

	for(int i=0; i<numberOfExtensions; i++)	{
		// read all 'extensions[i]' files in directory
    	vector<FileInfo> imageFiles;
    	FileList files(srcDir, true);
    	files.process(extensions[i], imageFiles);
		vector<FileInfo>::iterator it;

		// if image size not yet initialized get size of first image as standard
    	for (it = imageFiles.begin(); it != imageFiles.end(); it++) {
    		if(getSize)	{
    			ReadImage((*it).name, src);
    			imageSize = src.getSize();
    			getSize = false;
    		}
    		
    		// read every image
   			ReadImage((*it).name, src);
    		
    		// get complete file name
    		int position = (*it).name.rfind("/", (*it).name.length());
			string fileName = (*it).name.substr(position+1);
			
			// get file extension
			position = (*it).name.rfind(".", (*it).name.length());
			string extension = (*it).name.substr(position+1);

			// all letters to upper case
			for (uint8 k=0; k<extension.size(); k++)
				extension[k]=toupper(extension[k]);
				
			// if already gray scale imagecout
			if(extension == "PGM")	{
				ImageSize is = src.getSize();

				// if wrong size resize image
				if(is[DIM_X]!=imageSize[DIM_X] || is[DIM_Y]!=imageSize[DIM_Y])	{
					ScaleFilter<uint8> sf(imageSize);
					sf.process(src);
				}
			} // if(PGM)

			// if not a gray scale image
			else	{
				// convert color image to grayscale image
				GrayscaleFilter<uint8> gf;
				gf.process(src);		

				ImageSize is = src.getSize();

				// if wrong size resize image
				if(is[DIM_X]!=imageSize[DIM_X] || is[DIM_Y]!=imageSize[DIM_Y])	{
					ScaleFilter<uint8> sf(imageSize);
					sf.process(src);
				}
			} // else(no PGM)
			
			// create path and file name with pgm ending
			position = fileName.rfind(".", fileName.length());
			fileName = fileName.replace(position+1, fileName.length(), "pgm");
			string path = srcDir + fileName;

			// write pgm image
			WriteImage(src, path);	
		}
	}
} // convertImages


void AvgFaceCalculation::writeAverageImageVector(string destDir)	{
	AverageFace af(imageSize, averageFace);

	destDir += "avgFace.txt";

	ofstream ofs(destDir.c_str(), std::ios::out | std::ios::binary);
	if(!ofs)	{
		cerr << "\nError while opening file " << destDir << "\n\n";
		exit (-1);
	}
	else	{
		ofs << af;
		ofs.close();
	}
} // writeAverageImage


void AvgFaceCalculation::writeAverageImage(const string &destDir)	{
	// initialize destination image
	Image<uint8> dest;
	dest.resize(imageSize);

	// copy values from vector to image
	for(int y=0; y<imageSize[DIM_Y]; y++)	{
		for(int x=0; x<imageSize[DIM_X]; x++)	{
			dest(x, y, 0) = (uint8) gsl_vector_get(averageFace, (y*imageSize[DIM_X]+x));
		}
	}

 	// write average image to file
	WriteImage(dest, destDir + "avgFace.pgm");
} // writeAverageImage



// *****************************************
// Class: SizeOccurrence
// *****************************************
bool operator== (const SizeOccurrence &o1, const SizeOccurrence &o2)	{
		return(o1.imageSize[DIM_X] == o2.imageSize[DIM_X] && o1.imageSize[DIM_Y] == o2.imageSize[DIM_Y]);
} // operator==



// *****************************************
// Class: DifferenceImagesMatrixCalculation
// *****************************************
void DifferenceImagesMatrixCalculation::calculateMatrix(const string &pathToSourceImages, const gsl_vector *averageFace)	{
	// read all pgm files in directory
   	vector<FileInfo> pgmFiles;
   	FileList files(pathToSourceImages, true);
   	files.process("*.pgm", pgmFiles);
	vector<FileInfo>::iterator it;

	// counter for position of gsl_vectors in allDifferenceImages
	int position = 0;

	// for initialisation of values
	bool init = true;

	// local image container
	Image<uint8> src;

	// calculate for every image
	for(it = pgmFiles.begin(); it != pgmFiles.end(); it++)	{
		// read first image
		ReadImage((*it).name, src);
		
		// correct lighting
		LightingCorrectionFilter<uint8> lcf;
		lcf.process(src);
	
		// initialization and memory allocation
		if(init)	{
			init = false;

			// create new gsl_vector object 'allDifferenceImages'
			allDifferenceImages = new gsl_vector*[pgmFiles.size()];

			// get the dimensions of the image
			imageSize = src.getSize();
			
			// get amount of files which is equal to the size of the M*M matrix
			differenceImagesMatrixSize = pgmFiles.size();

			// allocate memory for the M*M matrix
			matrix = gsl_matrix_alloc(differenceImagesMatrixSize, differenceImagesMatrixSize);
		}

		// allocate memory for each field in the gsl_vector 'allDifferenceImages'
		allDifferenceImages[position] = gsl_vector_alloc(imageSize[DIM_X]*imageSize[DIM_Y]);

		// calculate difference image
		for(int y=0; y<imageSize[DIM_Y]; y++)	{
			for(int x=0; x<imageSize[DIM_X]; x++)	{
   				gsl_vector_set(allDifferenceImages[position], y*imageSize[DIM_X]+x, (double)src(x, y, 0)-(double)gsl_vector_get(averageFace, y*imageSize[DIM_X]+x));
			}
		}

		// increase position in allDifferenceImages
		position++;
	}

	// set the values of the M*M matrix
	for(int y=0; y<differenceImagesMatrixSize; y++)	{
		for(int x=0; x<differenceImagesMatrixSize; x++)	{
			// invoke method which calculates values
			gsl_matrix_set(matrix, y, x, calculateValues(y, x));
		}
	}
} // calculateMatrix


double DifferenceImagesMatrixCalculation::calculateValues(const int &y, const int &x)	{
	double value = 0;

	// vector multiplication
	for(int pos=0; pos<imageSize[DIM_Y]*imageSize[DIM_X]; pos++)	{
		value += (double)gsl_vector_get(allDifferenceImages[x], pos) * (double)gsl_vector_get(allDifferenceImages[y], pos);
	}

	return value;
} // calculateValues



// *****************************************
// Class: 
// *****************************************

void EigenVectorCalculation::calculateEigenValuesAndVectors(gsl_matrix *matrix)	{
	// local because not needed for further calculation
	gsl_vector *eigenValues = NULL;
	gsl_eigen_symmv_workspace *workspace = NULL;

	// allocate memory
	workspace = gsl_eigen_symmv_alloc(matrixSize);
	eigenVectors = gsl_matrix_alloc(matrixSize, matrixSize);
	eigenValues= gsl_vector_alloc(matrixSize);
	result = gsl_matrix_alloc(lengthOfImageVector, matrixSize);

	/* calculate eigenvalues and eigenvectors;
	 * eigenvectors are stored in columns of eigenVectors
	 */
	gsl_eigen_symmv(matrix, eigenValues, eigenVectors, workspace);

	// sort eigenvalues and eigenvectors
	gsl_eigen_symmv_sort (eigenValues, eigenVectors,
                        GSL_EIGEN_SORT_VAL_DESC);

	// free memory
	if(eigenValues != NULL)
		gsl_vector_free(eigenValues);
	if(workspace != NULL)
		gsl_eigen_symmv_free(workspace);
} // calculate


void EigenVectorCalculation::multiplyWithDifferenceImages()	{
	for(int y=0; y<lengthOfImageVector; y++)	{
  		for(int x=0; x<matrixSize; x++)	{
			gsl_matrix_set(result, y, x, calculateValue(y, x));
		}
	}
} // multiply


double EigenVectorCalculation::calculateValue(const int yCoordinate, const int xCoordinate)	{
	double value = 0;

	// vector of difference images multiplied with eigenvector
	for(int x=0; x<matrixSize; x++)	{
		value += gsl_vector_get(allDifferenceImages[x], yCoordinate)*gsl_matrix_get(eigenVectors, x, xCoordinate);
	}

	return value;
} // calculateValues


void EigenVectorCalculation::normalizeEigenVectors()	{
	double value = 0;
	// for every eigenvector
	for(int x=0; x<matrixSize; x++)	{
		value = 0;
		// calucate sum of squared values
		for(int y=0; y<lengthOfImageVector; y++)	{
			value += pow(gsl_matrix_get(result, y, x), 2);
		}

		/* prevents dividing by zero which means 
		 * that the corresponding eigenvector is zero
		 */
		if(value==0)
			break;
		else
			value = sqrt(value);

		// divide every value of eigenvector by 'value'
		for(int y=0; y<lengthOfImageVector; y++) {
			gsl_matrix_set(result, y, x, gsl_matrix_get(result, y, x)/value);
		}
	}
} // normalizeEigenVectors


void EigenVectorCalculation::writeEigenVectors(string output)	{
	EigenVectors ev(matrixSize-1, lengthOfImageVector, result);

	output += "eigenvectors.txt";
	ofstream ofs(output.c_str(), std::ios::out | std::ios::binary);

	if(!ofs)	{
		cerr << "\nError while opening file " << output << "\n\n";
		exit (-1);
	}
	else	{
		ofs << ev;
		ofs.close();
	}
} // writeEigenVectors


void EigenVectorCalculation::writeFaces(const ImageSize imageSize, const string &destDir)	{
	// adjust image size
	Image <uint8> eigenFace;
	eigenFace.resize(imageSize);
	ostringstream os;

	// for every eigenface [(M-1) times] with M-1 meaningful eigenvectors
	for(int i=0; i<matrixSize-1; i++)	{
		// create filenames
		os.str("");
		os << "eigenface" << i;

		// write values from gsl_vector into image
		for(int y=0; y<imageSize[DIM_Y]; y++)	{
			for(int x=0; x<imageSize[DIM_X]; x++)	{
				double value = gsl_matrix_get(result, y*imageSize[DIM_X]+x, i);
										
				if(value<0)
					value = 0;
				if(value>255)
					value = 255;

				eigenFace(x, y, 0) = (uint8) value;
			}
		}
		WriteImage(eigenFace, destDir + os.str() + ".pgm");
	}
} // writeFaces


// *****************************************
// Class: EigenVectors
// *****************************************

std::ostream& operator<<(ostream &os, const EigenVectors &ev)	{
	os << ev.numberOfEigenVectors << endl;
	os << ev.lengthOfEigenVectors << endl;

	for(int y=0; y<ev.lengthOfEigenVectors; y++)	{
		for(int x=0; x<ev.numberOfEigenVectors; x++)	{
			os << gsl_matrix_get(ev.eigenVectors, y, x) << " ";
		}
		os << endl;
	}

	return os;
} // operator<<


std::istream& operator>>(istream &is, EigenVectors &ev)	{
	is >> ev.numberOfEigenVectors;
	is >> ev.lengthOfEigenVectors;
	
	if(ev.numberOfEigenVectors<1)	{
		cerr << "Not enough eigenvectors for calculation." << endl;
		exit(0);
	}

	// allocate memory
	ev.eigenVectors = gsl_matrix_alloc(ev.lengthOfEigenVectors, ev.numberOfEigenVectors);

	// ready values and write them to the eigenVectors
	double value = 0;
	for(int y=0; y<ev.lengthOfEigenVectors; y++)	{
		for(int x=0; x<ev.numberOfEigenVectors; x++)	{
			is >> value;
			gsl_matrix_set(ev.eigenVectors, y, x, value);
		}
	}

	return is;
} // operator>>


// *****************************************
// Class: FaceWeightsCalculation
// *****************************************

void FaceWeightsCalculation::calculateDifference(const gsl_vector *averageFace)	{
	ImageSize is = src.getSize();
	difference = gsl_vector_alloc(is[DIM_X]*is[DIM_Y]);

	for(int y=0; y<is[DIM_Y]; y++)	{
		for(int x=0; x<is[DIM_X]; x++)	{
			gsl_vector_set(difference, y*is[DIM_X]+x, (double)src(x, y, 0) - (double)gsl_vector_get(averageFace, y*is[DIM_X]+x));
		}
	}
} // calculateDifference


void FaceWeightsCalculation::calculateWeights(const int &lengthOfImageVector, const int &numberOfEigenVectors, const gsl_matrix *eigenVectors)	{
	double value = 0;
	
	// initialisation of array holding the weights
	weights = new double[numberOfEigenVectors];

	// calculate weights corresponding to all eigenvectors
	for(int position=0; position<numberOfEigenVectors; position++)	{
		value = 0;
		weights[position]=0;
		
		/* calculate weight; eigenvectors stored 
		 * in the columns of eigenVectors
		 */
		for(int l=0; l<lengthOfImageVector; l++)	{
			value += gsl_matrix_get(eigenVectors, l, position) * gsl_vector_get(difference, l);
		}

		// write weight into array
		weights[position]=value;
	}
} // calculateWeights


void FaceWeightsCalculation::writeWeights(string fileName, string path, const int &numberOfEigenVectors)	{
	int position = fileName.find_last_of("/", fileName.length());
	fileName     = fileName.substr(position+1);
 	Weights wgt(fileName, numberOfEigenVectors, weights);

	// crop extension, replace it with 'wgt' and copy string to char array
	position = fileName.find_last_of(".", fileName.length());
	fileName.replace(position+1, fileName.length()-position, "wgt" );
	path += fileName;

	ofstream ofs(path.c_str(), std::ios::out | std::ios::binary);
	if(!ofs)	{
		cerr << "\nError while opening file " << path.c_str() << "\n\n";
		exit (-1);
	}
	else	{
		ofs << wgt;
		ofs.close();
	}
}  // writeWeights


// *****************************************
// Class: 
// *****************************************

std::ostream& operator<<(ostream &os, const Weights &w)	{
	os << w.fileName << endl;
	os << w.numberOfWeights << endl;

	for(int i=0; i<w.numberOfWeights; i++)	{
		os << w.weights[i] << endl;
	}

	return os;
} // operator<<

std::istream& operator>>(istream &is, Weights &w)	{
	is >> w.fileName;
	is >> w.numberOfWeights;

	w.weights = new double[w.numberOfWeights];

	for(int i=0; i< w.numberOfWeights; i++)	{
		is >> w.weights[i];
	}

	return is;
} // operator>>


// *****************************************
// Class: Threshold
// *****************************************


std::ostream& operator<<(ostream &os, const Threshold &t)	{
	os << t.threshold << endl;

	return os;
} // operator<<


std::istream& operator>>(istream &is, Threshold &t)	{
	is >> t.threshold;

	return is;
} // operator>>


// *****************************************
// Class: ThresholdCalculation
// *****************************************


void ThresholdCalculation::calculateThreshold(const string &srcDir)	{
	// read all pgm files in directory
    vector<FileInfo> wgtFiles;
	vector<FileInfo> wgtFiles2;

    FileList files(srcDir, true);
	FileList files2(srcDir, true);

    files.process("*.wgt", wgtFiles);
	files2.process("*.wgt", wgtFiles2);

	vector<FileInfo>::iterator it;
	vector<FileInfo>::iterator it2;

	int counter1 = 0;
	int counter2 = 0;

	threshold = 0;

	// if no weights in srcDir
	if(wgtFiles.size()==0)	{
		cerr << endl << "ERROR!" << endl << "No weights found.\n\n";
		exit (-1);
	}

	// process all wgt files in srcDir
    	for (it = wgtFiles.begin(); it != wgtFiles.end(); it++) {
  		ifstream ifs(((*it).name).c_str(), std::ios::in | std::ios::binary);
		if(!ifs)	{
			cerr << "\nError while opening file " << (*it).name << "\n\n";
			exit (-1);
		}
		else	{
			ifs >> wgt;
			ifs.close();
		}

		// compare with every wgt file
		for (it2 = wgtFiles2.begin(); it2 != wgtFiles2.end(); ++it2) {
			// don't calculate for same file
			if((*it).name==(*it2).name || counter2<=counter1)	{
				counter2++;
				continue;
			}

			ifstream ifs(((*it2).name).c_str(), std::ios::in | std::ios::binary);
			if(!ifs)	{
				cerr << "\nError while opening file " << (*it2).name << "\n\n";
				exit (-1);
			}
			else	{
				ifs >> wgt2;
				ifs.close();
			}

			// invoke method to calculate distance
			double buf= calculate();

			// compare distance with greatest distance at moment
			if(buf>threshold)
				threshold = buf;

			counter2++;
		}
		counter2=0;
		counter1++;
	}
} // calcluateThreshold


double ThresholdCalculation::calculate()	{
	double tsh = 0;
	for(int i=0; i<wgt.numberOfWeights; i++)	{
		tsh += pow((wgt.weights[i]-wgt2.weights[i]), 2);
	}

	return 0.5*sqrt(tsh);
} // calculate


// write threshold
void ThresholdCalculation::writeThreshold(const string &destDir)	{
	Threshold t(threshold);

	string fileName = destDir + "threshold.tsh";
	ofstream ofs(fileName.c_str(), std::ios::out | std::ios::binary);

	if(!ofs)	{
		cerr << "\nError while opening file " << fileName << "\n\n";
		exit (-1);
	}
	else	{
		ofs << t;
		ofs.close();
	}
} // writeThreshold


// *****************************************
// Class: 
// *****************************************

void FaceClassesCalculation::calculateClasses(const string &srcDir, const string &pathToWeights)	{
	// read all pgm files in directory
    vector<FileInfo> pgmFiles;
    FileList files(srcDir, true);
    files.process("*.pgm", pgmFiles);
	vector<FileInfo>::iterator it;

	// process all pgm files in srcDir
   	for (it = pgmFiles.begin(); it != pgmFiles.end(); it++) {
    	FaceWeightsCalculation fwc((*it).name, pathToWeights, lengthOfImageVector, numberOfEigenVectors, averageFace, eigenVectors);
	}
} // calculateClasses


void FaceClassesCalculation::readAverageFace()	{
	AverageFace af;
	string pathToAverageFace = dataPath + "avgFace.txt";

	ifstream ifs(pathToAverageFace.c_str(), std::ios::in | std::ios::binary);

	if(!ifs) {
		cerr << "\nError while opening file " << pathToAverageFace << "\n\n";
		exit (-1);
	}
	else	{
		ifs >> af;
		ifs.close();
	}

	xDim = af.xDim;
	yDim = af.yDim;
	averageFace = gsl_vector_alloc(xDim*yDim);

	gsl_vector_memcpy(averageFace, af.averageFace);
} // readAverageFace


void FaceClassesCalculation::readEigenVectors()	{
	EigenVectors ev;
	string pathToEigenVectors = dataPath + "eigenvectors.txt";
	
	ifstream ifs(pathToEigenVectors.c_str(), std::ios::in | std::ios::binary);
	if(!ifs)	{
		cerr << "\nError while opening file " << pathToEigenVectors << "\n\n";
		exit (-1);
	}
	else	{
		ifs >> ev;
		ifs.close();
	}
	
	// initialize values from file
	lengthOfImageVector = ev.lengthOfEigenVectors;
	numberOfEigenVectors = ev.numberOfEigenVectors;
			
	if(numberOfEigenVectors==0)	{
			cerr << "Not enough eigenvectors for calculation." << endl;
			exit(0);
	}
	eigenVectors = gsl_matrix_alloc(lengthOfImageVector, numberOfEigenVectors);

	// copy matrix
	gsl_matrix_memcpy(eigenVectors, ev.eigenVectors);
} // readEigenVectors


// *****************************************
// Class: 
// *****************************************

// *****************************************
// Class: 
// *****************************************
