/*
   This file is part of the MoCA video library.
   Name: LinearEstimatorFeature.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*
   LTS estimator
   --------------------------
   (c) 2002 Dirk Farin
   Dept. Computer Science IV
   University of Mannheim
 */

#ifndef LINEAR_ESTIMATOR_FEATURE_LTS_HH
#define LINEAR_ESTIMATOR_FEATURE_LTS_HH


// GSL includes
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_errno.h>

#include<iostream>
#include<vector>
using namespace std;

// MoCA includes
#include <MocaTypes.h>

class LinearEstimatorFeature
{
public:
	LinearEstimatorFeature();
	
	void SetNHTries(int n) 
	{ 
		d_htries = n; 
	}
	
	void SetOutlierInfo(vector<bool>* outlier) 
	{ 
		d_outlier = outlier; 
	}

	void SetResidualInfo(vector<double>* residual) 
	{ 
		d_residual = residual; 
	}
	
	void Estimate(vector<double>& parameters, double **data, int32 dataWidth, int32 dataHeight);
	
private:
	int d_htries;
	int d_csteps;
	double d_trim_fact;
	vector<bool>* d_outlier;
	vector<double>* d_residual;
};

#endif

