/*
   This file is part of the MoCA video library.
   Name: ObjectSegmentationFeature.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef OBJECT_EXTRACTION_FEATURE_H
#define OBJECT_EXTRACTION_FEATURE_H

// C++ includes
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <IYuvVideo.h>
#include <OYuvVideo.h>
#include <ContourParametrization.h>
#include <WriteMocaData.h>
#include <ReadMocaData.h>
#include <ObjectMaskFeature.h>
#include <RegionFeature.h>
#include <ContourEdgeList.h>
#include <DrawDot.h>
#include <ErosionFilter.h>
#include <InverseFilter.h>
#include <DifferenceFilter.h>



struct ObjectSegmentationContour
{
	ObjectSegmentationContour()
	{
		SignalSize size (ContourParametrization::DEFAULT_CONTOUR_SIZE, 2);
		contourParameter.realloc (size, SignalSize(0), BORDER_PERIODIC);
		for (int32 i=0; i<size[DIM_X]; i++) {
			contourParameter(i,0) = 0;
			contourParameter(i,1) = 0;
		} // for
	}

	Signal<int16> contourParameter;
	int32 frame;
};


static const string OBJECT_SEGMENTATION_VERSION  = "Version-0.50";
static const string OBJECT_SEGMENTATION_FILENAME = "objectsegmentation.idx";

struct ObjectSegmentationData
{
	ObjectSegmentationData ()
	{
		version = OBJECT_SEGMENTATION_VERSION;
		filename = "UNKNOWN";
		numberOfFrames = 0;
	}
	string version;
	string filename;
	int32 numberOfFrames;
	list<ObjectSegmentationContour> data;
};


std::ostream& operator<< (ostream&,    const ObjectSegmentationContour&);
OTagStream&   operator<< (OTagStream&, const ObjectSegmentationContour&);
ITagStream&   operator>> (ITagStream&, ObjectSegmentationContour&);
std::ostream& operator<< (ostream&,    const ObjectSegmentationData&);
OTagStream&   operator<< (OTagStream&, const ObjectSegmentationData&);
ITagStream&   operator>> (ITagStream&, ObjectSegmentationData&);


class ObjectSegmentation
{
public:

	ObjectSegmentation (string &pathFilename, int16 cropLeft, int16 cropRight, int16 cropTop, int16 cropBottom)
	{
		this->pathFilename = pathFilename;
		this->cropLeft   = cropLeft;
		this->cropRight  = cropRight;
		this->cropTop    = cropTop;
		this->cropBottom = cropBottom;
	}

	~ObjectSegmentation(){}

	void process (ObjectSegmentationData &data);

private:
	void readValues(ObjectSegmentationData &data);

	string pathFilename;
	int16 cropLeft;
	int16 cropRight;
	int16 cropTop;
	int16 cropBottom;

};

#endif
