/*
   This file is part of the MoCA video library.
   Name: SadFeature.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef SAD_FEATURE_H
#define SAD_FEATURE_H

// C++ includes
#include <vector>
 
// MoCA includes
#include <Image.h>
#include <Video.h>
#include <WriteMocaData.h>
#include <VideoDecoder.h>


//! Class to calculate SAD (sum of absolute differences)
/*!
  This class will calculate the SAD of the frames of a video.
*/

const static string SAD_FEATURE_VERSION  ="Version-0.01";
const static string SAD_FEATURE_FILENAME ="sad.idx";



struct SadData
{
	SadData (string &pathFilename) 
	{
		version = SAD_FEATURE_VERSION;
		filename = "";

		size              = 0;
		minSadValue       = 1e10;
		maxSadValue       = -1 ;
		avgSadValue       = 0;
		sad.clear();

		this->pathFilename = pathFilename;
		CreateMocaPath pathReader (pathFilename, false);
		mocaPath = pathReader.destinationPath;
	} 
    SadData& operator=(const SadData &other) {
		if (this != &other) {
			version            = other.version;
			filename           = other.filename;

			size               = other.size;
			minSadValue        = other.minSadValue;
			maxSadValue        = other.maxSadValue;
			avgSadValue        = other.avgSadValue;
			sad                = other.sad;
			mocaPath           = other.mocaPath;
			pathFilename       = other.pathFilename;
		} // if
        return *this;
    } // operator=
	
	
	ieee64 minSadValue;
	ieee64 maxSadValue;
	ieee64 avgSadValue;

	int32 size;
    vector<ieee64> sad;

	string version;
	string filename;
	string mocaPath;
	string pathFilename;
};


std::ostream& operator<< (ostream&,    const SadData&);
OTagStream&   operator<< (OTagStream&, const SadData&);
ITagStream&   operator>> (ITagStream&, SadData&);


class SadFeature
{
public:
	static const string DEFAULT_SAD_FEATURE_FILE_NAME;

	SadFeature () {
		pathFilename = "";
	}

	~SadFeature () {};

	// analyze video
	void process (SadData& sad);

	// analyze 2 images
	void process (Image<uint8> &img1, Image<uint8> &img2, ieee64 &sad);

private:
	void readValues (SadData& sad);

	string pathFilename;
	VideoSize size;
};
#endif
