/*
    StructureMatchingFeature.cpp     sum of absolute differences
    Copyright (C) 2005 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <StructureMatchingFeature.h>


std::ostream& operator<< (ostream &os, const StructureMatchingFeatureItem &data) 
{
	os << data.x << "\t" 
	   << data.y << "\t" 
	   << data.diff << endl;

	return os;
} // operator <<


void 
StructureMatchingFeature::process (const Image<uint8> &refImg, const Image<uint8> &srcImg, StructureMatchingFeatureData &data)
{
	data.region.clear();

	Image<uint8> ref, img;
	GrayscaleFilter<uint8> gray;
	gray.process (refImg, ref);
	gray.process (srcImg, img);

	int32 maskSize=5;
	uint8 th_low = 8;
	uint8 th_high= 32;
	CannyFilter<uint8> cf (maskSize, th_low, th_high);
	cf.process (ref);
	cf.process (img);

#if 0
	// debug
	WriteImage (img, "D:/temp/img.png");
	WriteImage (ref, "D:/temp/ref.png");
	exit(1);
#endif

	// get parameters
	ImageSize refSize = ref.getSize();
	ImageSize imgSize = img.getSize();

	// get refCnt factor
	ieee64 refCnt=0;
	for (int32 y=0; y<refSize[DIM_Y]; y++) {
		for (int32 x=0; x<refSize[DIM_X]; x++) {
			if (ref(x,y)!=0) refCnt++;
		} // for
	} // for
	//if (refCnt>0) scale = 1.0 / refCnt;

	// copy to ref data
	SignalSize refDataSize ((int32)refCnt, 2);
	Signal<int32> refData (refDataSize);
	refData.clear(0);
	refCnt=0;
	for (int32 y=0; y<refSize[DIM_Y]; y++) {
		for (int32 x=0; x<refSize[DIM_X]; x++) {
			if (ref(x,y)!=0) {
				refData ((int32)refCnt,0) = x;
				refData ((int32)refCnt,1) = y;
				refCnt++;
			} // if
		} // for
	} // for


	// count matching edges
	if (refCnt<1) refCnt=1;
	ieee64 refScale = 1.0 / refCnt;
	StructureMatchingFeatureItem item;
	for (int32 y=0; y<imgSize[DIM_Y]-refSize[DIM_Y]; y++) {
		cerr << "\r" << y << " / " << imgSize[DIM_Y]-refSize[DIM_Y] << ends;
		for (int32 x=0; x<imgSize[DIM_X]-refSize[DIM_X]; x++) {

			int32 diff=0;

#if 0
			for (int32 dy=0; dy<refDataSize[DIM_Y]; dy++) {
				for (int32 dx=0; dx<refDataSize[DIM_X]; dx++) {
					if ((ref(dx,dy)!=0)  &&  (img(x+dx,y+dy) == 0)) diff++;
				} // for
			} // for
#endif
			for (int32 i=0; i<refCnt; i++) {
				int32 dx=refData(i,0);
				int32 dy=refData(i,1);
				if ((ref(dx,dy)!=0)  &&  (img(x+dx,y+dy) == 0)) diff++;
			} //for

			item.x=x;
			item.y=y;
			item.diff = refScale * diff;
			if (item.diff < th) {
				cerr << item;
				data.region.push_back(item);
			} // if
		} // for
	} // for

	return;
}

	
