/*
   This file is part of the MoCA video library.
   Name: TextRegionDetection.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef TEXT_REGION_DETECTION_H
#define TEXT_REGION_DETECTION_H



// C++ includes
#include <iostream>
using namespace std;


// MoCA includes
#include <Image.h>
#include <IImageStream.h>
#include <OImageStream.h>
#include <ConvertColorSpace.h>
#include <ScaleFilter.h>
#include <ClosingFilter.h>
#include <DrawRectangle.h>
#include <RegionFeature.h>
#include <CropFilter.h>
#include <LuminanceFilter.h>




struct TextRegionRect
{
	TextRegionRect ()
	{
		minX = -1;
		maxX = -1;
		minY = -1;
		maxY = -1;
		rel  = 0.0; // reliability [0..1]
		analysed = false;
		delData  = false;

	}

	int32 minX, maxX, minY, maxY;
	ieee64 rel;
	bool analysed;
	bool delData;
};
std::ostream& operator<< (ostream &, const TextRegionRect&);



struct TextRegionData
{
	TextRegionData () 
	{
		size = 0;
	} 

	int32 size;
	// store (several) bounding boxes for one image (frame)
	vector<TextRegionRect> box;
};
std::ostream& operator<< (ostream &, const TextRegionData&);



/*
	Constructor: initializes variables
	process(): 
		   calculate the position of text regions
*/
class TextRegionDetection
{
public:

	TextRegionDetection() {}
	~TextRegionDetection() {};

	// img: source RGB image
	// data: result, text regions
	void process (const Image<uint8> &img, TextRegionData& data);

	// source data with text regions 
	// img: draw text regions to image
	void draw (TextRegionData&, Image<uint8> &img);

private:
	void store (Signal<int32> &line, TextRegionRect &r, string &fn);
	void getEdgeImage (Image<uint8> &src, Image<int32> &dest);
	void storeDebug (const Image<uint8> &srcOrig, const Image<int32> &src, const string& fnNr);
	void debugStoreTextRegions (const Image<uint8> &srcOrig, TextRegionData& result, const string &fn1);

};
#endif
