/*
    TextSegmentation.cpp     segment text in images
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <TextSegmentation.h>


void 
TextSegmentation::getCharacter (Image<uint8> &img, const Image<uint8> &textImg, Image<uint8> &dest)
{
	
	// analyse cropped region
	//     get regions
	//     merge regions, until 5 regions or less
	//     select character pixels
	
		
#if 0
	{
		WriteImage (imgIn, "D:/test/char-img.png");
		WriteImage (textImg, "D:/test/char-textImg.png");
	}
#endif
	
	ImageSize size = img.getSize();
	size[DIM_BAND]=1;
	dest.resize(size);
	dest.clear(255);
	
	// regions
	uint8 th=1; // initial number of regions
	Neighbours n=N8;
	RegionFeature<uint8> rf (th, n);
	Image<uint32> regions;
	rf.process (img, regions);
	vector<RegionType> rv;
	rf.getRegionVector(rv);
	int32 numberOfRegions = rv.size();
	
	int colorReduce = 16;
	while (numberOfRegions>4000) {
		//cerr << "regions: " << numberOfRegions << endl;

		colorReduce *=2;
		for (int32 y=0; y<size[DIM_Y]; y++) {
			for (int32 x=0; x<size[DIM_X]; x++) {
				img(x,y,0) = colorReduce*(img(x,y,0)/colorReduce);
				img(x,y,1) = colorReduce*(img(x,y,1)/colorReduce);
				img(x,y,2) = colorReduce*(img(x,y,2)/colorReduce);
			} // for
		} // for

		// update regions
		rf.process (img, regions);
		rf.getRegionVector(rv);
		numberOfRegions = rv.size();
		//cerr << "regions: " << numberOfRegions << endl;
	} // while	



	
#if 0
	{
		// store image
		ImageSize displaySize = size;
		displaySize[DIM_BAND] = 1;
		Image<uint8> display (displaySize);
		for (int32 y=0; y<displaySize[DIM_Y]; y++) {
			for (int32 x=0; x<displaySize[DIM_X]; x++) {
				display (x,y) = (regions (x,y) % 8)* 32;
			} // for
		} // for
		WriteImage (display, "D:/test/regions.png");
	}
#endif

	
	// initialize status of region vector
	SignalSize regionSize (rv.size());

	// store status for each region
	Signal<uint8> status (regionSize); // 0: undefined, 1:background, 2:text 
	status.clear(0);

	// store average position for each region
	Signal<int32> avgX (regionSize);
	avgX.clear(0);
	Signal<int32> avgY (regionSize);
	avgY.clear(0);

	// store color for each region
	Signal<int32> c0 (regionSize);
	c0.clear(0);
	Signal<int32> c1 (regionSize);
	c1.clear(0);
	Signal<int32> c2 (regionSize);
	c2.clear(0);

	// store number of pixels for each region
	Signal<int32> nPixel (regionSize);
	nPixel.clear(0);


	// initialize center of regions
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			int32 r = regions(x,y);
			avgX(r) += x;
			avgY(r) += y;
			nPixel(r)++;

			// store color
			c0(r) = img(x,y,0);
			c1(r) = img(x,y,1);
			c2(r) = img(x,y,2);
		} // for
	} // for
	for (int32 r=0; r<rv.size(); r++) {
		if (nPixel(r)>0) {
			avgX(r) /= nPixel(r);
			avgY(r) /= nPixel(r);
		} // if
	} // for

	
	// mark top and bottom regions as background region (status=1)
	for (int32 x=0; x<size[DIM_X]; x++) {
		int32 r = regions (x,0);
		status (r) = 1;
		r = regions (x,size[DIM_Y]-1);
		status (r) = 1;
	} // for

	// mark detected text pixels as text region (status=2)
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (textImg(x,y) ==0) {
				int32 r = regions (x,y);
				if (status (r) == 0) {
					status (r) = 2;
				} // if
			}// if
		} // for
	} // for


	// initialize distance between all regions 
	ImageSize distSize (rv.size(), rv.size());
	Image<ieee64> distData (distSize);
	distData.clear();
	for (int32 i=0; i<rv.size(); i++) {
		for (int32 j=0; j<rv.size(); j++) {
			distData (i,j) = 
				ABS((c0(i)-c0(j)) / 255.0) + 
				ABS((c1(i)-c1(j)) / 255.0) + 
				ABS((c2(i)-c2(j)) / 255.0) + 
				ABS(((ieee64)avgX(i)-avgX(j))   / size[DIM_X]) + 
				ABS(((ieee64)avgY(i)-avgY(j))   / size[DIM_Y]);
#if 0
			ieee64 w0 = c0(i);
			ieee64 w1 = c1(i);
			ieee64 w2 = c2(i);
			ieee64 w3 = c0(j);
			ieee64 w4 = c1(j);
			ieee64 w5 = c2(j);
			ieee64 w6 = avgX(i);
			ieee64 w7 = avgX(j);
			ieee64 w8 = avgY(i);
			ieee64 w9 = avgY(j);
			ieee64 v0=distData (i,j);
			ieee64 v1 = ABS((c0(i)-c0(j)) / 255.0);
			ieee64 v2 = ABS((c1(i)-c1(j)) / 255.0);
			ieee64 v3 = ABS((c2(i)-c2(j)) / 255.0);
			ieee64 v4 = ABS(((ieee64)avgX(i)-avgX(j))   / size[DIM_X]);
			ieee64 v5 = ABS(((ieee64)avgY(i)-avgY(j))   / size[DIM_Y]);
#endif			 

		} // for
	} // for


	// store status
	Signal<uint8> tmpStatus = status;


	// set all undefined text regions 
	for (int32 i=0; i<rv.size(); i++) {
		if (status (i) == 0) {
			int32 minDistP  = 0;
			ieee64 minDistV = 10e10;

			for (int32 j=0; j<rv.size(); j++) {
				if (status (j) != 0) {
					if (minDistV>distData (i,j)) {
						minDistV = distData (i,j);
						minDistP = j;
					} // if
				} // if
			} // for
			if (status (minDistP) == 1) {
				status (i) = 1;
			} else {
				status (i) = 2;
#if 1
				if (status (minDistP) != 2) {
					cerr << "error" << endl;
				} // if
#endif
			} // else
		} // if
	} // for


	// store to dest (skip borders!)
	for (int32 y=1; y<size[DIM_Y]-1; y++) {
		for (int32 x=1; x<size[DIM_X]-1; x++) {
			int32 r = regions(x,y);
			if (status(r) == 2) {
				dest(x,y)-=100;
			} // if
		} // for
	} // for


	// set all undefined text regions: inverse order! 
	status = tmpStatus;
	for (int32 i=rv.size()-1; i>=0; i--) {
		if (status (i) == 0) {
			int32 minDistP  = 0;
			ieee64 minDistV = 10e10;

			for (int32 j=rv.size()-1; j>=0; j--) {
				if (status (j) != 0) {
					if (minDistV>distData (i,j)) {
						minDistV = distData (i,j);
						minDistP = j;
					} // if
				} // if
			} // for
			if (status (minDistP) == 1) {
				status (i) = 1;
			} else {
				status (i) = 2;
#if 1
				if (status (minDistP) != 2) {
					cerr << "error" << endl;
				} // if
#endif
			} // else
		} // if
	} // for

	// store to dest (skip borders!)
	for (int32 y=1; y<size[DIM_Y]-1; y++) {
		for (int32 x=1; x<size[DIM_X]-1; x++) {
			int32 r = regions(x,y);
			if (status(r) == 2) {
				dest(x,y)-=100;
			} // if
		} // for
	} // for


	// store binary image to dest (skip borders!)
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (dest(x,y) < 100) {
				dest(x,y)=0;
			} else {
				dest(x,y)=255;
			} // else
		} // for
	} // for

	// remove borders
	for (int32 y=0; y<size[DIM_Y]; y++) {
		dest(0,y)=255;
		dest(1,y)=255;
		dest(size[DIM_X]-1,y)=255;
		dest(size[DIM_X]-2,y)=255;
	} // for
	for (int32 x=0; x<size[DIM_X]; x++) {
		dest(x,0)=255;
		dest(x,size[DIM_Y]-1)=255;
	} // for

	return;





#if 0
	WriteImage (dest,"D:/test/dest.png");
#endif


#if 0
	Image<uint8> display = img;
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			int32 r = regions(x,y);
			if (status (r) == 0) {
				display (x,y,0) = 0;
				display (x,y,1) = 0;
			} else if (status (r) == 1) {
				display (x,y,0) = 128;
				display (x,y,1) = 128;
			} else if (status (r) == 2) {
				display (x,y,0) = 255;
			} else {
				display (x,y,0) = 0;
				display (x,y,1) = 0;
				display (x,y,2) = 0;
			} // else
		} // for
	} // for
	WriteImage(display, "D:/test/display.png");
	return;
	exit(1);
#endif


#if 0
	int32 r0 = regions (0,0);
	for (int32 x=0; x<size[DIM_X]; x++) {
		int32 r1 = regions (x,0);
		if (r0!=r1) {
			rf.replaceRegionIndex(regions, r0, r1);
			numberOfRegions--;
			r0 = regions (x,0);
		} // if
		
		r1 = regions (x,size[DIM_Y]-1);
		if (r0!=r1) {
			rf.replaceRegionIndex(regions, r0, r1);
			numberOfRegions--;
			r0 = regions (x,size[DIM_Y]-1);
		} // if
	} // for
#endif

#if 0
	{
		// store image
		ImageSize displaySize = size;
		displaySize[DIM_BAND] = 1;
		Image<uint8> display (displaySize);
		for (int32 y=0; y<displaySize[DIM_Y]; y++) {
			for (int32 x=0; x<displaySize[DIM_X]; x++) {
				display (x,y) = (17*regions (x,y) % 8)* 32;
			} // for
		} // for
		WriteImage (display, "D:/test/regions.png");

		for (int32 y=0; y<displaySize[DIM_Y]; y++) {
			for (int32 x=0; x<displaySize[DIM_X]; x++) {
				display (x,y) = (regions (x,y) % 256);
			} // for
		} // for
		WriteImage (display, "D:/test/regions2.png");
	}
#endif


	// calculate average color for each region
	rf.updateRegions(img, regions, rv);
	
	// iterate: merge all similar regions, until number of regions < 50
	int32 dist = 3;
	while (numberOfRegions>6) {
		
		for (int32 y=0; y<size[DIM_Y]; y++) {
			for (int32 x=0; x<size[DIM_X]; x++) {
				
				// check neigbours
				int32 px=x+1;
				int32 py=y;
				if (px<=size[DIM_X]) {
					int32 r0 = regions(x,y);
					int32 r1 = regions(px,py);
					
					if (r0 != r1) {
						int32 d = 
							(rv[r0].c0-rv[r1].c0) * (rv[r0].c0-rv[r1].c0) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c1) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c2); 
						if (d<dist) {
							rf.replaceRegionIndex(regions, r0, r1);
							numberOfRegions--;
							rf.updateRegions(img, regions, rv);
						} // if
					} // if
				} // if
				
				
				px=x-1;
				py=y+1;
				if (px>=0) {
					int32 r0 = regions(x,y);
					int32 r1 = regions(px,py);
					
					if (r0 != r1) {
						int32 d = 
							(rv[r0].c0-rv[r1].c0) * (rv[r0].c0-rv[r1].c0) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c1) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c2); 
						if (d<dist) {
							rf.replaceRegionIndex(regions, r0, r1);
							numberOfRegions--;
							rf.updateRegions(img, regions, rv);
						} // if
					} // if
				} // if
				
				px=x;
				py=y+1;
				if (py<=size[DIM_Y]) {
					int32 r0 = regions(x,y);
					int32 r1 = regions(px,py);
					
					if (r0 != r1) {
						int32 d = 
							(rv[r0].c0-rv[r1].c0) * (rv[r0].c0-rv[r1].c0) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c1) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c2); 
						if (d<dist) {
							rf.replaceRegionIndex(regions, r0, r1);
							numberOfRegions--;
							rf.updateRegions(img, regions, rv);
						} // if
					} // if
				} // if
				
				px=x+1;
				py=y+1;
				if ((px<=size[DIM_X]) && (py<=size[DIM_Y])) {
					int32 r0 = regions(x,y);
					int32 r1 = regions(px,py);
					
					if (r0 != r1) {
						int32 d = 
							(rv[r0].c0-rv[r1].c0) * (rv[r0].c0-rv[r1].c0) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c1) + 
							(rv[r0].c1-rv[r1].c1) * (rv[r0].c1-rv[r1].c2); 
						if (d<dist) {
							rf.replaceRegionIndex(regions, r0, r1);
							numberOfRegions--;
							rf.updateRegions(img, regions, rv);
						} // if
					} // if
				} // if
				
			} // for
		} // for

		// increase maximal distance of similar regions
		dist+=2;
		
#if 0
		{
			// store image
			ImageSize displaySize = size;
			displaySize[DIM_BAND] = 1;
			Image<uint8> display (displaySize);
			for (int32 y=0; y<displaySize[DIM_Y]; y++) {
				for (int32 x=0; x<displaySize[DIM_X]; x++) {
					int32 v = 50+(17*regions (x,y) % 8)* 32;
					if (v>255) v=255;
					if (v<0) v=0;
					display (x,y) = v;
				} // for
			} // for
			cerr << numberOfRegions << endl;
			WriteImage (display, "D:/test/regions.png");

			for (int32 y=0; y<displaySize[DIM_Y]; y++) {
				for (int32 x=0; x<displaySize[DIM_X]; x++) {
					int32 v = 50+(regions (x,y) % 256);
					if (v>255) v=255;
					if (v<0) v=0;
					display (x,y) = v;
				} // for
			} // for
			WriteImage (display, "D:/test/regions2.png");
		}
#endif
		
	} // while
	
	
	// count number of text pixels for each region
	Signal<int32> cnt (rv.size());
	cnt.clear(0);
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (textImg (x, y) == 0) {
				cnt (regions(x,y))++;
			} // if
		} // for
	} // for

	// select region with most text pixels 
	int32 maxCnt=0;
	bool regionFound = false;
	for (int32 i=0; i<rv.size(); i++) {
		if (cnt(maxCnt) < cnt(i)) {
			maxCnt = i;
			regionFound = true;
		} // if
	} // for
	
	// store pixel to dest
	size[DIM_BAND]=1;
	dest.resize(size);
	dest.clear(255);
	if (regionFound) {
		for (int32 y=0; y<size[DIM_Y]; y++) {
			for (int32 x=0; x<size[DIM_X]; x++) {
				if (regions(x,y) == maxCnt) {
					dest (x,y) = 0;
				} // if
			} // for
		} // for
	} // if
	
#if 0
	{
		WriteImage (dest, "D:/test/char-dest.png");
	}
#endif

	return;
}



struct PixelData
{
	PixelData (int32 x, int32 y, int32 c, int32 b) 
	{
		this->x=x;
		this->y=y;
		this->c=c;
		this->b=b;
	}
    PixelData& operator=(const PixelData &other) {
		if (this != &other) {
			x = other.x;
			y = other.y;
			c = other.c;
			b = other.b;
		} // if
        return *this;
    } // operator=
    bool operator< (const PixelData& other) const {
		return (c < other.c);
	}
	int32 x;  // position x
	int32 y;  // position y
	int32 c;  // costs
	int32 b;  // backward (last) node, 1:right, 2:left, 3:up, 10:goal

};

void 
TextSegmentation::getCharacterSeparation (const Image<uint8> &img, int32 &sx0, int32 &sx1)
{
#if 0
		{
			// store image
			WriteImage (img, "D:/test/char-img.png");
		}
#endif

	ImageSize size = img.getSize();
	int32 sx = size[DIM_X];
	int32 sy = size[DIM_Y];

	SignalSize sepSize (sx);
	Signal<int32> separator(sepSize);
	separator.clear(0);


	for (int32 y=0; y<sy-1; y++) {
		for (int32 x=0; x<sx; x++) {
			separator(x) += 
				(img(x,y,0)-img(x,y+1,0))*(img(x,y,0)-img(x,y+1,0)) + 
				(img(x,y,1)-img(x,y+1,1))*(img(x,y,1)-img(x,y+1,1)) + 
				(img(x,y,2)-img(x,y+1,2))*(img(x,y,2)-img(x,y+1,2));
		} // for
	} // for

	// locate minimum
	int32 min0=0;
	int32 th = 0; // 20% of maximum value
	for (int32 x=0; x<sx; x++) {
		if (separator(min0) > separator(x)) min0 = x;
		if (th < separator(x)) th = separator(x);
	} // for	
	int32 v0 = separator(min0);


	// decrease values by min0
	for (int32 x=0; x<sx; x++) {
		separator(x) -= v0;
	} // for	
	v0=0;
	th-=v0;
	th /= 5;

#if 0
	cout << "-----" << endl;
	cout << "-----" << endl;
	for (int32 x=0; x<sx; x++) {
		cout << separator(x) << endl;
	} // for
	cout << "-----" << endl;
#endif


	// remove values near v0
	int32 p0 = min0;
	while ((p0>0) && (separator(p0)<th)) p0--;
	p0-=12; if (p0<0) p0=0;
	int32 p1 = min0;
	while ((p1<sx) && (separator(p1)<th)) p1++;
	p1 +=12; if (p1>sx) p1=sx;
	for (int32 x=p0; x<p1; x++) {
		separator(x) = 10000000;
	} // for	

	// find min1
	int32 min1=0;
	for (int32 x=0; x<sx; x++) {
		if (separator(min1) > separator(x)) min1 = x;
	} // for	
	int32 v1 = separator(min1);
	if (v1>th) min1=-1;

	int32 min2=-1;
	int32 v2=0;
	if (min1>=0) {

		// remove values near min1
		p0 = min1;
		while ((p0>0) && (separator(p0)<th)) p0--;
		p0-=12; if (p0<0) p0=0;
		p1 = min1;
		while ((p1<sx) && (separator(p1)<th)) p1++;
		p1 +=12; if (p1>sx) p1=sx;
		for (int32 x=p0; x<p1; x++) {
			separator(x) = 10000000;
		} // for	


		// find min2
		min2=0;
		for (int32 x=0; x<sx; x++) {
			if (separator(min2) > separator(x)) min2 = x;
		} // for	
		v2 = separator(min2);
		if (v2>th) min2=-1;
	} // if

	// remove min2, if v2>>v1
	if ((min2!=-1) && (v2>2*v1+th/10)) {
		min2 = -1;
	} // if

	sx0 = 0;
	sx1 = sy;
	if (min1==-1) {
		// only one value was found
		sx0 = min0;
	} else {

		if (min2==-1) {
			
			// two values are found
			if (min0<min1) {
				sx0 = min0;
				sx1 = min1;
			} else {
				sx0 = min1;
				sx1 = min0;
			} // else
		} else {

			// three values are found
			if ((min0<=min1) && (min0<=min2)) {
				sx0 = min0;
			} else if ((min1<=min0) && (min1<=min2)) {
				sx0 = min1;
			} else {
				sx0 = min2;
			} // else

			if ((min0<=min1) && (min1<=min2)) {
				sx1 = min1;
			} else if ((min1<=min0) && (min0<=min2)) {
				sx1 = min0;
			} else {
				sx1 = min2;
			} // else
		} // else
	} // else




	// rest can be ignored!


	return;
}


void 
TextSegmentation::getShortestPath (const Image<uint8> &img, const Image<uint8> &text, Image<int32> &path)
{
	ImageSize size = img.getSize();
	size[DIM_BAND] = 1;
	path.resize(size);
	path.clear(0);

	// general idea: calculate shortest path from one position at the top line to the bottom line 



	// initialize variables
	int32 sx = size[DIM_X];
	int32 sy = size[DIM_Y];
	int32 n = sx*sy;
	int32 *d0   = new int32[n];  // cost: path left or right
	int32 *d1   = new int32[n];  // cost: path down
	uint8 *s    = new uint8[n];  // not selected:0, no valid position: 20, selected and backward path is stored 

	// initilaize sets
	multiset<PixelData> data;
	multiset<PixelData>::iterator it;


	// initialize costs
	for (int32 y=0; y<sy; y++) {
		for (int32 x=0; x<sx; x++) {
			int32 p = x+y*sx;
			d0[p] = (int32) (20+
			  ABS ((int32)img(x,y,0)-img(x+1,y,0))+ 
			  ABS ((int32)img(x,y,1)-img(x+1,y,1))+ 
			  ABS ((int32)img(x,y,2)-img(x+1,y,2)));

			d1[p] = (int32) (20+
			  ABS ((int32)img(x,y,0)-img(x,y+1,0))+ 
			  ABS ((int32)img(x,y,1)-img(x,y+1,1))+ 
			  ABS ((int32)img(x,y,2)-img(x,y+1,2)));
		} // for
	} // for


	// get best path for each (every fourth) start position
	for (int32 startPos=0; startPos<size[DIM_X]; startPos+=4) {

		cerr << "\r" << startPos << "  " << ends;

		// clear variables
		data.clear();
		for (int32 i=0; i<n; i++) {
			s[i]  = 0;
		} // for
		for (int32 y=0; y<sy; y++) {
			for (int32 x=0; x<sx; x++) {
				if (text(x,y) == 0) {
					s[x+y*sx] = 20;
				} // if
			} // for
		} // for

		
		int32 p =0;
		PixelData d (0,0,0,0);
		data.insert (PixelData(startPos, 0, 0, 10));
		int32 startP =0;
		while (!data.empty()) {
			
			// find minimum
			//data.sort();  // multimap is already sorted
			bool searchAgain = true;
			while (searchAgain) {
				it = data.begin(); 
				d = (*it);
				//data.pop_front();
				data.erase(it);
				
				p = d.x+d.y*sx;
				if (s[p]==0) {
					searchAgain = false;
				} // if
				if (data.empty()) {
					searchAgain = false;
				} // if
			} // while
			
			if (s[p]==0) {
				s[p]=d.b;  // set backward path
				
				// stop if last line is found
				if (d.y == sy-1) {
					startP = d.x;
					break;
				} // if
				
				if ((d.x>0) && (s[p-1]==0)) {
					data.insert (PixelData (d.x-1, d.y, d0[p-1]+d.c, 1));
				} // if
				if ((d.x<sx-1) && (s[p+1]==0)) {
					data.insert (PixelData (d.x+1, d.y, d0[p]+d.c, 2));
				} // if
				if ((d.y<sy-1) && (s[p+sx]==0)) {
					data.insert (PixelData (d.x, d.y+1, d1[p]+d.c, 3));
				} // if
			} // if
		} // while
		
		
		// mark best path
		int32 px = startP;
		int32 py = (sy-1);
		p = px+py*sx;
		while (s[p]!=10) {
			if (s[p]==1) {
				px+=1;
			} else if (s[p]==2) {
				px-=1;
			} else {
				py-=1;
			} // else
			path(px,py)++;
			p = px+py*sx;
		} // while
		
		
#if 0
		{
			// store image
			ImageSize displaySize = size;
			displaySize[DIM_BAND] = 1;
			Image<uint8> display (displaySize);
			for (int32 y=0; y<displaySize[DIM_Y]; y++) {
				for (int32 x=0; x<displaySize[DIM_X]; x++) {
					int32 v=10*path(x,y);
					if (v>0) v+=50;
					if (v>255) v=255;
					display (x,y) = (uint8)v;
				} // for
			} // for
			WriteImage (display, "D:/test/path.png");
		}
#endif
	} // for

	delete[] d0;
	delete[] d1;
	delete[] s;



#if 0
	{
		// store image
		ImageSize displaySize = size;
		displaySize[DIM_BAND] = 1;
		Image<uint8> display (displaySize);
		for (int32 y=0; y<displaySize[DIM_Y]; y++) {
			for (int32 x=0; x<displaySize[DIM_X]; x++) {
				int32 v=10*path(x,y);
				if (v>0) v+=50;
				if (v>255) v=255;
				display (x,y) = (uint8)v;
			} // for
		} // for
		WriteImage (display, "D:/test/path.png");
	}
#endif


#if 0
	// analyse charSeparator
	bool found = true;
	while (found) {

		// find maximum value 
		int32 px = 0;
		int32 py = sy-1;
		for (int32 x=1; x<size[DIM_X]; x++) {
			if (charSeparator(px,py) < charSeparator(x,py)) px=x;
		} // for

		// follow line and delete line
		int32 minX=px;
		int32 maxX=px;
		while (py > 0) {
			charSeparator (px,py) = 0;
			if (minX > px) minX=px;
			if (maxX < px) maxX=px;

			if ((px>0) && 
			    (charSeparator (px-1,py) >= charSeparator (px+1,py)) && 
				(charSeparator (px-1,py) >= charSeparator (px,py-1))) {
				px--;
			} else if ((px<=size[DIM_X]) && 
			    (charSeparator (px+1,py) >= charSeparator (px-1,py)) && 
				(charSeparator (px+1,py) >= charSeparator (px,py-1))) {
				px++;
			} else if ((py>0) && 
			    (charSeparator (px,py-1) >= charSeparator (px+1,py)) && 
				(charSeparator (px,py-1) >= charSeparator (px-1,py))) {
				py--;
			} // if
		} // for

	} // while

#endif


	return;
}



// separator: 
//   2: no text
//   3: text
//   4: text, but exact border is unknown
void 
TextSegmentation::separateLine (const Image<uint8> &img, const Image<uint8> &text1, 
								const Image<uint8> &text2, Signal<uint8> &separator)
{
	ImageSize size = img.getSize();
	SignalSize sigSize (size[DIM_X],2);
	separator.resize (sigSize);
	separator.clear(0);

	Signal<int32> data (sigSize); 
	data.clear(0);

	// set values
	for (int32 x=0; x<size[DIM_X]; x++) {
		for (int32 y=0; y<size[DIM_Y]-1; y++) {
			data(x,0) += 
				(img(x,y,0)-img(x,y+1,0))*(img(x,y,0)-img(x,y+1,0)) + 
				(img(x,y,1)-img(x,y+1,1))*(img(x,y,1)-img(x,y+1,1)) + 
				(img(x,y,2)-img(x,y+1,2))*(img(x,y,2)-img(x,y+1,2));
			if (text2(x,y)==0) {
				data(x,1)++;
			} // if
		} // for
	} // for	

#if 0
	// get average
	int32 avg=0;
	for (int32 x=0; x<size[DIM_X]; x++) {
		avg+=data(x,0);
	} // for
	avg/=size[DIM_X];
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (data(x,0)>avg) data(x,0)=avg;
	} // for

	// update average
	avg=0;
	for (int32 x=0; x<size[DIM_X]; x++) {
		avg+=data(x,0);
	} // for
	avg/=size[DIM_X];
#endif

	// locate local minima (+/- 12 pixels area): cutPosition
	sigSize[DIM_BAND] = 1;
	Signal<uint8> cutPosition (sigSize);
	cutPosition.clear(0);
	for (int32 x=0; x<size[DIM_X]; x++) {
		int32 p0=x-12; if (p0<0) p0=0;
		int32 p1=x+12; if (p1>size[DIM_X]) p1=size[DIM_X];

		int32 minP = p0;
		for (int32 j=p0+1; j<p1; j++) {
			if (data (minP,0) > data (j,0)) minP=j;
		} // for
		if (minP==x) {
			cutPosition(minP) = 1; 
		} // if
	} // for

	// set text positions
	Signal<uint8> textPosition (sigSize);
	textPosition.clear(0);
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (data(x,1)>0) textPosition(x) = 1;
	} // for


	// validate cut positions (=no text)
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (cutPosition(x) == 1) {
			if (textPosition(x) == 1) {
				cutPosition(x) = 3; // not valid
			} else {
				cutPosition(x) = 2; // valid
			} // else
		} // if
	} // for
	cutPosition(0) = 2; // valid
	cutPosition(sigSize[DIM_X]-1) = 2; // valid

	int32 x0 = 0;
	int32 x1 = 0;
	while (x0<sigSize[DIM_X]) {
		while ((x0<sigSize[DIM_X]) && (cutPosition(x0) != 2)) {
			x0++;
		} // while

		x1 = x0+1;
		while ((x1<sigSize[DIM_X]) && (cutPosition(x1) != 2)) {
			x1++;
		} // while

		if ((cutPosition(x0) == 2) && cutPosition(x1) == 2) {
			bool textFound = false;
			bool separatorFound = false;
			for (int32 i=x0; i<=x1; i++) {
				if (cutPosition(i) == 3) separatorFound = true;
				if (textPosition(i)== 1) textFound = true;
			} // for

			uint8 v = 0;
			if (textFound) {
				if (separatorFound) {
					v = 4; // text, but illegal separators (must be validated)
				} else {
					// no separator found, valid text position 
					v = 3; // text, validatetd
				} // else
			} else {
				v = 2; // no text, validatetd
			} // else

			// set textPosition to 2 (no text), 3(text), or 4 (text, with unknown borders)
			for (int32 i=x0+1; i<x1; i++) {
				textPosition(i) = v; 
			} // for
			textPosition(x0) = 2; 
			textPosition(x1) = 2; 

		} // if
		x0 = x1;

	} // while

#if 0
	cout << "-------------" << endl;
	for (int32 x=0; x<size[DIM_X]; x++) {
		cout << (int)textPosition (x) << endl;
	} // for
	cout << "-------------" << endl;
#endif


	// verify non-validated text areas (textPosition==4)
int32 dbgCnt = 0;
	for (int32 x=0; x<size[DIM_X]; x++) {
dbgCnt++;
		if (textPosition(x) == 4) {
			
			// get last and next valid text cut
			int32 x0 = x;
			while ((x0>0) && (textPosition(x0) != 2)) {
				x0--;
			} // while
			int32 x1 = x;
			while ((x1<size[DIM_X]-1) && (textPosition(x1) != 2)) {
				x1++;
			} // while

			// check width 
			if (x1-x0<=10) {

				// delete text region
				for (int32 i=x0; i<=x1; i++) {
					textPosition(i) = 2; // no text
					cutPosition(i) = 0; // no cut position
				} // for
				
				cutPosition(x0) = 2; // valid cut position
				cutPosition(x1) = 2; // valid cut position
				x0++;
				x1++;
			} // if

			if ((textPosition(x0) == 2) && (textPosition(x1) == 2)) {
				
				// crop region
				int32 x2 = x0-1; if (x2<0) x2=0;
				int32 x3 = x1+1; if (x3>=size[DIM_X]) x3=size[DIM_X]-1;
				Image<uint8> cropImg;
				ImagePosition p0 (x2,0);
				ImagePosition p1 (x3,size[DIM_Y]-1);
				CropFilter<uint8> crop (p0,p1);
				crop.process (img, cropImg);
				ImageSize cropSize = cropImg.getSize();

				

#if 1
				// debug: store cropped image
				{
					WriteImage (cropImg, "D:/test/cropImg5.png");
				}
#endif


				// analyse region
				Image<int32> path;
//path.resize(cropSize);
//path.clear(0);
//if ((dbgCnt == 34) || (dbgCnt == 100) || (dbgCnt == 386) || (dbgCnt == 640) || (dbgCnt == 689)) {
//} else {
				getShortestPath (cropImg, text1, path);
//} // else				

				// count number of destinations
				int32 destNumber=0;
				for (int32 i=0; i<cropSize[DIM_X]; i++) {
					if (path(i,cropSize[DIM_Y]-2) > 0) {
						destNumber++;
					} // if
				} // for

				// non maxima supression (+/-10 pixels)
				if (destNumber>1) {
  					for (int32 i=0; i<cropSize[DIM_X]; i++) {
						int32 v = path (i,cropSize[DIM_Y]-2);
						int32 v0 = i-10; if (v0<0) v0=0;
						int32 v1 = i+10; if (v0>cropSize[DIM_X]) v1=cropSize[DIM_X];
						if (v>0) {
							for (int32 j=v0; j<v1; j++) {
								if (v<path (j,cropSize[DIM_Y]-2)) {
									path (i,cropSize[DIM_Y]-2) = 0;
								} // if
							} // for
						} // if
					} // for

					// update: count number of destinations
					destNumber=0;
					for (int32 i=0; i<cropSize[DIM_X]; i++) {
						if (path(i,cropSize[DIM_Y]-2) > 0) {
							destNumber++;
						} // if
					} // for
				} // if

				// aggregate path image to signal values (analyse center of path image)
				SignalSize pathSize (cropSize[DIM_X]); 
				Signal<int32> pathValues (pathSize);
				pathValues.clear(0);
				for (int32 i=0; i<cropSize[DIM_X]; i++) {
					if (path (i,cropSize[DIM_Y]-2) > 0) {
						pathValues (i) = 1;
					} // if
				} // for
				for (int32 j=cropSize[DIM_Y]-2; j>0; j--) {
					for (int32 i=0; i<cropSize[DIM_X]; i++) {
						if (path (i,j) > 0) {
							if ((pathValues(i-1)>0) || (pathValues(i)>0) || (pathValues(i+1)>0)) {
								if ((j>cropSize[DIM_Y]/3) && (j<2*cropSize[DIM_Y]/3)) {
									pathValues (i)+=10;
								} else {
									pathValues (i)+=1;
								} // else
							} // if
						} // if
					} // for
				} // for

#if 1
				// debug: store path values
				{
					cout << "xxxxxxxxxxxxxx" << endl;
					for (int32 i=0; i<cropSize[DIM_X]; i++) {
						cout << (int) pathValues (i) << endl; 
					} // for
					cout << "xxxxxxxxxxxxxx" << endl;
#endif

#if 1
					// store image
					Image<uint8> display = img;
					DrawLine line (Color::RED);
					DrawLine line2 (Color::GREEN);
					for (int32 x=0; x<size[DIM_X]; x++) {
						if (textPosition(x)>2) {
							ImagePosition p0 (x,size[DIM_Y]-1-5*textPosition(x));
							ImagePosition p1 (x,size[DIM_Y]-1);
							line.process (p0,p1,display);
						} // if
						if (cutPosition(x)==3) {
							ImagePosition p0 (x,20);
							ImagePosition p1 (x,size[DIM_Y]-20);
							line.process (p0,p1,display);
						} // if
						if (cutPosition(x)==2) {
							ImagePosition p0 (x,20);
							ImagePosition p1 (x,size[DIM_Y]-20);
							line2.process (p0,p1,display);
						} // if
						if (data (x,1)>10) {
							ImagePosition p0 (x,0);
							ImagePosition p1 (x,10);
							line2.process (p0,p1,display);
						} else if (data (x,1)>0) {
							ImagePosition p0 (x,0);
							ImagePosition p1 (x,data (x,1));
							line2.process (p0,p1,display);
						} // if
					} // for
					WriteImage (display, "D:/test/lines2.png");
				}
#endif



				// just one character found, no border could be detected (keep old borders)
				if (destNumber==0) {

					// textPosition: 2 (no text), 3(text), or 4 (text, with unknown borders)
					for (int32 i=x0+1; i<x1; i++) {
						textPosition(i) = 3; // text
						cutPosition(i)  = 0; // no cut position
					} // for
				} // if


				// just one character found, and only one exact border found
				if (destNumber==1) {
					int32 p0 = 0;
					while ((p0<cropSize[DIM_X]-1) && (path(p0,cropSize[DIM_Y]-2)==0)) {
						p0++;
					} // while

					// locate maximum values in pathValues near p0 
					int32 p2 = p0-10; if (p2<0) p2=0;
					int32 p3 = p0+10; if (p3>=cropSize[DIM_X]) p3=cropSize[DIM_X]-1;
					int32 maxV = p2;
					for (int32 i=p2; i<=p3; i++) {
						if (pathValues(maxV) < pathValues(i)) maxV=i;
					} // for
					p0 = maxV;

					int32 t = p0;
					while ((t>0) && (pathValues(t-1)>0)) {
						t--;
					} // while
					for (int32 i=p0-1; i>=t; i--) {
						if (pathValues(i) > pathValues(maxV)/2) {
							p0=i;
						} // if
					} // for


					// text position in textPosition 
					int32 x4 = x2 + 0;
					int32 x5 = x2 + cropSize[DIM_X]-1;
					if (p0 < cropSize[DIM_X]/2) {
						//replace x4
						x4 = x2 + p0;
					} else {
						//replace x5
						x5 = x2 + p0;
					} // else


					// delete text region
					for (int32 i=x4; i<=x5; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for
					for (int32 i=x0; i<=x1; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for

					// set text region
					for (int32 i=x4+1; i<x5; i++) {
						textPosition(i) = 3; // text
					} // for
					cutPosition(x4) = 2; // valid cut position
					cutPosition(x5) = 2; // valid cut position
				} // if


				// just one character found, set exact borders
				if (destNumber==2) {
					int32 p0 = 0;
					while ((p0<cropSize[DIM_X]-1) && (path(p0,cropSize[DIM_Y]-2)==0)) {
						p0++;
					} // while
					int32 p1 = cropSize[DIM_X]-1;
					while ((p1>0) && (path(p1,cropSize[DIM_Y]-2)==0)) {
						p1--;
					} // while

					// locate maximum values in pathValues near p0 
					int32 p2 = p0-10; if (p2<0) p2=0;
					int32 p3 = p0+10; if (p3>=cropSize[DIM_X]) p3=cropSize[DIM_X]-1;
					int32 maxV = p2;
					for (int32 i=p2; i<=p3; i++) {
						if (pathValues(maxV) < pathValues(i)) maxV=i;
					} // for
					p0 = maxV;

					int32 t = p0;
					while ((t>0) && (pathValues(t-1)>0)) {
						t--;
					} // while
					for (int32 i=p0-1; i>=t; i--) {
						if (pathValues(i) > pathValues(maxV)/2) {
							p0=i;
						} // if
					} // for



					// locate maximum values in pathValues near p1
					p2 = p1-10; if (p2<0) p2=0;
					p3 = p1+10; if (p3>=cropSize[DIM_X]) p3=cropSize[DIM_X]-1;
					maxV = p2;
					for (int32 i=p2; i<=p3; i++) {
						if (pathValues(maxV) < pathValues(i)) maxV=i;
					} // for
					p1 = maxV;

					t = p1;
					while ((t<cropSize[DIM_X]-1) && (pathValues(t+1)>0)) {
						t++;
					} // while
					for (int32 i=p1+1; i<=t; i++) {
						if (pathValues(i) > pathValues(maxV)/2) {
							p1=i;
						} // if
					} // for


					// text position in textPosition 
					int32 x4 = x2 + p0;
					int32 x5 = x2 + p1;


					// delete text region
					for (int32 i=x4; i<=x5; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for
					for (int32 i=x0; i<=x1; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for

					// set text region
					for (int32 i=x4+1; i<x5; i++) {
						textPosition(i) = 3; // text
					} // for
					cutPosition(x4) = 2; // valid cut position
					cutPosition(x5) = 2; // valid cut position
				} // if


				// more than one character found, set exact borders for first character, 
				// mark second character as unknown text  
				if (destNumber>2) {
					int32 p0 = 0;
					while ((p0<cropSize[DIM_X]-1) && (path(p0,cropSize[DIM_Y]-2)==0)) {
						p0++;
					} // while
					int32 p1 = p0;
					while ((p1<cropSize[DIM_X]-1) && (path(p1,cropSize[DIM_Y]-2)!=0)) {
						p1++;
					} // while
					while ((p1<cropSize[DIM_X]-1) && (path(p1,cropSize[DIM_Y]-2)==0)) {
						p1++;
					} // while


					// locate maximum values in pathValues near p0 
					int32 p2 = p0-10; if (p2<0) p2=0;
					int32 p3 = p0+10; if (p3>=cropSize[DIM_X]) p3=cropSize[DIM_X]-1;
					int32 maxV = p2;
					for (int32 i=p2; i<=p3; i++) {
						if (pathValues(maxV) < pathValues(i)) maxV=i;
					} // for
					p0 = maxV;

					int32 t = p0;
					while ((t>0) && (pathValues(t-1)>0)) {
						t--;
					} // while
					for (int32 i=p0-1; i>=t; i--) {
						if (pathValues(i) > pathValues(maxV)/2) {
							p0=i;
						} // if
					} // for



					// locate maximum values in pathValues near p1
					p2 = p1-10; if (p2<0) p2=0;
					p3 = p1+10; if (p3>=cropSize[DIM_X]) p3=cropSize[DIM_X]-1;
					maxV = p2;
					for (int32 i=p2; i<=p3; i++) {
						if (pathValues(maxV) < pathValues(i)) maxV=i;
					} // for
					p1 = maxV;

					t = p1;
					while ((t<cropSize[DIM_X]-1) && (pathValues(t+1)>0)) {
						t++;
					} // while
					for (int32 i=p1+1; i<=t; i++) {
						if (pathValues(i) > pathValues(maxV)/2) {
							p1=i;
						} // if
					} // for



					// text position in textPosition 
					int32 x4 = x2 + p0;
					int32 x5 = x2 + p1;


					// delete text region
					for (int32 i=x4; i<=x5; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for
					for (int32 i=x0; i<x4; i++) {
						textPosition(i) = 2; // no text
						cutPosition(i) = 0; // no cut position
					} // for

					// set text region
					for (int32 i=x4+1; i<x5; i++) {
						textPosition(i) = 3; // text
					} // for
					cutPosition(x4) = 2; // valid cut position
					cutPosition(x5) = 2; // valid cut position

					// analyse next part of windows again
					cutPosition(x5+2) = 3; // non-valid cut position
				} // if

				


#if 1
				// debug: store path values
				{
					cout << "xxxxxxxxxxxxxx" << endl;
					for (int32 i=0; i<cropSize[DIM_X]; i++) {
						cout << (int) pathValues (i) << endl; 
					} // for
					cout << "xxxxxxxxxxxxxx" << endl;

					// store image
					Image<uint8> display = img;
					DrawLine line (Color::RED);
					DrawLine line2 (Color::GREEN);
					for (int32 x=0; x<size[DIM_X]; x++) {
						if (textPosition(x)>2) {
							ImagePosition p0 (x,size[DIM_Y]-1-5*textPosition(x));
							ImagePosition p1 (x,size[DIM_Y]-1);
							line.process (p0,p1,display);
						} // if
						if (cutPosition(x)==3) {
							ImagePosition p0 (x,20);
							ImagePosition p1 (x,size[DIM_Y]-20);
							line.process (p0,p1,display);
						} // if
						if (cutPosition(x)==2) {
							ImagePosition p0 (x,20);
							ImagePosition p1 (x,size[DIM_Y]-20);
							line2.process (p0,p1,display);
						} // if
						if (data (x,1)>10) {
							ImagePosition p0 (x,0);
							ImagePosition p1 (x,10);
							line2.process (p0,p1,display);
						} else if (data (x,1)>0) {
							ImagePosition p0 (x,0);
							ImagePosition p1 (x,data (x,1));
							line2.process (p0,p1,display);
						} // if
					} // for
					WriteImage (display, "D:/test/lines3.png");
				}
#endif

			} // if
				
		} // if
	} // for
		
		

#if 1
		{
			// store image
			Image<uint8> display = img;
			DrawLine line (Color::RED);
			DrawLine line2 (Color::GREEN);
			for (int32 x=0; x<size[DIM_X]; x++) {
				if (textPosition(x)>0) {
					ImagePosition p0 (x,size[DIM_Y]-1-4*textPosition(x));
					ImagePosition p1 (x,size[DIM_Y]-1);
					line.process (p0,p1,display);
				} // if
				if (cutPosition(x)==3) {
					ImagePosition p0 (x,20);
					ImagePosition p1 (x,size[DIM_Y]-20);
					line.process (p0,p1,display);
				} // if
				if (cutPosition(x)==2) {
					ImagePosition p0 (x,20);
					ImagePosition p1 (x,size[DIM_Y]-20);
					line2.process (p0,p1,display);
				} // if
				if (data (x,1)>10) {
					ImagePosition p0 (x,0);
					ImagePosition p1 (x,10);
					line2.process (p0,p1,display);
				} else if (data (x,1)>0) {
					ImagePosition p0 (x,0);
					ImagePosition p1 (x,data (x,1));
					line2.process (p0,p1,display);
				} // if
			} // for
			WriteImage (display, "D:/test/lines4.png");
			exit(1);
		}
#endif

	// set begin and end of character
	for (int32 x=0; x<size[DIM_X]; x++) {
	} // for

	

	// validate character separators

#if 0
	cout << "-------------" << endl;
	for (int32 x=0; x<size[DIM_X]; x++) {
		cout << (int)data(x,0) << endl;
	} // for
	cout << "-------------" << endl;
	exit(1);
#endif


#if 1
	cout << "-------------" << endl;
	for (int32 x=0; x<size[DIM_X]; x++) {
		cout << (int)separator(x) << endl;
	} // for
	cout << "-------------" << endl;
#endif


	exit(1); 
	return;
}



void 
TextSegmentation::getCharacter(const Image<uint8> &cropImg, Image<uint8> &dest)
{
	ImageSize size = cropImg.getSize();
	dest.resize(size);
	dest.clear(255);

	// apply region growing algorithm
	return;
}


void 
TextSegmentation::selectTextPixel (const Image<uint8> &imgYUV8, const Image<uint8> &imgRGB, 
								   const Image<uint8> &imgTextSmall1, const Image<uint8> &imgTextSmall2, 
								   int32 rx0, int32 rx1, int32 ry0, int32 ry1, Image<uint8> &dest) 
{
#if 0
	WriteImage (imgYUV8, "D:/test/source-yuv.png");
	WriteImage (imgRGB, "D:/test/source-rgb.png");
	WriteImage (imgTextSmall1, "D:/test/source-textSmall-1.png");
	WriteImage (imgTextSmall2, "D:/test/source-textSmall-2.png");
#endif

	// initialize scaled images
	const int scale = 4;
	ImageSize orgSize = imgRGB.getSize();
	ImageSize size = orgSize;
	size[DIM_X]*=scale;
	size[DIM_Y]*=scale;
	Image<uint8> img (size);
	ImageSize destSize = size;
	destSize[DIM_BAND] = 1;
	dest.resize (destSize);
	dest.clear(255);




	// scale image (interpolate)
	int32 f0[scale][scale];
	int32 f1[scale][scale];
	int32 f2[scale][scale];
	int32 f3[scale][scale];
	for (int32 j=0; j<scale; j++) {
		for (int32 i=0; i<scale; i++) {
			f0[i][j] = (4-i)*(4-j);
			f1[i][j] = (  i)*(4-j);
			f2[i][j] = (4-i)*(  j);
			f3[i][j] = (  i)*(  j);
		} // for
	} // for
	
	for (int32 b=0; b<orgSize[DIM_BAND]; b++) {
		for (int32 y=0; y<orgSize[DIM_Y]; y++) {
			for (int32 x=0; x<orgSize[DIM_X]; x++) {
				for (int32 j=0; j<scale; j++) {
					for (int32 i=0; i<scale; i++) {
						int32 v = f0[i][j]*imgRGB(x,y,b) + f1[i][j]*imgRGB(x+1,y,b) + f2[i][j]*imgRGB(x,y+1,b) + f3[i][j]*imgRGB(x+1,y+1,b);
						img (scale*x+i, scale*y+j,b) = (uint8) (v / (scale*scale));
					} // for
				} // for
			} // for
		} // for
	} // for




#if 0
	// debug

	WriteImage (img, "D:/test/img.png");
	bool debugOn=true;
	if (debugOn) {

		// calculate vertical projection profile
		ImageSize tSize = img.getSize();
		tSize[DIM_Y]+=50;
		//tSize[DIM_BAND]=1;
		Image<uint8> display (tSize);
		display.clear(255);
		display.copy(img);
		for (int32 y=0; y<tSize[DIM_Y]-50; y++) {
			for (int32 x=0; x<tSize[DIM_X]; x++) {
				display (x,y,0) = (img(x,y,0)+255)/2;
				display (x,y,1) = (img(x,y,1)+255)/2;
				display (x,y,2) = (img(x,y,2)+255)/2;
			} // for
		} // for
		

		SignalSize dySize (tSize[DIM_X]);
		Signal<double> dy(dySize);
		dy.clear(0);
		for (int32 y=0; y<tSize[DIM_Y]-50; y++) {
			for (int32 x=0; x<tSize[DIM_X]; x++) {
				dy(x) += ABS (img(x,y)-img(x,y+1));
			} // for
		} // for
		double maxV = 1;
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			if (maxV < dy(x)) maxV = dy(x);
		} // for
		maxV = 50.0 / maxV;
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			dy(x) *= maxV;
		} // for
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			ImagePosition p0 (x,tSize[DIM_Y]-50);
			ImagePosition p1 (x,tSize[DIM_Y]-50+dy(x));
			DrawLine line (Color::BLACK);
			line.process(p0,p1,display);
		} // for
		
		// local minima
		const int32 width = 24;
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			int32 p0=x-width; if (p0<0) p0=0;
			int32 p1=x+width; if (p1>tSize[DIM_X]) p1=tSize[DIM_X];
			
			int32 minV=p0;
			for (int32 p=p0; p<p1; p++) {
				if (dy(minV)>dy(p)) minV=p;
			} // for
			if (minV == x) {
				ImagePosition p0 (x,0);
				ImagePosition p1 (x,tSize[DIM_Y]-50-1);
				DrawLine line (Color::BLACK);
				line.process(p0,p1,display);
			} // if
		} // for
		
		WriteImage (display, "D:/test/display.png");
		
		exit(1);
	}
#endif





#if 0
	WriteImage (img, "D:/test/scaled.png");
#endif

	// scale text image
	Image<uint8> imgText1, imgText2;
	ScaleFilter<uint8> sf;
	sf.expand (imgTextSmall1, imgText1, 4, 4);
	sf.expand (imgTextSmall2, imgText2, 4, 4);


	// calculate shotest paths (top to bottom)
	Image<int32> path;	


	getShortestPath (img, imgText1, path);
#if 0
	ReadImage ("D:/test/fullPath.png", path);
#endif


#if 0
	
	// copy to display
	ImageSize dSize = size;
	dSize[DIM_BAND] = 1;
	Image<uint8> display (dSize);
	display.clear(0);
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			int32 v = path(x,y);
			if (v>255) v=255;
			if (v<0) v=0;
			display (x,y) = (uint8) v;
		} // for
	} // for
	WriteImage (display, "D:/test/fullPath.png");


	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (display (x,y)>0) {
				int32 v = display (x,y) + 128;
				if (v>255) v=255;
				if (v<0) v=0;
				display (x,y) = (uint8) v;
			} // if
		} // for
	} // for
	WriteImage (display, "D:/test/fullPath-vis.png");


	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (display (x,y)>0) {
				img(x,y,0) = 0;
				img(x,y,1) = 0;
				img(x,y,2) = 0;
			} else {
				img(x,y,0) = (img(x,y,0)+255)/2;
				img(x,y,1) = (img(x,y,1)+255)/2;
				img(x,y,2) = (img(x,y,2)+255)/2;
			} // else
		} // for
	} // for
	WriteImage (img, "D:/test/fullPath-img.png");
	
	exit(1);
#endif


	// locate most significant shortest paths
	Image<uint8> optPath (destSize);
	optPath.clear(255);
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (path(x,size[DIM_Y]-2)>0) {
			int32 px = x;
			int32 py = size[DIM_Y]-2;

			while (py>0) {
				optPath(px,py) = 0;
				int32 v0 = path(px-1,py);
				int32 v1 = path(px+1,py);
				int32 v2 = path(px,py-1);
				path(px,py)=0;
				if ((px>0) && (v0>v1) && (v0>v2)) {
					px--;
				} else {
					if ((px<size[DIM_X]-1) && (v1>v0) && (v1>v2)) {
						px++;
					} else {
						py--;
					} // else
				} // else
			} // while
			optPath(px,py) = 0;
		} // if
	} // for

#if 0
	WriteImage (optPath, "D:/test/fullPath-one.png");

	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			if (optPath (x,y)==0) {
				img(x,y,0) = 0;
				img(x,y,1) = 0;
				img(x,y,2) = 0;
			} else {
				img(x,y,0) = (img(x,y,0)+255)/2;
				img(x,y,1) = (img(x,y,1)+255)/2;
				img(x,y,2) = (img(x,y,2)+255)/2;
			} // else
		} // for
	} // for
	WriteImage (img, "D:/test/fullPath-one-img.png");


	exit(1);
#endif


	// get separation for characters
	SignalSize sepSize(size[DIM_X]);
	Signal<uint8> separator (sepSize);
	separator.clear(0);

	for (int32 x=0; x<size[DIM_X]; x++) {
		if (optPath(x,2*size[DIM_Y]/3)==0) {
			int32 px = x;
			int32 py = 2*size[DIM_Y]/3;

			ieee64 c = 0;
			ieee64 avg=0;
			while (py>size[DIM_Y]/3) {

				if ((py>size[DIM_Y]/3) && (py<2*size[DIM_Y]/3)) {
					avg+=px;
					c++;
				} // if
				int32 v0 = optPath(px-1,py);
				int32 v1 = optPath(px+1,py);
				int32 v2 = optPath(px,py-1);
				optPath(px,py) = 255;
				if (v2==0) {
					py--;
				} else if ((px<size[DIM_X]-1) && (v1==0)) {
					px++;
				} else if ((px>0) && (v0==0)) {
					px--;
				} else {
					c=0;
					break;
				} // else
			} // while
			if (c>0) {
				avg=avg/c+0.5;
				separator ((int32)avg) = 255;
			} // if
		} // if
	} // for


#if 0
	Image<uint8> display = img;
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (separator (x)==255) {
			ImagePosition p0(x,0);
			ImagePosition p1(x,155);
			DrawLine draw(Color::BLACK);
			draw.process(p0,p1,display);
		} // if
	} // for
	WriteImage (display, "D:/test/fullPath-separators.png");
#endif


#if 1
	// reduce number of colors in image
	const int colorReduce = 16;
	for (int32 y=0; y<size[DIM_Y]; y++) {
		for (int32 x=0; x<size[DIM_X]; x++) {
			img(x,y,0) = colorReduce*(img(x,y,0)/colorReduce);
			img(x,y,1) = colorReduce*(img(x,y,1)/colorReduce);
			img(x,y,2) = colorReduce*(img(x,y,2)/colorReduce);
		} // for
	} // for
#endif




	// analyse each region between two separators, get text pixels in this region
	size[DIM_BAND]=1;
	Image<uint8> charImg (size);
	charImg.clear(255);
#if 0
	charImg = display;
#endif
	cerr << endl;
	for (int32 x=0; x<size[DIM_X]; x++) {
		if (separator (x)==255) {
			int32 x0=x;
			int32 x1=x+1;
			while ((x1<size[DIM_X]) && (separator(x1)!=255)) x1++;

			if ((separator(x0)==255) && (separator(x1)==255)) {
				cerr << "\r" << x << "/" << size[DIM_X] << ends;
				
				// check minimum width of a character
				if (x1-x0>10) {
					x0--; if (x0<0) x0=0;
					x1++; if (x1>=size[DIM_X]) x1=size[DIM_X]-1;
					
					ImagePosition p0 (x0,0);
					ImagePosition p1 (x1,size[DIM_Y]);
					CropFilter<uint8> cf (p0, p1);

					Image<uint8> cropImg;
					cf.process (img, cropImg);
					ImageSize cropSize = cropImg.getSize();

					Image<uint8> cropImgText;
					cf.process (imgText1, cropImgText);


#if 0
					cropCharImg.resize(cropSize);
					cropCharImg.clear(128);
#endif
					Image<uint8> cropCharImg;
					getCharacter(cropImg, cropImgText, cropCharImg);

#if 0
					WriteImage (cropCharImg,"D:/test/res.png");
#endif

#if 1
					// get regions
					uint8 th=1; 
					Neighbours n=N4;
					RegionFeature<uint8> rf (th, n);
					Image<uint32> regions;
					rf.process (cropCharImg, regions);
					vector<RegionType> rv;
					rf.getRegionVector(rv);

#if 0
					Image<uint8> display = cropCharImg;
					WriteImage (display,"D:/test/a--cropCharImg.png");
					display.clear(255);
					for (int32 py=0; py<cropSize[DIM_Y]; py++) {
						for (int32 px=0; px<cropSize[DIM_X]; px++) {
							display(px,py) = 64*(regions(px,py)%4);
						} // for
					} // for
					cerr << "number of regions: " << rv.size() << endl;
					WriteImage (display,"D:/test/a--regions.png");

#endif
					// copy text pixels in largest region
					int32 thPixel = 100;
					for (int32 py=0; py<cropSize[DIM_Y]; py++) {
						for (int32 px=0; px<cropSize[DIM_X]; px++) {
							if (cropCharImg(px,py)==0) {
								int32 r = regions(px,py);
								if (rv[r].numPixels > thPixel) {
									charImg (px+x0,py,0) = 0;
								} // if
							} // if
						} // for
					} // for

#if 0
					WriteImage (charImg,"D:/test/a--charImg.png");
#endif
#endif
#if 0

					int32 maxR = 0;
					int32 maxV = -1;
					for (int32 i=0; i<rv.size(); i++) {
						if (rv[i].c0 == 0) {
							if (maxV<rv[i].numPixels) {
								maxV=rv[i].numPixels;
								maxR=i;
							} // if
						} // if
					} // for


#endif

#if 0
					// copy text pixels 
					for (int32 py=0; py<cropSize[DIM_Y]; py++) {
						for (int32 px=0; px<cropSize[DIM_X]; px++) {
							if (cropCharImg(px,py)==0) {
								charImg (px+x0,py,0) = 0;
							} // if
						} // for
					} // for
#endif

#if 0
					// copy binary image to charImg
					SampleBlockPosition p(x0,0,0,0);
					charImg.copy(cropCharImg, p);
#endif


#if 0
					// mark text pixels
					for (int32 py=0; py<cropSize[DIM_Y]; py++) {
						for (int32 px=0; px<cropSize[DIM_X]; px++) {
							if (cropCharImg(px,py)==0) {
								charImg (px+x0,py,0) = 255;
								charImg (px+x0,py,1) /=2;
								charImg (px+x0,py,2) /=2;
							} // if
						} // for
					} // for

					WriteImage (cropCharImg,"D:/test/out-char.png");
					WriteImage (charImg,"D:/test/out-char-line.png");
#endif
					
				} // if
			} // if

		} // if
	} // for
	cerr << endl;

#if 0
	WriteImage (charImg,"D:/test/out-char-line-a.png");
#endif
	
	ClosingFilter<uint8> of (5);
	of.process(charImg);
#if 0
	WriteImage (charImg,"D:/test/closing-1.png");
#endif
	OpeningFilter<uint8> opf (3);
	opf.process(charImg);
#if 0
	WriteImage (charImg,"D:/test/opening-2.png");
#endif

	dest = charImg;
	return;


exit(1);


	// initialize window
	int32 x0=0;
	int32 y0=0;
	int32 width = orgSize[DIM_Y];

	// set py0
	bool found = false;
	for (int32 y=0; y<orgSize[DIM_Y]; y++) {
		for (int32 x=0; x<width; x++) {
			if (imgTextSmall2 (x,y) == 0) {
				x0 = x;
				y0 = y;
				found = true;
				break;
			} // if
		} // for
		if (found) break;
	} // for

	// set py1
	int32 x1=x0+width;
	int32 y1=orgSize[DIM_Y];
	found = false;
	for (int32 y=orgSize[DIM_Y]-1; y>=0; y--) {
		for (int32 x=0; x<width; x++) {
			if (imgTextSmall2 (x,y) == 0) {
				x1 = x;
				y1 = y;
				found = true;
				break;
			} // if
		} // for
		if (found) break;
	} // for
	width = y1-y0+3;

	// set px0
	found = false;
	for (int32 x=0; x<width; x++) {
		for (int32 y=y0; y<=y1; y++) {
			if (imgTextSmall2 (x,y) == 0) {
				x0 = x;
				found = true;
				break;
			} // if
		} // for
		if (found) break;
	} // for

	// set px1
	x0-=3;			 if (x0<0) x0=0;
	y0-=3;			 if (y0<0) y0=0;
	y1+=3;		     if (y1>=orgSize[DIM_Y]) y1=orgSize[DIM_Y]-1;
	x1 = x0+y1-y0; 
	if (x1-x0<10) x1=x0+10;
	if (x1>=orgSize[DIM_X]) x1=orgSize[DIM_X]-1;




	// work on scaled image
	x0 = 4*x0;
	x1 = 4*x1+3;
	y0 = 4*y0;
	y1 = 4*y1+3;
	width = x1-x0;

	while (x1<size[DIM_X]) {

		cerr << "pos: " << x0 << " / " << x1 << endl;

		// crop scaled image
		Image<uint8> cropImg;
		{
			ImagePosition p0 (x0, y0);
			ImagePosition p1 (x1, y1);
			CropFilter<uint8> cf (p0, p1);
			cf.process (img, cropImg);
		}


		// get separation fo characters
		int32 sx0, sx1;
		getCharacterSeparation (cropImg, sx0, sx1);
		x0 += sx0;
		x1  = x0+sx1;
		
		// crop images
		ImagePosition p0 (x0, y0);
		ImagePosition p1 (x1, y1);
		CropFilter<uint8> cf (p0, p1);
		cf.process (img, cropImg);
		ImageSize cropSize = cropImg.getSize();

		Image<uint8> cropText;
		cf.process (imgText2, cropText);


#if 1
		{
			// store image
			Image<uint8> display = img;
			DrawRectangle rect (Color::RED);
			ImagePosition p0 (x0,y0);
			ImagePosition p1 (x1,y1);
			rect.process (p0,p1,display);
			WriteImage (display, "D:/test/separation.png");

			WriteImage (cropImg, "D:/test/crop.png");
			WriteImage (cropText, "D:/test/crop-text.png");
		}
#endif



		// get text characters of small window
		Image<uint8> cropDest;
		getCharacter (cropImg, cropText, cropDest);
#if 1
		WriteImage (cropDest, "D:/test/crop-dest.png");
#endif


		// copy characters to dest
		for (int32 y=0; y<cropSize[DIM_Y]; y++){
			for (int32 x=0; x<cropSize[DIM_X]; x++){
				if (cropDest(x,y) == 0) {
					dest (x0+x,y0+y) = 0;
				} // if
			} // for
		} // for
#if 1
		WriteImage (dest, "D:/test/dest.png");
#endif




		// set start position for next window
		bool found = false;
		for (int32 x=cropSize[DIM_X]-1; x>=0; x--){
			for (int32 y=0; y<cropSize[DIM_Y]; y++){
				if (cropDest(x,y)==0) {
					x0+=x;
					x1=x0+width;
					found = true;
					break;
				} // if
			} // for
			if (found) break;
		} // for

	} // while
		
	cerr << "finished" << endl;
	exit(1);

	return;
}


void 
TextSegmentation::estimateTextPixel (const Image<uint8> &img, int32 rx0, int32 rx1, int32 ry0, int32 ry1, 
									 Image<uint8> &dest1, Image<uint8> &dest2) 
{
#if 0
	{
		Image<uint8> display = img;
		VideoCodec srcMode  = CODEC_YUV8;
		VideoCodec destMode = CODEC_YUV;
		ConvertColorSpace::process (srcMode, destMode, display);
		WriteImage(display, "D:/test/yuv-region.png");
	}
#endif

	// get image size
	ImageSize size = img.getSize();
	int32 sx = size[DIM_X];
	int32 sy = size[DIM_Y];
	int32 height = ry1-ry0+1;
	size[DIM_BAND] = 1;
	dest1.resize(size);
	dest1.clear(255);
	dest2.resize(size);
	dest2.clear(255);

	// histogram
	ImageSize histSize (256, size[DIM_Y], 1);
	Image<int32> hist (histSize);

	
	// calculate one histogram for each line
	hist.clear(0);
	for (int32 y=0; y<sy; y++) {      // for full height (text and border)
		for (int32 x=rx0; x<rx1; x++) {  // skip left and right border 
			hist (img(x,y),y)++;
		} // for
	} // for
	
	// calculate average histogram for text region
	SignalSize textHistSize (256);
	Signal<int32> textHist (textHistSize);
	textHist.clear(0);
	for (int32 y=ry0; y<ry1; y++) {
		for (int32 x=0; x<256; x++) {
			textHist(x) += hist (x,y);
		} // for
	} // for
	for (int32 x=0; x<256; x++) {
		textHist(x) /= height;
	} // for
	


	// calculate average histogram for non-text region
	Signal<int32> nontextHist (textHistSize);
	nontextHist.clear(0);
	int32 cnt = 0;
	for (int32 y=0; y<sy; y++) {
		if ((y<ry0) || (y>ry1)) {
			cnt++;
			for (int32 x=0; x<256; x++) {
				nontextHist(x) += hist (x,y);
			} // for
		} // if
	} // for
	for (int32 x=0; x<256; x++) {
		nontextHist(x) /= cnt;
	} // for

	
	
#if 0
	cout << "---textHist---" << endl;
	for (int32 x=0; x<256; x++) {
		cout << textHist(x) << endl;
	} // for
	cout << "---" << endl;
	cout << "---" << endl;
	cout << "---" << endl;
	cout << "---nonTextHist---" << endl;
	for (int32 x=0; x<256; x++) {
		cout << nontextHist(x) << endl;
	} // for
	cout << "---" << endl;
	exit(1);
#endif

#if 0
	// calculate vertical projection profile
	ImageSize tSize = img.getSize();
	tSize[DIM_Y]+=50;
	Image<uint8> display (tSize);
	display.clear(255);
	display.copy(img);
	for (int32 y=0; y<tSize[DIM_Y]-50; y++) {
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			int32 v = display (x,y);
			

			v += (255-v)/4;

			if (v<0) v=0;
			if (v>255) v=255;
			display (x,y) = (uint8)v;
		} // for
	} // for

	SignalSize dySize (tSize[DIM_X]);
	Signal<double> dy(dySize);
	dy.clear(0);
	for (int32 y=0; y<tSize[DIM_Y]-50; y++) {
		for (int32 x=0; x<tSize[DIM_X]; x++) {
			dy(x) += ABS (img(x,y)-img(x,y+1));
		} // for
	} // for
	double maxV = 1;
	for (int32 x=0; x<tSize[DIM_X]; x++) {
		if (maxV < dy(x)) maxV = dy(x);
	} // for
	maxV = 50.0 / maxV;
	for (int32 x=0; x<tSize[DIM_X]; x++) {
		dy(x) *= maxV;
	} // for
	for (int32 x=0; x<tSize[DIM_X]; x++) {
		ImagePosition p0 (x,tSize[DIM_Y]-50);
		ImagePosition p1 (x,tSize[DIM_Y]-50+dy(x));
		DrawLine line (Color::BLACK);
		line.process(p0,p1,display);
	} // for

	// local minima
	const int32 width = 8;
	for (int32 x=0; x<tSize[DIM_X]; x++) {
		int32 p0=x-width; if (p0<0) p0=0;
		int32 p1=x+width; if (p1>tSize[DIM_X]) p1=tSize[DIM_X];

		int32 minV=p0;
		for (int32 p=p0; p<p1; p++) {
			if (dy(minV)>dy(p)) minV=p;
		} // for
		if (minV == x) {
			ImagePosition p0 (x,0);
			ImagePosition p1 (x,tSize[DIM_Y]-50-1);
			DrawLine line (Color::BLACK);
			line.process(p0,p1,display);
		} // if
	} // for

	WriteImage (display, "D:/test/display.png");

	exit(1);
#endif


	Signal<uint8> textColor (textHistSize);
	textColor.clear(0);
	ieee64 maxDiff = 0;
	for (int32 x=0; x<256; x++) {
		int v = textHist(x) - nontextHist(x);
		if (maxDiff < v) maxDiff=v;
	} // for
	if (maxDiff < 1.0) maxDiff = 1.0;
	ieee64 f = 255.0 / maxDiff;
	for (int32 x=0; x<256; x++) {
		ieee64 v = f * MAX(textHist(x) - nontextHist(x), 0);
		if (v<0) v=0;
		if (v>255) v=255;
		textColor (x) = (uint8) v;
	} // for
	
	
	// get all significant text colors (set dest2 to 128)
	for (int32 maxP=0; maxP<256; maxP++) {
		if (textColor(maxP)>0) {
			for (int32 y=0; y<sy; y++) {      
				for (int32 x=0; x<sx; x++) {  
					if (img(x,y) == maxP) {
						dest2(x,y) = 128;
					} // if
				} // for
			} // for
		} // if
	} // for
	
	// get most significant text color (set dest1 and dest2 to zero)
	int32 maxP = 0;
	for (int32 x=0; x<256; x++) {
		if (textColor(maxP) < textColor(x)) maxP=x;
	} // for
	for (int32 y=0; y<sy; y++) {      
		for (int32 x=0; x<sx; x++) {  
			if (img(x,y) == maxP) {
				dest1(x,y) = 0;
				dest2(x,y) = 0;
			} // if
		} // for
	} // for
	
	
#if 0
	WriteImage (dest1, "D:/test/chars1.png");
	WriteImage (dest2, "D:/test/chars2.png");
#endif
	
	
	// mark all gray pixels that have a black pixel as neigbour
	Image<uint8> textPixel = dest2;
	Image<uint8> tmp = dest2;
	for (int32 j=0; j<2; j++) {
		for (int32 y=0; y<sy; y++) {      
			for (int32 x=0; x<sx; x++) {  
				if (textPixel(x,y) == 128) {
					if ((textPixel(x+1,y+1) == 0) ||
						(textPixel(x+1,y  ) == 0) ||
						(textPixel(x+1,y-1) == 0) ||
						(textPixel(x  ,y+1) == 0) ||
						(textPixel(x  ,y-1) == 0) ||
						(textPixel(x-1,y+1) == 0) ||
						(textPixel(x-1,y  ) == 0) ||
						(textPixel(x-1,y-1) == 0)) {
						tmp(x,y) = 0;
					} // if
				} // if
			} // for
		} // for
		textPixel = tmp;
	} // for
	
#if 0
	WriteImage (textPixel, "D:/test/textpixel-1.png");
#endif
	
	
	// remove gray pixels
	for (int32 y=0; y<sy; y++) {      
		for (int32 x=0; x<sx; x++) {  
			if (textPixel(x,y) == 128) {
				textPixel(x,y) = 255;
			} // if
		} // for
	} // for
	
#if 0
	WriteImage (textPixel, "D:/test/textpixel-2.png");
#endif
	
	// analyse regions, set border to background and remove non-connencted text pixels for image textPixel
	for (int32 y=0; y<sy; y++) {      
		textPixel(0,y) = 255;
		textPixel(sx-1,y) = 255;
	} // for
	for (int32 x=0; x<sx; x++) {  
		textPixel(x,0) = 255;
		textPixel(x,sy-1) = 255;
	} // for

	{
		Image<uint32> regions;
		vector<RegionType> regionVector;
		uint8 threshold=1;
		Neighbours n=N8;
		RegionFeature<uint8> rf (threshold, n);
		rf.process(textPixel, regions);
		rf.getRegionVector(regionVector);
		
		SignalSize isTextSize (regionVector.size());
		Signal<uint8> isText (isTextSize);
		isText.clear(0);
		for (int32 y=ry0; y<ry1; y++) {      
			for (int32 x=rx0; x<rx1; x++) {  
				isText(regions(x,y)) = 1;
			} // for
		} // for
		for (int32 y=0; y<sy; y++) {      
			for (int32 x=0; x<sx; x++) {  
				if (isText(regions(x,y)) == 0) {
					textPixel(x,y)=255;
				} // if
			} // for
		} // for
	}


	// analyse regions, set border to background and remove non-connencted text pixels for image dest1
	for (int32 y=0; y<sy; y++) {      
		dest1(0,y) = 255;
		dest1(sx-1,y) = 255;
	} // for
	for (int32 x=0; x<sx; x++) {  
		dest1(x,0) = 255;
		dest1(x,sy-1) = 255;
	} // for

	{
		Image<uint32> regions;
		vector<RegionType> regionVector;
		uint8 threshold=1;
		Neighbours n=N8;
		RegionFeature<uint8> rf (threshold, n);
		rf.process(dest1, regions);
		rf.getRegionVector(regionVector);
		
		SignalSize isTextSize (regionVector.size());
		Signal<uint8> isText (isTextSize);
		isText.clear(0);
		for (int32 y=ry0; y<ry1; y++) {      
			for (int32 x=rx0; x<rx1; x++) {  
				isText(regions(x,y)) = 1;
			} // for
		} // for
		for (int32 y=0; y<sy; y++) {      
			for (int32 x=0; x<sx; x++) {  
				if (isText(regions(x,y)) == 0) {
					dest1(x,y)=255;
				} // if
			} // for
		} // for
	}

	

#if 0
	WriteImage (textPixel, "D:/test/textpixel-3.png");
#endif
	
	
#if 0
	cout << "---" << endl;
	for (int32 x=0; x<256; x++) {
		cout << (int )textColor(x) << endl;
	} // for
	cout << "---" << endl;
	for (int32 x=0; x<256; x++) {
		cout << textHist(x) << endl;
	} // for
	cout << "---" << endl;
	for (int32 x=0; x<256; x++) {
		cout << nontextHist(x) << endl;
	} // for
	cout << "---" << endl;
	exit(1);
#endif

	// return text pixels
	dest2 = textPixel;
	return;
}



void 
TextSegmentation::process(const Image<uint8> &input, TextRegionData& data, Image<uint8> &dest) 
{

	// convert input image to YUV image with 8 bits
	Image<uint8> img;
	VideoCodec srcMode  = CODEC_RGB;
	VideoCodec destMode = CODEC_YUV8;
	ConvertColorSpace::process (srcMode, destMode, input, img);
	ImageSize size = img.getSize();



	// store characters to dest
	ImageSize destSize = size;
	destSize[DIM_BAND] = 1;
	destSize[DIM_X] *=4;
	destSize[DIM_Y] *=4;
	dest.resize(destSize);
	dest.clear(255);


#if 0

	{
		Image<uint8> display = img;
		srcMode  = CODEC_YUV8;
		destMode = CODEC_YUV;
		ConvertColorSpace::process (srcMode, destMode, display);
		WriteImage(display, "D:/test/yuv.png");
	}

#endif


	VideoSize vidSize (destSize[DIM_X], destSize[DIM_Y], data.size, 1);
	Video<uint8> destImages (vidSize);
	destImages.clear(255);
	for (int32 i=0; i<data.size; i++) {

		int32 y0 = data.box[i].minY;
		int32 y1 = data.box[i].maxY;
		int32 x0 = data.box[i].minX;
		int32 x1 = data.box[i].maxX;
		int32 height = y1-y0+1;

		// increase text box 
		int32 x2 = x0-height;  if (x2<0) x2=0;
		int32 x3 = x1+height;  if (x3>=size[DIM_X]) x3=size[DIM_X]-1;
		int32 y2 = y0-height;  if (y2<0) y2=0;
		int32 y3 = y1+height;  if (y3>=size[DIM_Y]) y3=size[DIM_Y]-1;


		Image<uint8> cropYUV8, cropRGB;
		ImagePosition p0 (x2,y2);
		ImagePosition p1 (x3,y3);
		CropFilter<uint8> cf (p0,p1);
		cf.process (img, cropYUV8);
		cf.process (input, cropRGB);
		ImageSize cropSize = cropYUV8.getSize();

		// get estimation for text pixels
		int32 rx0 = x0-x2;
		int32 rx1 = cropSize[DIM_X] - (x3-x1);
		int32 ry0 = y0-y2;
		int32 ry1 = cropSize[DIM_Y] - (y3-y1);
		Image<uint8> textEstImg1, textEstImg2;
		estimateTextPixel (cropYUV8, rx0, rx1, ry0, ry1, textEstImg1, textEstImg2);


		// find exact text pixels
		Image<uint8> textImg;
		selectTextPixel (cropYUV8, cropRGB, textEstImg1, textEstImg2, rx0, rx1, ry0, ry1, textImg);
		

		// copy tempDest to dest (crop border!)
		for (int32 y=1; y<textImg.getSize()[DIM_Y]-1; y++) {
			for (int32 x=1; x<textImg.getSize()[DIM_X]-1; x++) {
				if (textImg(x,y) == 0) {
					destImages(x+4*x2,y+4*y2,i,0) = 0;
				} // if
			} // for
		} // for
		//ImagePosition p(x0, y0);
		//dest.copy(textImg, p);

#if 0
		// debug: save 
		{
			string fn = FileHandling::getNumberedFilename("D:/test/dest",i,".png");
			WriteImage (destImages.getImage(i), fn);
			cerr << "text line written!" << endl;
		}
#endif

#if 0
		// debug: load
		{
			// debug!!!
			string fn = FileHandling::getNumberedFilename("D:/test/dest",i,".png");
			Image<uint8> tmp;
			ReadImage (fn, tmp);
			destImages.setImage(i,tmp);
		}
#endif



	} // for

#if 0
	WriteImage (dest, "D:/test/chars.png");
#endif


	// aggregate the text color of all text lines
	int32 nImages = vidSize[DIM_Z];
	SignalSize histSize (256, nImages+1);
	Signal<ieee64> hist (histSize);
	hist.clear(0);
	SignalSize hdSize (nImages+1,2);
	Signal<ieee64> hd (hdSize); // (i,0): count number of pixel,  (i,1): histogram difference 
	hd.clear(0);

	// calculate histogram
	for (int32 z=0; z<nImages; z++) {      
		for (int32 y=0; y<vidSize[DIM_Y]; y++) {      
			for (int32 x=0; x<vidSize[DIM_X]; x++) {
				if (destImages(x,y,z) == 0) {
					hist (img(x/4,y/4),z)++;
					hd(z,0)++;
				} // if
			} // for
		} // for
	} // for

	// get average histogram
	for (int32 z=0; z<nImages; z++) { 
		for (int32 i=0; i<256; i++) {
			hist (i,nImages) += hist (i,z);
		} // for
		hd   (nImages,0) += hd (z,0);
	} // for

#if 0
	// debug
	for (int32 z=0; z<nImages+1; z++) { 
		string fn = FileHandling::getNumberedFilename("D:/test/d_" , z, ".txt");
		ofstream ofs (fn.c_str());
		for (int32 i=0; i<256; i++) {
			ofs << hist (i,z) << endl;
		} // for
		ofs.close();
	} // for

#endif

	// normalize histogram
	for (int32 z=0; z<nImages+1; z++) {      
		if (hd(z,0)==0) hd(z,0)=1;
		for (int32 i=0; i<256; i++) {
			hist(i,z) /=hd(z,0);
		} // for
	} // for

#if 0
	// debug
	for (int32 z=0; z<nImages+1; z++) { 
		string fn = FileHandling::getNumberedFilename("D:/test/e_" , z, ".txt");
		ofstream ofs (fn.c_str());
		for (int32 i=0; i<256; i++) {
			ofs << hist (i,z) << endl;
		} // for
		ofs.close();
	} // for

#endif

	// get histogram differences
	for (int32 z=0; z<nImages; z++) {      
		for (int32 i=0; i<256; i++) {
			hd(z,1) += ABS (hist(i,z) - hist(i,nImages));
		} // for
	} // for


	// get histogram differences
	ieee64 avgHd = 0.0;
	for (int32 z=0; z<nImages; z++) {      
		avgHd += hd(z,1);
	} // for
	if (nImages>0) {
		avgHd /=nImages;
	} // if
	avgHd+=0.1; // increase threshold 


	// delete text lines with very large histogram differences
	for (int32 z=0; z<nImages; z++) {      

		// text line?
		if (hd(z,1) < avgHd) {

			// copy destImages to dest
			for (int32 y=0; y<vidSize[DIM_Y]; y++) {      
				for (int32 x=0; x<vidSize[DIM_X]; x++) {
					if (destImages(x,y,z) == 0) {
						dest(x,y)=0;
					} // if
				} // for
			} // for
		} // if

	} // for


#if 0
	string fn = "D:/test/f_.txt";
	ofstream ofs (fn.c_str());
	for (int32 z=0; z<nImages; z++) { 
		ofs << hd(z,1) << endl;
	} // for
	ofs.close();
#endif

	return;
} // process

