/*
   This file is part of the MoCA video library.
   Name: TextSegmentation.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

 
  
   \author    Stephan Kopf
   
	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de
	
	 (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt
	 
*/


#ifndef TEXT_SEGMENTATION_H
#define TEXT_SEGMENTATION_H


// MoCA includes
#include <MocaTypes.h>
#include <Image.h>
#include <Video.h>

#include <TextRegionDetection.h>
#include <CropFilter.h>
#include <CannyFilter.h>
#include <RegionFeature.h>
#include <DrawRectangle.h>
#include <OpeningFilter.h>


// C++ includes
#include <set>
#include <iostream>
using namespace std;


/*
	Constructor: initializes variables
	
	process(): 
		   calculate the position of text regions

*/


class TextSegmentation
{
public:

	TextSegmentation() {}
	~TextSegmentation() {};

	// img:  source RGB image
	// data: source text regions
	// dest: binary image with characters
	void process (const Image<uint8> &img, TextRegionData& data, Image<uint8> &dest);

private:
	void estimateTextPixel (const Image<uint8> &img, int32 rx0, int32 rx1, int32 ry0, int32 ry1, 
						    Image<uint8> &dest1, Image<uint8> &dest2); 

	void selectTextPixel (const Image<uint8> &imgYUV8, const Image<uint8> &imgRGB, 
						  const Image<uint8> &imgText1, const Image<uint8> &imgText2, 
		                  int32 rx0, int32 rx1, int32 ry0, int32 ry1, Image<uint8> &dest);
	void getCharacter (Image<uint8> &img, const Image<uint8> &textImg, Image<uint8> &dest);
	void getCharacterSeparation (const Image<uint8> &cropImg, int32 &sx0, int32 &sx1);
	void separateLine (const Image<uint8> &img, const Image<uint8> &text1, 
					   const Image<uint8> &text2, Signal<uint8> &separator);
	void getShortestPath (const Image<uint8> &img, const Image<uint8> &text, Image<int32> &path);
	void getCharacter(const Image<uint8> &cropImg, Image<uint8> &cropCharImg);

};

#endif
