/*
    TrackingFeature.cpp     track an object
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <TrackingFeature.h>


void 
TrackingFeature::process (vector<TrackingFeatureData> &data)
{
	int32 size = data.size();
	if (size<2) {
		cerr << "TrackingFeature: at least 2 values required!" << endl;
		return;
	} // if


	// approximate: f(frame) = alpha + beta*frame
	ieee64 muFrame = 0;
	ieee64 muValue = 0;
	for (int32 i=0; i<size; i++) {
		muFrame += data[i].frame;
		muValue += data[i].value;
	} // for
	muFrame /= size;
	muValue /= size;


	ieee64 sxx = 0;
	ieee64 sxy = 0;
	ieee64 v1 = 0;
	ieee64 v2 = 0;
	for (int32 i=0; i<size; i++) {
		v1 = data[i].frame - muFrame;
		v2 = data[i].value - muValue;
		sxx += v1*v1;
		sxy += v1*v2;
	} // for
	sxx /= size-1;
	sxy /= size-1;

	if (sxx <= 0) sxx = 10e-10;
	ieee64 beta  = sxy / sxx;
	ieee64 alpha = muValue - beta*muFrame;


	ieee64 muError = 0.0;
	for (int32 i=0; i<size; i++) {
		data[i].error = fabs(data[i].value- (alpha + beta*data[i].frame));
		muError += data[i].error;
	} // for
	muError /= size;

	for (int32 i=0; i<size; i++) {
		data[i].valid = 0;
		if (data[i].error > 1*muError) data[i].valid++;
		if (data[i].error > 2*muError) data[i].valid++;
		if (data[i].error > 3*muError) data[i].valid++;
		if (data[i].error > 4*muError) data[i].valid++;
	} // for

	for (int32 i=0; i<size; i++) {
		int32 frame = data[i].frame;
		ieee64 error = data[i].error;
		int32 valid = data[i].valid;
		cerr << "frame: " << frame << "  valid: " << valid << "  error:" << error << endl;
	} // for

	cout << endl;
		
	return;
}

