/*
   This file is part of the MoCA video library.
   Name: ColorSubsamplingFilter.h 
   Copyright (C) 2005 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef COLOR_SUBSAMPLING_FILTER_H
#define COLOR_SUBSAMPLING_FILTER_H

// MoCA includes
#include<Image.h>
#include<GrayscaleFilter.h>
#include<OImageStream.h>
#include<HistogramFeature.h>
#include<BinarizeFilter.h>
#include<CannyFilter.h>



enum ColorSubsamplingFilterMode {
  ADAPTIVE_COLOR_SUBSAMPLING,
  LINEAR_COLOR_SUBSAMPLING
};

/*!

  \brief Filter reduces color depth.

  This filter reduces color depth of the image.

 */
class ColorSubsamplingFilter 
{
public:
    //! the default constructor
    /*!
      \param number if colors should be between 2 and 255
	         TODO: create color images (image is always converted to a grayscale image)
     */
    ColorSubsamplingFilter  (int numberOfColors = 16, ColorSubsamplingFilterMode mode = ADAPTIVE_COLOR_SUBSAMPLING) 
	{
		this->numberOfColors=numberOfColors;
		if (this->numberOfColors <  2) this->numberOfColors=2;
		if (this->numberOfColors >256) this->numberOfColors=256;
		this->mode=mode;
	}

    //! the destructor
    ~ColorSubsamplingFilter (){}

    //! the filter
    void process(Image<uint8> &srcDest)
	{
		Image<uint8> tmp=srcDest;
		process(tmp,srcDest);
		return;
	}

    //! the filter
    void process(const Image<uint8> &src, Image<uint8> &dest);

private:
	void createBinary(Image<uint8> &img);
	void subsample_adaptive (Image<uint8> &dest);
	void subsample_linear (Image<uint8> &dest);

	
    //! color depth (number of colors)
    int numberOfColors;

    //! adaptation method
	ColorSubsamplingFilterMode mode;
}; // ColorSubsamplingFilter


#endif
