/*
   This file is part of the MoCA video library.
   Name: ContrastFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   \author    Stephan Kopf

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kopf@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

*/

#ifndef CONTRAST_FILTER_H
#define CONTRAST_FILTER_H

// MoCA includes
#include <MocaExceptions.h>
#include <ImageFilterBase.h>


/*!

  \brief Filter enhances contrast.

  This filter modifies the contrast of the image.

 */
class ContrastFilter 
{
public:
    //! the default constructor
    /*!
      \param contrastValue should range between -255 and +255
	         a positive value means an increase of contrast 
     */
    ContrastFilter(int contrastV = 40) 
	{
		contrastValue = contrastV;
		if (contrastValue < -255) contrastValue = -255;
		if (contrastValue >  255) contrastValue =  255;
	}

    //! the destructor
    ~ContrastFilter(){}

    //! the filter
    void process(Image<uint8> &srcDest)
	{
		Image<uint8> tmp=srcDest;
		process(tmp,srcDest);
		return;
	}

    //! the filter
    void process(const Image<uint8> &src, Image<uint8> &dest);

private:
    //! contrast value
    int contrastValue;
}; // ContrastFilter


#endif
