/*
   This file is part of the MoCA video library.
   Name: EdgeFilter.h
   Copyright (C) 2004 Markus Beier

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: beier@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Markus Beier

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: beier@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: EdgeFilter.h,v 1.2 2004/03/02 17:04:11 kopf Exp $

*/

#ifndef EDGE_FILTER_H
#define EDGE_FILTER_H

// C includes
#include <math.h>

// MoCA includes
#include <Image.h>
#include <ImageFilterBase.h>

/*!

  \brief Class to filter the edges of a given image

  This class implements an edge finding filter. Use this class to
  filter the edges of the image. The image itself will be changed to
  the edge filtered image.

*/
template <class T>
class EdgeFilter: public ImageFilterBase<T>
{
public:

	/*!
    This constructor takes up tp 2 parameters
  
	\param barrier of type uint16 above which a found "edge" will be marked as one
	\param filterStyle indicating the style of the filter. "monochrome" for a black&white image or "grayscale" for a grayscale image.
	\sa process()
	*/
    EdgeFilter(uint16 barrier = 50, string filterStyle = "monochrome")
		:barrier(barrier), filterStyle(filterStyle) {}

    //! Default destructor
    ~EdgeFilter(){}

    //! Filtering the edges of the image
    void process (Image<T>& srcDest)
	{
		Image<T> tmp = srcDest;
		process(tmp, srcDest);
		return;
	}
    void process (const Image<T>& src, Image<T>& dest);

private:
    //! direction needed to indicate the iteration direction
    enum direction {horizontal, vertical};

    //! temp images to hold the temporary deviation data
    Image<int32> tempH, tempV;
    indexType width, height, band;

    //! this function iterates over the image in a given direction, evaluates the edge indicating data and stores it in the corresponding temp image
    void iterator (const Image<T>& src, indexType Band, double lowCoef, double valueCoef, double highCoef, direction dir);

	uint16 barrier;
	string filterStyle;
}; //class EdgeFilter

#include <EdgeFilter.cpt>

#endif
