/*
   This file is part of the MoCA video library.
   Name: ErosionFilter.h
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: ErosionFilter.h,v 1.1 2004/02/17 11:53:52 kopf Exp $

*/

#ifndef EROSION_FILTER_H
#define EROSION_FILTER_H

#include <MocaExceptions.h>
#include <ImageFilterBase.h>


/*!

  \brief An Erosion Filter

  This filter lays a mask of a given width over each pixel of the
  source image and writes the minimum value of this mask into the
  pixel in the destination image.

 */
template<class T>
class ErosionFilter : public ImageFilterBase<T>
{
public:
    //! the default constructor
    /*!
      \param width the width of the mask, must be odd
     */
    ErosionFilter(int width = 5, bool invertColors = false);

    //! the filter
    void process(Image<T> &srcDest);

    //! the filter
    void process(const Image<T> &src, Image<T> &dest);

    //! the destructor
    ~ErosionFilter();

private:
    //! mask width
    int width;
	bool invertColors;
}; // ErosionFilter


#include <ErosionFilter.cpt>

#endif
