/*
   This file is part of the MoCA video library.
   Name: LuminanceFilter.cpp
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include<LuminanceFilter.h>


/*!
  The filter itself.

  \param source image
  \param dest image  
*/
void
LuminanceFilter::process(const Image<uint8> &src, Image<uint8> &dest)
{
    ImageSize size = src.getSize();
    dest.resize(size);

	if (value >= 1.0) {

		// for each pixel
		for (int band = 0; band < size[DIM_BAND]; band++) {
			for (int y = 0; y < size[DIM_Y]; y++) {
				for (int x = 0; x < size[DIM_X]; x++) {
					
					ieee64 v = src (x,y,band);
					v += (maxV-v)/maxV * value;
					if (v<0) v=0;
					if (v>255) v=255;
					
					dest (x,y,band) = (uint8)v;
				} // for
			} // for
		} // for
	} else if (value <= -1.0) {

		// for each pixel
		for (int band = 0; band < size[DIM_BAND]; band++) {
			for (int y = 0; y < size[DIM_Y]; y++) {
				for (int x = 0; x < size[DIM_X]; x++) {
					
					ieee64 v = src (x,y,band);
					v += (v)/maxV * value;
					if (v<0) v=0;
					if (v>255) v=255;
					
					dest (x,y,band) = (uint8)v;
				} // for
			} // for
		} // for
	} else {
		dest = src;
	} // else


    return;
} // filter
