/*
   This file is part of the MoCA video library.
   Name: ModifiedBlockMotionFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef BLOCKMOTIONFILTER_H
#define BLOCKMOTIONFILTER_H

#include <MocaTypes.h>
#include <Video.h>
#include <LucasKanadeFilter.h>
#include <GaussFilter.h>


//! Calculation of motion vectors based on block search around "reliable" centers of search
/*!
  To be inserted
 */

template<class T>
class ModifiedBlockMotionFilter
{
public:
    //! The constructor
    /*!
      \param blockRadius of Type sizeType indicates the radius of the block around a reliable pixel for which block motion will be calculated
      \param searchRange of Type sizeType indicates the maximum expected motion between two frames
      \param relThresh of Type ieee64 indicates a threshold above which a pixel will be taken as reliable. It refers to a reliablility measure calculated internally which ranges from 0 to 1. 0.7 or 0.8 seems to be a good choice.
      \param noiseLevel of type ieee64 is a threshold influencing the reliability measure. The higher this threshold the better a pixel must be to be regarded as a reliable pixel. Values should range from 5 to 30
      While calculating the reliability measure the method will produce an output for each frame consisting in a png image called rel<number of frame>.png. The red pixel are the most reliable, then follow the green ones an the blue ones. Grayscaled Pixel are the least reliable pixel (below a level of 0.6).
    */
    ModifiedBlockMotionFilter(sizeType blockRadius, sizeType searchRange, ieee64 relThresh, ieee64 noise);

    //! The destructor
    ~ModifiedBlockMotionFilter() {}

    //! Calculate motion vectors
    /*!
      This method first calculates
     */

    void process (const Video<T>& src, Video<ieee64>& dest);

private:
    //! This method calculates a reliability measure for the input video
    void calculateReliability(const Video<T>& src, Video<ieee64>& reliability, ieee64 noiseLevel);

    //! This method produces an output video showing the reliabily values
    void showReliablePixel(const Video<T>& src, const Video<ieee64>& reliability, char* filename);

    ieee64 match(const Video<T>& src, indexType frame,
                 indexType curX, indexType curY,
                 indexType nextX, indexType nextY);

    ieee64 matchBM(const Video<T>& src, indexType frame,
                 indexType curX, indexType curY,
                 indexType nextX, indexType nextY);


    //! This methods cross compares blocks in consecutive frames around reliable pixel
    void crossCompare(const Video<T>& src, const Video<ieee64>& reliability, Video<ieee64>& dest);

    //! The threshold for the reliability
    ieee64 relThresh;

    //! the noiseLevel for reliability
    ieee64 noiseLevel;

    //! The size of the macroblocks
    sizeType blockRadius;
    //! The search range for the full search algorithm
    sizeType searchRange;
}; // BlockMotionFilter


#include <ModifiedBlockMotionFilter.cpt>

#endif
