/*
   This file is part of the MoCA video library.
   Name: ShearFilter.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


/* -*- C++ -*-

   ShearFilter.h  -- implementation of the flip filter

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/



#ifndef SHEAR_FILTER_H
#define SHEAR_FILTER_H

#include <ImageFilterBase.h>
#include <Image.h>

/*
  Shears an image:

  SHEAR_DIM_X:
   ----         ----
  |    |      /    /
  |    |     /    /
   ----      ----
  
  SHEAR_DIM_Y:
   ----      
  |    |    |\    
  |    |    | \   
   ----     |  \
             \  |
              \ |
               \| 
  
  The shearing algorithm is adapted from
  "A Fast Algorithm for General Raster Rotation"
  by Alan Paeth, Graphics Interface '86, pp. 77-81.
*/

enum {SHEAR_UNKNOWN_TYPE = 0, SHEAR_DIM_X=1, SHEAR_DIM_Y=2};

template<class T>
class ShearFilter : public ImageFilterBase<T>
{
public:
    ShearFilter (int mode, ieee64 shear)
    {
        this->mode = mode;
        this->shear = shear;

        if ((mode != SHEAR_DIM_X) && (mode != SHEAR_DIM_Y)) {
            this->shear = SHEAR_UNKNOWN_TYPE;
        } // if
        if (this->shear == SHEAR_UNKNOWN_TYPE) {
            cout << "ShearFilter ()  unknown mode" << endl;
            exit(1);
        } // if
    } //  constructor

    //! Apply shear filter to src, write out into dest
    void process (const Image<T>& src, Image<T>& dest);

    //! Apply shear filter to src, write into same image
    void process (Image<T>& srcDest);

private:
    ieee64 shear;
    int mode;
}; // ShearFilter

#include <ShearFilter.cpt>

#endif

