/*
    FileHandling.cpp     many important IO routines for files,directories,...
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <FileHandling.h>

void
FileHandling::
deleteFile (const string &filename, bool displayMessage)
{
	// spaces in path/filename
	string fn=filename;
	convertToValidFilename (fn);

	// define command
	string cmd = "";
#ifdef _WIN32
	cmd = "del /Q " + fn;
#else
	cmd = "rm " + fn;
#endif

	// execute command
	if (displayMessage) cout << cmd << endl;
	system(cmd.c_str());

	return;
} // deleteFile


void
FileHandling::
copyFile (const string &filenameSrc, const string &filenameDest, bool displayMessage)
{
	string fnSrc=filenameSrc;
	string fnDest=filenameDest;

	// spaces in path/filename
	convertToValidFilename (fnSrc);
	convertToValidFilename (fnDest);

	convertToUnixFilename(fnSrc);
	convertToUnixFilename(fnDest);

	// delete existing destination file
	deleteFile (fnDest, displayMessage);


	// define command
	string cmd = "";
#ifdef _WIN32
	cmd = "copy " + fnSrc + " " + fnDest;
#else
	cmd = "cp " + fnSrc + " " + fnDest;
#endif

	// execute command
	if (displayMessage) cout << cmd << endl;
	system(cmd.c_str());

	return;
} // copyFile



void
FileHandling::
deleteDirectory (const string &pathName, bool displayMessage)
{
	// spaces in path/filename
	string path=pathName;
	convertToValidFilename (path);

	// define command
	string cmd = "";
#ifdef _WIN32
	cmd = "rmdir " + path;
#else
	cmd = "rmdir " + path;
#endif

	// execute command
	if (displayMessage) cout << cmd << endl;
	system(cmd.c_str());

	return;
} // deleteFile


void
FileHandling::
renameFile (const string &filenameSrc, const string &filenameDest, bool displayMessage)
{
	string fnSrc=filenameSrc;
	string fnDest=filenameDest;

	// spaces in path/filename
	convertToValidFilename (fnSrc);
	convertToValidFilename (fnDest);

	// delete existing destination file
	if (fileExists (fnDest)) {
		deleteFile (fnDest, displayMessage);
	} // if


	// define command
	string cmd = "";
#ifdef _WIN32
	cmd = "move " + fnSrc + " " + fnDest;
#else
	cmd = "mv " + fnSrc + " " + fnDest;
#endif

	// execute command
	if (displayMessage) cout << cmd << endl;
	system(cmd.c_str());

	return;
} // renameFile


void
FileHandling::
createDirectory (const string &longDirectoryName, bool displayMessage)
{
	// remove leading or ending '"'
	string tmp=longDirectoryName;
	while (tmp[0] == '"') tmp = tmp.substr(1);
	while (tmp[tmp.size()-1] == '"') tmp = tmp.substr(0,tmp.size()-1);

	string dirName=tmp;

	string::size_type pos1=0;
	string::size_type pos2=0;
	char pathChar = '/';
#ifdef _WIN32 
	pathChar = '\\';
#endif
    // replace '/' with pathChar
	for (pos1=0; pos1<dirName.size(); pos1++) {
        if (dirName[pos1] == '/') {
            dirName[pos1] = pathChar;
        } // if
        if (dirName[pos1] == '\\') {
            dirName[pos1] = pathChar;
        } // if
    } // for
    // add last pathChar
    if (dirName[dirName.size()-1] != pathChar) {
        dirName += pathChar;
    } // if


    pos1 = 0;
    pos2 = 0;
    do {
        pos1 = dirName.find(pathChar, pos1);
        pos2 = dirName.find(pathChar, pos1+1);
        
        // get text between slashes
        string result;
        if (pos2 != string::npos) {      
            result.assign(dirName, 0, pos2);

            // if file status can be obtains, file already exists => don't create it
            struct stat buf;
            if (stat(result.c_str(), &buf) != 0) {

				// spaces in path
				convertToValidFilename (result);


				// define command
				string cmd = "";
#ifdef _WIN32
				cmd   = "md " + result;
#else
				cmd   = "mkdir " + result;
#endif

				// execute command
				if (displayMessage) cout << cmd << endl;
				system(cmd.c_str());
            
			} // if
        } // if
        pos1++;
    } while (string::npos != pos2);

	return;
} // createDirectory


void
FileHandling::
convertToValidFilename (string &filename)
{
	// path contains spaces
	string::size_type pos = filename.find(" ", 0);
	if (pos!=string::npos) {

		// if path/filename does not contain '"' then add '"'
		pos=0;
		pos = filename.find("\"", 0);
		if (pos==string::npos) {
			filename = "\"" +  filename +  "\"";
		} // if
	} // if

	// path contains slash and backslash "\\" and "/"
	char pathChar = '/';
#ifdef _WIN32 
	pathChar = '\\';
#endif
	for (pos=0; pos<filename.size(); pos++) {
		if (filename[pos] == '/') {
			filename[pos] = pathChar;
		} // if
		if (filename[pos] == '\\') {
			filename[pos] = pathChar;
		} // if
	} // for

	// if there is no path specified, add current path
	if (filename != "") { // do not modify empty string
		pos=string::npos;
		pos = filename.find(pathChar, 0);
		if (pos==string::npos) {
			string currentDirectory="";
			getCurrentDirectory (currentDirectory);
			filename = currentDirectory + filename;
		} // if
	} // if

	return;
} // convertToValidFilename


void
FileHandling::
convertToValidUnixFilename (const string &srcFilename, string &destFilename)
{
    destFilename = srcFilename;
    convertToValidFilename (destFilename);
    convertToUnixFilename (destFilename);
    return;
}

void
FileHandling::
convertToUnixFilename (string &filename)
{
	// path contains slash and backslash "\\" and "/"
	char pathChar = '/';
	for (string::size_type pos=0; pos<filename.size(); pos++) {
		if (filename[pos] == '/') {
			filename[pos] = pathChar;
		} // if
		if (filename[pos] == '\\') {
			filename[pos] = pathChar;
		} // if
	} // for

	return;
} // convertToUnixFilename


void
FileHandling::getCurrentDirectory (string &directory)
{
	directory = "";

#ifdef _WIN32
	char buffer[4096];
	if( _getcwd( buffer, 4096) == NULL ) {
		cerr << "FileHandling::getCurrentDirectory:  Unable to get current directory "<< endl;
		exit(1);
	} else {
		directory = buffer;
	} // else
#else
	char buffer[4096];
	if( getcwd( buffer, 4096) == NULL ) {
		cerr << "FileHandling::getCurrentDirectory:  Unable to get current directory "<< endl;
		exit(1);
	} else {
		directory = buffer;
	} // else
#endif

    for (int32 p=0; p<(int32)directory.size(); p++) {
        if (directory[p] == '\\') {
            directory[p] = '/';
        } // if
    } // for
    if (directory[directory.size()-1] != '/') {
        directory += "/";
    } // if

	return;
}


void
FileHandling::setCurrentDirectory (const string &currentDirectory)
{
	string directory = currentDirectory;
    for (int32 p=0; p<(int32)directory.size(); p++) {
        if (directory[p] == '\\') {
            directory[p] = '/';
        } // if
    } // for
    if (directory [directory.size()-1] != '/') {
        directory += "/";
    } // if


#ifdef _WIN32
	if ( _chdir(directory.c_str()) != 0) {
		cerr << "FileHandling::setCurrentDirectory()  Unable to change to directory ("<< directory << ")." << endl;
	} // if

#else
	if ( chdir(directory.c_str()) != 0) {
		cerr << "FileHandling::setCurrentDirectory()  Unable to change to directory ("<< directory << ")." << endl;
	} // if
#endif

	return;
}






bool
FileHandling::
fileExists (const string &pathFilename)
{
	if (pathFilename == "") return false;

	// remove leading or ending '"'
	string tmp=pathFilename;
	while (tmp[0] == '"') tmp = tmp.substr(1);
	while (tmp[tmp.size()-1] == '"') tmp = tmp.substr(0,tmp.size()-1);

	// does the file exist? 
	bool error = true;
	try {
		ifstream ifs (tmp.c_str(), std::ios::in | std::ios::binary); 
		//ifstream ifs (pathFilename.c_str(), std::ios::in | std::ios::binary); 
		error = !ifs.is_open();
		if (error) return false;

	    char c;
		if (!(ifs >> c)) error=true;
		if (ifs.rdstate() & ios::failbit) {
			error = true;
		} // if

		ifs.close();
	} catch (MocaException ex) {} 	
	if (error) return false;

	return true;
} // fileExists


void
FileHandling::
getFileSize (string &pathFilename, int32 &sizeInBytes)
{
	sizeInBytes = 0;
	try {
	    ifstream istr(pathFilename.c_str(), std::ios::in | std::ios::binary);
	    istr.seekg(0,std::ios::end);
	    long length=istr.tellg();
	    if (length>=0) {
		sizeInBytes=length;
	    } // length
	} catch (...) {
		sizeInBytes = 0;
	} // catch
	return;
} 


void
FileHandling::
getFileExtension (string &pathFilename, string &extension)
{
	// get file extension and convert it to lower case letters
	extension="";
	string::size_type p = pathFilename.find_last_of('.');
	if (p!=string::npos) {
		string fileExt=pathFilename.substr(p+1);
		for (size_t i=0; i< fileExt.length(); i++) {
			fileExt[i] = tolower(fileExt[i]);  
		} // for
		extension=fileExt;
	} // if
	return;
} // getFileExtension

void
FileHandling::
getFileExtension (string &pathFilename, string &extension, string &beforeExtension)
{
	// get file extension and convert it to lower case letters
	extension="";
	beforeExtension="";

	string::size_type p = pathFilename.find_last_of('.');
	if (p!=string::npos) {
		string fileExt=pathFilename.substr(p+1);
		for (size_t i=0; i< fileExt.length(); i++) {
			fileExt[i] = tolower(fileExt[i]);  
		} // for
		extension=fileExt;
		beforeExtension =pathFilename.substr(0, p);
	} // if
	return;
} // getFileExtension
