/*
   This file is part of the MoCA video library.
   Name: FileList.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/

#ifndef FILE_LIST
#define FILE_LIST


// C++ includes
#include <vector>
#include <list>
#include <string>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaTypes.h>
#include <MocaExceptions.h>
#include <CreateMocaPath.h>
#include <FileHandling.h>

#ifndef COMPILER_MSVC
#include <FileHandling.h>
#endif

/*
  Class reads files in a directory (and subdirectories).

  The class is used to find all files with a special filename, e.g.
  all "img*.pgm" files
  results are sorted
*/


struct FileInfo
{
    FileInfo() 
	{
		name="";
		shortName="";
		path="";
	}
    ~FileInfo() {}
	string name;
	string shortName;
	string path;

    bool operator< (const FileInfo& other) const
    {
        return name < other.name;
    } // operator<
};



class FileList
{
	const static int RECURSE_DEPTH;

public:
    //! the default constructor
    FileList(string path="./", bool recurse = false, int depth = RECURSE_DEPTH) :
            recurse   (recurse),
            fileId    (""),
            path      (path),
            prefixId  (""),
            suffixId  (""),
            depth     (depth)
    {
       
        // get current working directory
		currentDirectory = "";
		FileHandling::getCurrentDirectory(currentDirectory);
	}
    
	// destructor	
	~FileList ()
    {
		FileHandling::setCurrentDirectory (currentDirectory);
	}


    // Interface
    void process (const string &fileId, vector<FileInfo>& fileList);

	// sort a filelist (it is not very efficient, to sort a vector)
    void sort (vector<FileInfo>& fileList);

    static void show (vector<FileInfo>&);

    // unselect all files with ID
    static void unselect (const string &fileId, vector<FileInfo>& fileList);

    // select all files with ID
    static void select (const string &fileId, vector<FileInfo>& fileList);

	// deletes all files in fileList from hard disc !!!
    void deleteFiles (vector<FileInfo>& fileList);

private:
    // Interface
	void init (); 
	void recurseDirectory (string&, vector<FileInfo>&, int);
	void addFile (string& path, string& fn, vector<FileInfo>& allFiles);
    


    // variables
	bool recurse;
	string fileId;
	string path;
    string prefixId;
    string suffixId;
    int depth;
	string currentDirectory;
};


#endif

