/*
    FormatedText.cpp     handle formated texts
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <FormatedText.h>

std::ostream& operator<< (ostream &os, const FormatedTextItem &item)
{
	// print status
	int status = item.status;
	if ((status<0) || (status>9))
		os << "status=" << status << "  ";
	else
		os << "status=" << status << "   ";

	// print tag
	//string s="<"+item.tag+">";
	string s=item.tag;
	while(s.length()<25) s+=" ";
	os << s + " ";

	// print name of data field
	s="["+item.dataName+"]";
	while(s.length()<30) s+=" ";
	os << s + " ";

	// print data
	os << item.data << endl;

	return os;
}



std::ostream& operator<< (ostream &os, const vector<FormatedTextItem> &data)
{
	vector<FormatedTextItem>::const_iterator it;

	for (it=data.begin(); it!=data.end(); it++) {
		os << (*it);
	} // for

	return os;
}



FormatedText::FormatedText(const string &file)
{
	filename = file;
	FileHandling::convertToValidFilename (filename);
}


bool 
FormatedText::openFile(ifstream &ifs) 
{
	bool error = true;
	try {
		ifs.open (filename.c_str(), std::ios::in | std::ios::binary); 
		error = !ifs.is_open();
	} catch (MocaException ex) {
		error = true;
	} // catch
	if (error) {
		cerr << "could not open file: " << filename << endl;
		return false;
	} // if
	ifs.unsetf(ios::skipws);

	return true;
} 

void 
FormatedText::createList (ifstream &ifs, vector<FormatedTextItem> &formatedTextData)
{
	bool eofError=false;
	string s1;
	int8 status; // -1: unknown, 0 : data, 1: startTag, 2 : endTag
	char *buf=NULL;
	int bufSize= 32 * 1024;
	buf = new char[bufSize];

	bool failFlag=false;
	while (!eofError) {
		s1 = "";
		status = -1;
		
		// read next line 
		ifs.getline(buf, bufSize);
		if (ifs.rdstate() & ios::badbit) {
			failFlag=true;
		} // if
		if (ifs.rdstate() & ios::failbit) {
			failFlag=true;
		} // if
		if (ifs.rdstate() & ios::eofbit) {
			eofError=true; 
		} // if
		s1 = buf;

		if (failFlag) {
			int32 v1=s1.length();
			int32 v2 = bufSize-10;
			if (v1>v2) {
				cerr << "FormatedText::buffer size increased." << endl;

				// delete buffer
				if (buf != NULL) {
					delete[] buf;
				} // if
				buf=NULL;

				// create buffer
				bufSize*=32;
				buf = new char[bufSize];

				// try to read line again
				ifs.clear();
				failFlag=false;
				ifs.getline(buf, bufSize);
				string s2 = buf;
				s1 += s2;

				if (ifs.rdstate() & ios::failbit) {
					failFlag=true;
					eofError=true; 
				} // if
			} // if
		} // if

		// store data
		if (s1.length()>0) {
			FormatedTextItem item;
			item.data   = s1;
			item.status = 0;
			formatedTextData.push_back (item);
		} // if
	} // while

	if (buf != NULL) {
		delete[] buf;
	} // if
	buf=NULL;


	// remove spaces in data fields
	vector<FormatedTextItem>::iterator it;
	for (it=formatedTextData.begin(); it!=formatedTextData.end(); it++) {
		string s = (*it).data;
		while ((s.length()>0) && (s[0] == ' ')) s = s.substr(1);
		while ((s.length()>0) && (s[s.length()-1] == ' ')) s = s.substr(0, s.length()-1);
		(*it).data = s;
	} // for

	return;
}

void 
FormatedText::removeFirstIdentifier (string &id, vector<FormatedTextItem> &FormatedTextData)
{
	// remove META tag
	vector<FormatedTextItem>::iterator it;
	for (it=FormatedTextData.begin(); it!=FormatedTextData.end(); it++) {

		if ((*it).tag == id) {
			(*it).tag      = (*it).dataName;
			(*it).dataName = (*it).data;
			(*it).data     = "";

			string s = (*it).dataName;
			string::size_type p = s.find_first_of(' ');
			if (p!=string::npos) {
				s = (*it).dataName.substr(p);
				(*it).data = s;
				s = (*it).dataName.substr(0, p);
				(*it).dataName  = s;
			} // if
		} // if
	} // for


	// print tags
	for (it=FormatedTextData.begin(); it!=FormatedTextData.end(); it++) {
		cout << "|" << (*it).tag << "|" << (*it).dataName << "|" << (*it).data << "|" << ends;
		cout << endl;
	} // for

	return;
}

void 
FormatedText::aggregateList (vector<FormatedTextItem> &FormatedTextData)
{
	vector<FormatedTextItem>::iterator it;
	for (it=FormatedTextData.begin(); it!=FormatedTextData.end(); it++) {

		string s1 = "";
		string s2 = "";
		string s3 = "";

		// get first word
		string s = (*it).data;
		string::size_type p = s.find_first_of(' ');

		// if space was found
		if (p!=string::npos) {
			s1 = s.substr(0, p);
			
			// get second word
			while ((p<s.length()) && (s[p] == ' ')) p++;
			s = s.substr(p);
			p = s.find_first_of(' ');
			if (p==string::npos) {
				s2 = s;
				s = "";
				p=0;
			} else {
				s2 = s.substr(0, p);
			} // else
			
			// get third parameter
			if (s.length()>0) {
				while ((p<s.length()) && (s[p] == ' ')) p++;
				s = s.substr(p);
				s3 = s;
			} // if
		} // if
		(*it).tag      = s1;
		(*it).dataName = s2;
		(*it).data     = s3;
	} // for

	return;
}

void 
FormatedText::findData (string &tagName, vector<FormatedTextItem> &FormatedTextData, FormatedTextItem &data)
{
	data.status   = -1;
	data.data     = "";
	data.dataName = "";
	data.tag      = "";
	vector<FormatedTextItem>::iterator it;
	for (it=FormatedTextData.begin(); it!=FormatedTextData.end(); it++) {
		string s = (*it).tag;
		if (s.compare(tagName) == 0) {
			data.tag      = (*it).tag;
			data.dataName = (*it).dataName;
			data.data     = (*it).data;
			data.status   = 1;
			return;
		} // if

		s = (*it).dataName;
		if (s.compare(tagName) == 0) {
			data.tag      = (*it).dataName;
			data.dataName = (*it).data;
			data.data     = "";
			data.status   = 1;
			return;
		} // if
	} // for

	return;
}

void 
FormatedText::findDataList (FormatedTextItem &firstSearchItem, FormatedTextItem &secondSearchItem, vector<FormatedTextItem> &srcList, vector<FormatedTextItem> &destList)
{
	FormatedTextItem tmp;
	vector<FormatedTextItem>::iterator it;
	vector<FormatedTextItem>::iterator firstIt = srcList.end();
	for (it=srcList.begin(); it!=srcList.end(); it++) {
		if (((firstSearchItem.tag == "") || (firstSearchItem.tag == (*it).tag)) &&
			((firstSearchItem.dataName == "") || (firstSearchItem.dataName == (*it).dataName))) {
			tmp = (*it);
			firstIt = it;
			break;
		} // if
	} // for

	vector<FormatedTextItem>::iterator lastIt = firstIt;
tmp = (*lastIt);
	if (lastIt!=srcList.end()) lastIt++;
tmp = (*lastIt);
	for (it=lastIt; it!=srcList.end(); it++) {
		if (((secondSearchItem.tag == "") || (secondSearchItem.tag == (*it).tag)) &&
			((secondSearchItem.dataName == "") || (secondSearchItem.dataName == (*it).dataName))) {
			tmp = (*it);
			lastIt = it;
			break;
		} // if
	} // for

	destList.clear();
	for (it=firstIt; it!=lastIt; it++) {
		FormatedTextItem tmp = (*it);
		destList.push_back(*it);
	} // for
	return;
} // findDataList 


void 
FormatedText::process (vector<FormatedTextItem> &data)
{
	// open file
	ifstream ifs;
	if (openFile(ifs)) {

		// read data
	    char c;
	
	    if (ifs >> c) {
		ifs.putback(c);
		
		// read data to FormatedTextList
		createList (ifs, data);
		
		// close file
		ifs.close();
		
		// aggregate FormatedTextList
		aggregateList (data);
		
		//string removeId = "META";
		//removeFirstIdentifier (removeId, data);
	    } // if
	    
	} // if
	return;
}

