/*
   This file is part of the MoCA video library.
   Name: IImageFormatStream.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IImageFormatStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef I_IMAGE_FORMAT_STREAM_H
#define I_IMAGE_FORMAT_STREAM_H

#include <Image.h>

/*!
  Abstract base class for input image format streams
*/
template<typename T>
class IImageFormatStream
{
public:
    //! the default constructor
    IImageFormatStream(istream &is);

    //! the destructor
    virtual ~IImageFormatStream();

    //! to open a stream
    void open(istream &);

    //! close the image stream, not is!
    void close();

    //! read an image
    virtual void read(Image<T> &) = 0;

    //! read an image (calls read)
    IImageFormatStream &operator>>(Image<T> &);

    //! check if "is" is of given format
    virtual bool checkFormat() = 0;

protected:
	char getChar(istream *is) throw (MocaException);
	void rmWhiteSpace(istream *is) throw (MocaException);
	int32 getInt(istream *is) throw (MocaException);


    //! the underlying input stream
    istream *is;

private:

}; // IImageFormatStream

#include <IImageFormatStream.cpt>

#endif
