/*
   This file is part of the MoCA video library.
   Name: IPpmVideoFormatStream.cpt
   Copyright (C) 2004 Oliver Schuster

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: schuster@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Oliver Schuster

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: schuster@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: IPpmVideoFormatStream.cpt,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#include <IPpmVideoFormatStream.h>


/*!
 */
void
IPpmVideoFormatStream::read(Video<uint8> &dest, int start, int num) throw (MocaException)
{

    // check if istream is open
    if (this->is == NULL) {
        throw MocaException("No open file stream, open stream first!");
    } // if

    // generate an image pgm stream
    IPpmFormatStream<uint8> ipfs(*this->is);

    int z = 0;

    // temporary buffer to hold images
    list<Image<uint8>*> imgList;

    ImageSize imgSize;

    // get first image seperately
    if (ipfs.checkFormat()) {
        Image<uint8> *img = new Image<uint8>;
        ipfs.read(*img);

        if (start <= z && z < start + num) {
            imgList.push_back(img);
        } // if

        imgSize = img->getSize();

        z++;
    } // if

    // get all other images
    while ( ipfs.checkFormat() && z < start + num) {
        Image<uint8> *img = new Image<uint8>;
        ipfs.read(*img);

        if (start <= z) {
            imgList.push_back(img);
        } // if

        z++;
    } // while

    // now we know the size of the video, resize it appropriately
    VideoSize vidSize(imgSize[DIM_X], imgSize[DIM_Y], z, imgSize[DIM_BAND]);

    dest.resize(vidSize);

    // copy images from temporary storage into video object
    for (int i = 0; i < z; i++) {
        Image<uint8> *img = imgList.front();
        imgList.pop_front();
        dest.setImage(i, *img);

        delete img;
    } // for

} // IPpmVideoFormatStream::read


/*!

  return if provided istream is a pgm image stream thingy

 */
bool
IPpmVideoFormatStream::checkFormat()
{

    // generate an image pgm stream
    IPpmFormatStream<uint8> ipfs(*this->is);

    // and call it to check the format for us
    return ipfs.checkFormat();

} // IPpmVideoFormatStream::checkFormat
