/*
   This file is part of the MoCA video library.
   Name: IYuvVideo.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

  IYuvVideo.h -- Interface for YUV video
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef I_YUV_VIDEO_H
#define I_YUV_VIDEO_H

// C includes
#include<stdio.h>

// C++ includes
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaEnum.h>
#include <MocaTypes.h>
#include <MocaSignal.h>
#include <FileHandling.h>
#include <Image.h>
#include <Video.h>
#include <VideoSize.h>
#include <OpenFile.h>


// this class opens the YUV video and returns the requested frames
class IYuvVideo
{
public:

	// constuctor defines filename and size of yuv video file
	IYuvVideo (const string &pathFile, int32 width, int32 height, VideoCodec cMode, bool interleave, bool silent=false);

	// constuctor defines filename of yuv video file
	IYuvVideo (const string &pathFile);

	// destructor
	~IYuvVideo ()
	{
		close();
	}
	
	void getImage(int32 frame, Image<uint8> &img);
	void getVideo(Video<uint8> &vid);

	VideoSize getSize()
	{
		return videoSize;
	} // getSize

	ImageSize getImageSize()
	{
		return imgSize;
	} // getImageSize

	VideoCodec getCodec()
	{
		return codec;
	} // getCodec
	bool isInterleaved()
	{
		return interleave;
	} // isInterleaved


private:
	// automatically calculate resolution of yuv video
	void getResolution(int32 sizeInBytes, int32 &width, int32 &height, VideoCodec &codec, ieee64 &distance, bool &interleave);

	// check is size is valid
	bool checkSize (int32 width, int32 height, VideoCodec c, bool interleave, int32 vidSize, ieee64 &imgDifference);

	// decode two image and get the difference between these images
	void getImgDifference (string &fn, int32 width, int32 height, VideoCodec c, bool interleave, ieee64 &diff);


	void close()
	{
		if (fh_in!=NULL) {
			fclose(fh_in);
		} // if
		fh_in = NULL;
		return;
	}
	void open()
	{
		close();
		
		fh_in=NULL;
		OpenFile::process(&fh_in, pathFilename, F_READ, false);
		return;
	}

	VideoCodec codec;
	ImageSize imgSize;
	VideoSize videoSize;
	FILE* fh_in;
	int32 bufferSize;
	bool getUVplanes;
	bool interleave;
	bool silent;

	// file name and index
	string pathFilename;
};

// TODO: create class similar to WriteImage (the file extension should be checked automatically!)
class ReadYVideo
{
public:
	ReadYVideo (const string &filename, const VideoSize &size, Video<uint8> &vid)
	{
		try {
			// open output filestream and write image
			VideoCodec codec = CODEC_Y;
			IYuvVideo reader (filename, size[DIM_X], size[DIM_Y], codec, false);
			VideoSize tmpSize = reader.getSize();

			vid.resize(tmpSize);
			reader.getVideo(vid);

		} catch (MocaException e) {
			cout << "ReadYVideo: " << e.what() << endl;
			exit (-1);
		} // try
	}
};




#endif
