/*
    MpegStatus.cpp     get status information of MPEG file (new version: VideoIndex.cpp)
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// MoCA includes
#include <MpegStatus.h>


std::ostream& operator<< (ostream& os, const MpegData &data)
{
	os << "version         : " << data.version << endl;
	os << "size[DIM_X]     : " << (int)data.size[DIM_X] << endl;
	os << "size[DIM_Y]     : " << (int)data.size[DIM_Y] << endl;
	os << "number of bands : " << (int)data.size[DIM_BAND] << endl;
	os << "number of frames: " << (int)data.size[DIM_Z] << endl;

	os << "filename        : " << data.filename << endl;
	os << endl;

	return os;
} // operator<<

OTagStream& operator<< (OTagStream& os, const MpegData& data)
{
    TagFormat::writeNodeValue(os, "version", data.version);
    TagFormat::writeNodeValue(os, "DIM_X", data.size[DIM_X]);
    TagFormat::writeNodeValue(os, "DIM_Y", data.size[DIM_Y]);
    TagFormat::writeNodeValue(os, "DIM_BAND", data.size[DIM_BAND]);
    TagFormat::writeNodeValue(os, "DIM_Z", data.size[DIM_Z]);

	TagFormat::writeNodeValue(os, "filename", data.filename);
	os << endl;

    return os;
} // operator<<

ITagStream& operator>> (ITagStream& is, MpegData& data)
{
    char c;
    bool failFlag = false;

	if (is >> c) {
		is.putback(c);

		TagFormat::readNodeValue(is, "version", data.version);
		TagFormat::readNodeValue(is, "DIM_X", data.size[DIM_X]);
		TagFormat::readNodeValue(is, "DIM_Y", data.size[DIM_Y]);
		TagFormat::readNodeValue(is, "DIM_BAND", data.size[DIM_BAND]);
		TagFormat::readNodeValue(is, "DIM_Z", data.size[DIM_Z]);

		TagFormat::readNodeValue(is, "filename", data.filename);
	} // if

	if (is.rdstate() & ios::failbit) {
		failFlag = true;
	} // if
    if (failFlag) {
		throw MocaException ("MpegStatus: Illegal MpegStatus format!");
	} else {
        is.clear();
    } 

    return is;
} // operator>> 




void 
MpegStatus::process(MpegData &data) 
{
	// read data
	ReadMocaData<MpegData> reader (pathFilename, MPEG_STATUS_FILENAME);
	bool dataValid = reader.process (MPEG_STATUS_VERSION, data); 

	if (!dataValid) {

		// create new index
		cerr << "MpegStatus: Creating index file ..." << endl;
		readValues(data);
		cerr << "MpegStatus: Creating index file ...done" << endl;
			
		// save index file
		WriteMocaData<MpegData> writer (pathFilename, MPEG_STATUS_FILENAME);
		writer.process(data);
	} // if

	return;
} // process


void 
MpegStatus::readValues(MpegData &data) 
{
	data.version = MPEG_STATUS_VERSION;

	//const int DEFAULT_GRAYSCALE_BUFFER = 2;
	VideoDecoder mpeg (pathFilename);
	data.size = mpeg.getSize();
	
	cerr << "MpegStatus: TODO " << endl;
	//if (mpeg.similarGrayscale()) {
	//	data.isColor = TYPE_NEAR_GRAYSCALE;
	//} // if

	// get path of filename, do not create new directory
	CreateMocaPath mocaPath (pathFilename, false);
	data.filename = mocaPath.file;
	
	return;
} // readValues
