/*
    NumberedFileList.cpp     handle numbered file lists
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <NumberedFileList.h>

// static variable
const int NumberedFileList::RECURSE_DEPTH = 5;


NumberedFileList::
NumberedFileList(const string &fileId, bool recurse)
{
    //cout << " debug1" << endl;
	this->fileId = fileId;
	path     = "";
	shortId  = "";
    prefixId = "";
    suffixId = "";
    //cout << " debug2" << endl;
	this->recurse = recurse;
	nrOfFiles = 0;
    searchIdLength = 6;
	fileList.clear();
    //cout << " debug3" << endl;
    //cout << "1:" << this->fileId << endl;
    //cout << "2:" << path << endl;
    //cout << "3:" << shortId << endl;
    //cout << "4:" << prefixId << endl;
    //cout << "5:" << suffixId << endl;
    //cout << "6:" << this->recurse << endl;
    //cout << "7:" << nrOfFiles << endl;
    //cout << "8:" << searchIdLength << endl;
	init();
    //cout << " debug4" << endl;
} // NumberedFileList()

//throw (MocaException);
//throw MocaException("File allready open...");

NumberedFileList::
~NumberedFileList ()
{
	fileList.clear();	
} // ~NumberedFileList

string&
NumberedFileList::
operator[] (sizeType i)
{
	if (i>=nrOfFiles) {
		cerr << "file with index " << i << " does not exist"<< endl;
		exit(1);
	} // if
    return fileList[i].name;
} // operator[]

const string&
NumberedFileList::
operator[] (sizeType i) const
{
	if (i>=nrOfFiles) {
		cerr << "file with index " << i << " does not exist"<< endl;
		exit(1);
	} // if
	return fileList[i].name;
} // operator[]


int
NumberedFileList::
size()
{
    return nrOfFiles;
} //getNrOfFiles

void 
NumberedFileList::
show(bool displayAll)
{
	if (recurse) {
		cout << endl << "recurse  : yes, depth: " << RECURSE_DEPTH << endl;
    } else {
		cout << endl << "recurse  : no" << endl;
	}
    cout << "fileID   : " << fileId   << endl;
    cout << "path     : " << path     << endl;
    cout << "shortID  : " << shortId  << endl;
    cout << "prefixID : " << prefixId << endl;
    cout << "suffixID : " << suffixId << endl;
    cout << "# files  : " << nrOfFiles<< endl;
	
	int i;
	for (i=0; i<nrOfFiles; i++) {
        cout << fileList[i].name << endl;
        if (displayAll) {
            cout << "(" << fileList[i].path << "|" << fileList[i].shortName << "|" <<  fileList[i].sort << ")" <<endl;
        } // if
	} // for
    cout << endl;
	return;
} // show





void
NumberedFileList::
init()
{
	// TODO  throw exception
	
	// define path
    // unsigned changed for LINUX
    // get path
	long p = (long)fileId.find_last_of('/');
	if (p == (long)string::npos) {
		path = "./";
		shortId  = fileId;
#ifdef _WIN32
		char buffer[4096];
		// get current working directory
		if( _getcwd( buffer, 4096) == NULL ) {
			cerr << "Unable to get current directory "<< endl;
			exit(1);
		} else {
			path = buffer;
			for (p=0; p<path.size(); p++) {
				if (path[p] == '\\') {
					path[p] = '/';
				} // if
			} // for
			if (path[path.size()-1] != '/') {
				path += "/";
			} // if
		} // else
#endif
	} else {
		path = fileId.substr(0, p+1);
		shortId  = fileId.substr(p+1);
	} // else
	
    // get prefix and suffix
	p = shortId.find_last_of('*');
    prefixId = "";
    suffixId = "";
	if (p != (long)string::npos) {
        prefixId = shortId.substr(0, p);
		suffixId = shortId.substr(p+1);
    } // if
	
    
	// change to directory
#ifdef _WIN32
	if( _chdir(path.c_str())) {					
		cerr << "Unable to locate the directory: "<< path << endl;
		exit(1);
	} // if
#else
	DIR* d = opendir(path.c_str());
	if (d == NULL) { 
		cerr << "Unable to locate the directory: "<< path << endl;
		exit(1);
	} // if
#endif
	
	list<NumberedDirInfo> allFiles;
	recurseDirectory (path, allFiles, 1);
    allFiles.sort();
	
	nrOfFiles = 0;
    bool numbers;
    unsigned int i;
    char c;
	list<NumberedDirInfo>::iterator it;
	for (it = allFiles.begin(); it != allFiles.end(); it++) {
		
		// check numbers
		numbers=true;
		if (prefixId.size() >= (*it).shortName.size() - suffixId.size()) {
			numbers=false;
		} else {
			for (i=prefixId.size(); i < (*it).shortName.size() - suffixId.size(); i++) {
				c = (*it).shortName[i];
				if ((c < '0') || (c > '9')) {
					numbers = false;
				} // if
			} // for
			if (numbers) {
				fileList.push_back(*it);	
			} // if
		} // else
	} // for
	allFiles.clear();
	nrOfFiles = fileList.size();
	return;
} // init



void 
NumberedFileList::
recurseDirectory (string& path, list<NumberedDirInfo>& allFiles, int depth)
{ 
	
    //cout << "PATH " << path << endl; 
    
	NumberedDirInfo temp;
	
	int actDepth = depth+1;
	if (actDepth > RECURSE_DEPTH) {
		return;
	}
	
#ifdef _WIN32
	
	// check if path exists
	if( !_chdir(path.c_str())) {					
		
		struct _finddata_t data;
		long dStatus;
		string fn;
		string newPath;
		
		if( (dStatus = _findfirst("*.*", &data)) != -1L ) { 
			fn = data.name;
			if (data.attrib & _A_SUBDIR) {
				if ((fn != ".") && (fn != "..")) {
					newPath = path + fn + "/";
					if (recurse) {
						recurseDirectory (newPath, allFiles, actDepth);
					} // 
				} // if
			} else {
				temp.name = path+fn;
				
				// check suffix and prefix
				if (prefixId.empty() || (fn.find (prefixId) == 0)) {
					if (suffixId.empty() || (temp.name.rfind(suffixId) == temp.name.size() - suffixId.size())) {
						temp.path = path;
						temp.shortName = fn;
						temp.sort = fn;
						
						// add sort criterium (1,..9,10,11 and 01,02,03,...)
						int sortSize = prefixId.size();
						while (temp.sort.size() < (prefixId.size() + suffixId.size()+ searchIdLength)) {
							temp.sort.insert(sortSize,"0");
						} // while
						allFiles.push_back(temp);
					} // if
				} // if
			} // else
			
			
			while( _findnext( dStatus, &data ) == 0 ) {
				fn = data.name;
				if (data.attrib & _A_SUBDIR) {
					if ((fn != ".") && (fn != "..")) {
						newPath = path + fn + "/";
						if (recurse) {
							recurseDirectory (newPath, allFiles, actDepth);
						} // if
					} // if
				} else {
					temp.name = path+fn;
					
					// check suffix and prefix
					if (prefixId.empty() || (fn.find (prefixId) == 0)) {
						if (suffixId.empty() || (temp.name.rfind(suffixId) == temp.name.size() - suffixId.size())) {
							temp.path = path;
							temp.shortName = fn;
							temp.sort = fn;
							
							// add sort criterium (1,..9,10,11 and 01,02,03,...)
							int sortSize = prefixId.size();
							while (temp.sort.size() < (prefixId.size() + suffixId.size()+ searchIdLength)) {
								temp.sort.insert(sortSize,"0");
							} // while
							allFiles.push_back(temp);
						} // if
					} // if
				} // else
			} // while
		} // if
		_findclose(dStatus);
	} // if
	
#else
	
    DIR* d = opendir(path.c_str());
    if (d != NULL) { 
		
		string newPath;
		struct dirent* dr;
		while ((dr = readdir(d))) {
			string fn = dr->d_name;
			
			newPath = path + fn + "/";
			DIR* isDir = opendir(newPath.c_str());
			if (isDir != NULL) {
				if ((fn != ".") && (fn != "..")) {
					if (recurse) {
						recurseDirectory (newPath, allFiles, actDepth);
					} // if
				} // if
			} else {
				temp.name = path+fn;
				
				// check suffix and prefix
				if (prefixId.empty() || (fn.find (prefixId) == 0)) {
					if (suffixId.empty() || (temp.name.rfind(suffixId) == temp.name.size() - suffixId.size())) {
						temp.path = path;
						temp.shortName = fn;
						temp.sort = fn;
							
						// add sort criterium (1,..9,10,11 and 01,02,03,...)
						int sortSize = prefixId.size();
						while (temp.sort.size() < (prefixId.size() + suffixId.size()+ searchIdLength)) {
							temp.sort.insert(sortSize,"0");
						} // while
						allFiles.push_back(temp);
					} // if
				} // if
			} // else
		} // while
	} // if
#endif
	return;
} // recurseDirectory



