/*
   This file is part of the MoCA video library.
   Name: NumberedFileList.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/

#ifndef NUMBERED_FILE_LIST
#define NUMBERED_FILE_LIST



// C++ includes
#include <vector>
#include <list>
#include <string>
#include <iostream>
using namespace std;

// C++ includes
#include <MocaTypes.h>
#include <MocaExceptions.h>

// C includes for directory commands
#ifdef COMPILER_MSVC
#include <direct.h>		
#include <io.h>		
#else
#include <dirent.h>	
#endif


/*
  Class reads files in a directory (and subdirectories).

  The class is used to find all files with a special filename, e.g.
  all "img*.pgm" files
  example results are sorted: 
	img001.pgm, img002.pgm, ... 
	img1.pgm, ...img9.pgm, img10.pgm, ...  
*/


struct NumberedDirInfo
{
    NumberedDirInfo() {}
    ~NumberedDirInfo() {}
	string name;
	string shortName;
	string path;
	string sort;

    bool operator< (const NumberedDirInfo& other) const
    {
        return sort < other.sort;
    } // operator<
};



class NumberedFileList
{
public:
    //! the default constructor
    NumberedFileList(const string &fileId="", bool recurse = false); // throw (MocaException);
    
	// destructor	
	~NumberedFileList ();

	// operator
	string& operator[](sizeType);
	const string& operator[](sizeType) const;

    // Interface
	int size ();
	void show(bool displayAll=false);

private:
    // Interface
	void init (); //throw (MocaException)
	void recurseDirectory (string&, list<NumberedDirInfo>&, int);
    


    // variables
	const static int RECURSE_DEPTH;

	bool recurse;
	string fileId;
	string path;
	string shortId;
    string prefixId;
    string suffixId;
    int nrOfFiles;
    int searchIdLength;
    vector<NumberedDirInfo> fileList;
};


#endif

