/*
   This file is part of the MoCA video library.
   Name: OMpegAudio.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

 OMpegAudio.h -- Interface for MPEG audio
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  

*/

#ifndef O_MPEG_AUDIO_H
#define O_MPEG_AUDIO_H

// MoCA includes
#include <MocaTypes.h>
#include <SignalSize.h>
#include <MocaSignal.h>
#include <WaveHeader.h>
#include <CreateMocaPath.h>
#include <OpenProcess.h>


// C includes
#include <stdio.h>

// C includes
#ifdef COMPILER_MSVC
#include <fcntl.h>
#include <io.h>
#endif


// C++ includes
#include <iostream>
using namespace std;



// an MPEG audio layer 2 encoder
class OMpegAudio 
{

private:	
	static const string WINDOWS_TOOLAME_EXECUTABLE;
	static const string LINUX_TOOLAME_EXECUTABLE;

public:

	// constuctor setting filename and WAVE information
	OMpegAudio (const string &pathFilename, int samplesPerSec, int channels);

	// destructor
	~OMpegAudio ();

	// encode signal
	void write (Signal<int16>& sig);

	// convert a WAV file to a MP2 file, no constructor required
	static void Wav2Mp2 (const string &srcFilenameWav, const string &destFilenameMp2, int samplesPerSecond, int channels);

private:

        // get MP2 encoder
        static string getMp2Encoder ()
	{
	    string cmd = LINUX_TOOLAME_EXECUTABLE;
#ifdef _WIN32
	    cmd = WINDOWS_TOOLAME_EXECUTABLE;
#endif
	    return cmd;
	} // getMp2Encoder


	// close file
	void close ();

	// initialize audio encoder 
	void initEncoder ();

	FILE* fh_out;
	string samplesPerSec;  // e. g. "44.1"
	int channels;
	string outputMp2Filename;
};


#endif
