/*
   This file is part of the MoCA video library.
   Name: OPngFormatStream.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: OPngFormatStream.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef O_PNG_FORMAT_STREAM_H
#define O_PNG_FORMAT_STREAM_H

// MoCA includes
#include <MocaTypes.h>

#ifdef HAVE_LIBPNG

#include <OImageFormatStream.h>
#include <GetType.h>
#include <PngParameter.h>

// png includes
#include <png.h>

// C includes
#include <limits.h>


/*!
  png format specific stream
*/
template<class T>
class OPngFormatStream : public OImageFormatStream<T>
{
public:
    //! the default constructor
    OPngFormatStream(ostream &os, const FormatParameter *fp = NULL) : OImageFormatStream<T>(os), pp(NULL) { open (os, fp); }

    //! the destructor
    ~OPngFormatStream() { if (pp != NULL) delete pp; }

    //! open format stream
    void open(ostream &, const FormatParameter *);

    //! write an image
    void write(const Image<T> &) throw (MocaException);

private:
    //! local png parameter pointer copy thingy
    PngParameter* pp;

    //! our own write method
    static void user_write_fn (png_structp, png_bytep, png_size_t);

    //! our own flush method
    static void user_flush_fn (png_structp);
}; // OPngFormatStream

#include <OPngFormatStream.cpt>

#endif

#endif

