/*
    OYuvVideo.cpp     store YUV videos (raw data)
    Copyright (C) 2004 Stephan Kopf

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <OYuvVideo.h>


OYuvVideo::OYuvVideo (const string &pathFilename, VideoCodec srcCodec, VideoCodec destCodec)
{
	fh_out = NULL;
	this->pathFilename = pathFilename;
	this->srcCodec = srcCodec;
	this->destCodec = destCodec;
	
	switch (destCodec) {
	case CODEC_Y:
	case CODEC_YUV444:
		break;
	default:
		cerr << "OYuvVideo:: destination codec not supported!" << endl;
		exit(1);
	} // switch
	
	switch (srcCodec) {
	case CODEC_Y:
	case CODEC_YUV444:
	case CODEC_RGB:
		break;
	default:
		cerr << "OYuvVideo:: source codec not supported!" << endl;
		exit(1);
	} // switch
	
	open();
}

void 
OYuvVideo::write(const Image<uint8> &src)
{
	Image<uint8> dest;
	ConvertColorSpace::process(srcCodec, destCodec, src, dest);

	ImageBuffer buf;
	buf.setImage (destCodec, dest);

	uint8* bufPtr = buf.ptr;
	for (int32 i=0; i<buf.bufSize; i++) {
		fwrite(bufPtr++, 1, 1, fh_out);
	} // for

	return;
}

void 
OYuvVideo::write(const Video<uint8> &src)
{
	Image<uint8> dest;
	Image<uint8> srcImg;

	for (int32 z=0; z<src.getSize()[DIM_Z]; z++) {
		srcImg = src.getImage(z);

		ConvertColorSpace::process(srcCodec, destCodec, srcImg, dest);

		ImageBuffer buf;
		buf.setImage (destCodec, dest);
		
		uint8 *bufPtr = buf.ptr;
		for (int32 i=0; i<buf.bufSize; i++) {
			fwrite(bufPtr++, 1, 1, fh_out);
		} // for
	} // for

	return;
}
