/*
   This file is part of the MoCA video library.
   Name: ProcessReader.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   ProcessReader.h  -- open a process, read all data and store it in memory 

   Author:    Stephan Kopf
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kopf@pi4.informatik.uni-mannheim.de

*/

#ifndef PROCESS_READER
#define PROCESS_READER


// C++ includes
#include <vector>
using namespace std;


// MoCA includes
#include <OpenFile.h>
#include <FileHandling.h>
#include <OpenProcess.h>


// C includes
#ifdef COMPILER_MSVC
#include <fcntl.h>
#endif




/*
  Class opens a process, reads all data  and stores it in memory
*/

struct ProcessReaderBuffer
{
public:
	ProcessReaderBuffer ()
	{
		size = 0;
	}
	char buf [1024];
	int32 size;
};

class ProcessReader
{

public:
    //! the default constructor
    ProcessReader(string &cmd) 
    {
		this->cmd=cmd;
		buf=NULL;
		size = 0;
		init();       
	}
    
	// destructor	
	~ProcessReader ()
    {
		close();
	}
	void getBufPtr (char *bufPtr) 
	{
		bufPtr = buf;
		return;
	}
	void convertToText (vector<string> &textLines);


private:
    // Interface
	void init (); 
	void close ()
	{
		if (buf != NULL) {
			delete[] buf;
		} // if
		return;
	}
    


    // variables
	int32 size;
	string cmd;
	char *buf;
};


#endif

