/*
    TagFormat.cpp     store data with tags 
    Copyright (C) 2004 Gerald Kuehne

	Lehrstuhl fuer Praktische Informatik IV
	Universitaet Mannheim
	68131 Mannheim
	Germany
	email: kopf@informatik.uni-mannheim.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <TagFormat.h>


void
TagFormat::
checkTag (istream& is, const char* tag, bool endTag)
{
    // start tag (endTag == false) has the form <tag>
    // end tag (endTag == true) has the form </tag>

    // should change this to stringstream
    char buffer[MAX_TAG_LENGTH];

    if (strlen(tag) > MAX_TAG_LENGTH - 1) {
        is.setstate (ios::failbit);
        return;
    }

    // check for '<'
    is >> buffer[0];
    if (buffer[0] != '<') {
        is.setstate (ios::failbit);
        return;
    }

    // check for '/' if end tag is requested
    if (endTag) {
        is >> buffer[0];
        if (buffer[0] != '/') {
            is.setstate (ios::failbit);
            return;
         }
    }


    // check for the tag name
    is.get (buffer, strlen(tag) + 1);
    if (strcmp(tag, buffer) != 0) {
        is.setstate (ios::failbit);
        return;
    }

    // check for '>'
    is >> buffer[0];
    if (buffer[0] != '>') {
        is.setstate (ios::failbit);
        return;
    }
}

// readNodeValue
void
TagFormat::
readNodeValue (istream& is, const char* tag, uint8& value)  
{
    readStartTag (is, tag);
	int t = 0;
    is >> t;
	value = (uint8) t;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, uint16& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, uint32& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
	return;
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, int8& value)  
{
    readStartTag (is, tag);
	int t = 0;
    is >> t;
	value = (int8) t;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, int16& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, int32& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, ieee32& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, ieee64& value)  
{
    readStartTag (is, tag);
    is >> value;
    readEndTag (is, tag);
}
void
TagFormat::
readNodeValue (istream& is, const char* tag, string& value)  
{
  readStartTag (is, tag);

  if (is.rdstate() & ios::failbit) {
    return;
  } // if

  // read string until '<'
  char buffer[3];
  value = "";

  is.unsetf(ios::skipws);
  is >> buffer[0];
  while (buffer[0] != '<') {
    value += buffer[0];
    is >> buffer[0];
  } // while
  is.setf(ios::skipws);
  is.putback('<');
  
  readEndTag (is, tag);
  return;
}

void
TagFormat::
readNodeValue (istream& is, const char* tag, vector<int16>& value)  
{
    readStartTag (is, tag);

	if (is.rdstate() & ios::failbit) {
		return;
	} // if

	int i;
    char buffer[3];
	value.clear();

    is >> buffer[0];
    while (buffer[0] != '<') {
		is.putback(buffer[0]);
		i = 0;
	    is >> i;
		value.push_back(i);
		is >> buffer[0];
		// is.setstate (ios::failbit);
    } // while
	is.putback('<');
    readEndTag (is, tag);
	return;
}

void
TagFormat::
readNodeValue (istream& is, const char* tag, bool& value)  
{
	string tmp;
	TagFormat::readNodeValue(is, tag, tmp);
	value = false;
	if (tmp == "yes") {
		value = true;
	} // if
}



// read list without TAG
void
TagFormat::
readNodeValue (istream& is, vector<ieee32>& value)  
{

	value.clear();
	ieee32 tmp;
	while (1) {
		tmp = 0;
		is >> tmp;
		if (is.rdstate() & ios::failbit) {
			return;
		} // if
		value.push_back(tmp);
	} // if
	return;
}
void
TagFormat::
readNodeValue (istream& is, vector<ieee64>& value)  
{

	value.clear();
	ieee64 tmp;
	while (1) {
		tmp = 0;
		is >> tmp;
		if (is.rdstate() & ios::failbit) {
			return;
		} // if
		value.push_back(tmp);
	} // if
	return;
}
void
TagFormat::
readNodeValue (istream& is, Signal<int32>& value, SignalSize &size)  
{
	value.resize(size);

	int32 tmp;
	for (int32 x=0; x<size[DIM_X]; x++) {
		for (int32 band=0; band<size[DIM_BAND]; band++) {
			tmp = 0;
			is >> tmp;
			value(x,band) = tmp;
		} // for
	} // for
	if (is.rdstate() & ios::failbit) {
		return;
	} // if
	return;
}
void
TagFormat::
readNodeValue (istream& is, Signal<ieee64>& value, SignalSize &size)  
{
	value.resize(size);

	ieee64 tmp;
	for (int32 x=0; x<size[DIM_X]; x++) {
		for (int32 band=0; band<size[DIM_BAND]; band++) {
			tmp = 0;
			is >> tmp;
			value(x,band) = tmp;
		} // for
	} // for
	if (is.rdstate() & ios::failbit) {
		return;
	} // if
	return;
}


// writeNodeValue
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const uint8 value, int level)  
{
    writeStartTag (os, tag, level);
    os << (int)value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const uint16 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const uint32 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const int8 value, int level)  
{
    writeStartTag (os, tag, level);
    os << (int)value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const int16 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const int32 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const ieee32 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const ieee64 value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const string& value, int level)  
{
    writeStartTag (os, tag, level);
    os << value;
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const vector<int16>& value, int level)  
{
    writeStartTag (os, tag, level);
	int i;
	for (i=0; i<(int)value.size(); i++) {
	    os << value[i] << endl;
	} // for
    writeEndTag (os, tag);
}
void
TagFormat::
writeNodeValue (ostream& os, const char* tag, const bool value, int level)  
{
    writeStartTag (os, tag, level);
	string tmp = "no";
	if (value) {
		tmp = "yes";
	} // if
    os << tmp;
    writeEndTag (os, tag);
}


// write list without TAG
void
TagFormat::
writeNodeValue (ostream& os, const vector<ieee32>& value)  
{
  for (uint32 i=0; i<value.size(); i++) {
    os << value[i] << endl;
  } // for
  return;
}
void
TagFormat::
writeNodeValue (ostream& os, const vector<ieee64>& value)  
{
  for (uint32 i=0; i<value.size(); i++) {
    os << value[i] << endl;
  } // for
  return;
}
void
TagFormat::
writeNodeValue (ostream& os, const Signal<int32>& value)  
{
	for (int32 x=0; x<value.getSize()[DIM_X]; x++) {
		for (int32 b=0; b<value.getSize()[DIM_BAND]; b++) {
		    os << value(x,b) << " ";
		} // for
		os << endl;
	} // for
	return;
}
void
TagFormat::
writeNodeValue (ostream& os, const Signal<ieee64>& value)  
{
	for (int32 x=0; x<value.getSize()[DIM_X]; x++) {
		for (int32 b=0; b<value.getSize()[DIM_BAND]; b++) {
		    os << value(x,b) << " ";
		} // for
		os << endl;
	} // for
	return;
}

