/*
   This file is part of the MoCA video library.
   Name: TagFormat.h
   Copyright (C) 2004 Gerald Kuehne

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file



   \author    Gerald Kuehne

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: kuehne@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: TagFormat.h,v 1.1 2004/02/17 12:00:48 kopf Exp $

*/

#ifndef TAG_FORMAT_H
#define TAG_FORMAT_H


// C++ includes
#include <vector>
#include <string>
#include <fstream>
#include <iomanip>
#include <iostream>
using namespace std;

// MoCA includes
#include <MocaSignal.h>
#include <ReadMocaData.h>
#include <MocaTypes.h>
#include <MocaExceptions.h>


class TagFormat
{
public:
    enum { MAX_TAG_LENGTH = 1024 };

    static void readStartTag (istream&, const char*);
    static void readEndTag (istream&, const char*);

    static void writeStartTag (ostream&, const char*, int level=0);
    static void writeEndTag (ostream&, const char*,   int level=0);



    static void readNodeValue (istream&, const char*, bool&);
    static void readNodeValue (istream&, const char*, uint8&);
    static void readNodeValue (istream&, const char*, uint16&);
    static void readNodeValue (istream&, const char*, uint32&);
    static void readNodeValue (istream&, const char*, int8&);
    static void readNodeValue (istream&, const char*, int16&);
    static void readNodeValue (istream&, const char*, int32&);
    static void readNodeValue (istream&, const char*, ieee32&);
    static void readNodeValue (istream&, const char*, ieee64&);
    static void readNodeValue (istream&, const char*, string&);
    static void readNodeValue (istream&, const char*, vector<int16>&);

	// read list without TAG
    static void readNodeValue (istream&, vector<ieee32>&);
    static void readNodeValue (istream&, vector<ieee64>&);
    static void readNodeValue (istream&, Signal<int32>&,  SignalSize &size);
    static void readNodeValue (istream&, Signal<ieee64>&, SignalSize &size);


    static void writeNodeValue (ostream&, const char*, const bool,           int level=0); 
    static void writeNodeValue (ostream&, const char*, const uint8,          int level=0); 
    static void writeNodeValue (ostream&, const char*, const uint16,         int level=0); 
    static void writeNodeValue (ostream&, const char*, const uint32,         int level=0); 
    static void writeNodeValue (ostream&, const char*, const int8,           int level=0); 
    static void writeNodeValue (ostream&, const char*, const int16,          int level=0); 
    static void writeNodeValue (ostream&, const char*, const int32,          int level=0); 
    static void writeNodeValue (ostream&, const char*, const ieee32,         int level=0); 
    static void writeNodeValue (ostream&, const char*, const ieee64,         int level=0); 
    static void writeNodeValue (ostream&, const char*, const string&,        int level=0); 
    static void writeNodeValue (ostream&, const char*, const vector<int16>&, int level=0); 

	// write list without TAG
    static void writeNodeValue (ostream&, const vector<ieee32>&); 
    static void writeNodeValue (ostream&, const vector<ieee64>&); 
    static void writeNodeValue (ostream&, const Signal<int32>&); 
    static void writeNodeValue (ostream&, const Signal<ieee64>&); 

private:
    static void checkTag (istream&, const char*, bool);
    static void insertWhitespace (ostream&, int level);
};

inline
void
TagFormat::
readStartTag (istream& is, const char* tag)
{
    checkTag (is, tag, false);
}

inline
void
TagFormat::
readEndTag (istream& is, const char* tag)
{
    checkTag (is, tag, true);
}

inline
void
TagFormat::
writeStartTag (ostream& os, const char* tag, int level)
{
	insertWhitespace (os, level);
    os << "<" << tag << ">";
}

inline
void
TagFormat::
writeEndTag (ostream& os, const char* tag, int level)
{
	insertWhitespace (os, level);
	if (level == -1) {
		os << "</" << tag << ">";
	} else {
		os << "</" << tag << ">" << endl;
	} // else
}


inline
void
TagFormat::
insertWhitespace (ostream& os, int level)
{
	for (int n=0; n<level*2; n++) {
		os << " ";
	} // for
}


#endif




