/*
   This file is part of the MoCA video library.
   Name: VideoEncoder.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

 VideoEncoder.h -- Interface for MPEG1/MPEG2 video
 
  Author: Stephan Kopf
  Lehrstuhl fuer Praktische Informatik IV
  Universitaet Mannheim, Germany
  email: kopf@pi4.informatik.uni-mannheim.de
  
   
*/

#ifndef VIDEO_ENCODER_H
#define VIDEO_ENCODER_H


// MoCA includes
#include <MocaTypes.h>
#include <Image.h>
#include <Video.h>
#include <OpenProcess.h>
#include <OpenFile.h>
#include <ConvertColorSpace.h>



#ifdef COMPILER_MSVC
#include <process.h>
#include <fcntl.h>
#include <io.h>
#endif



// C includes
#include <stdio.h>


// C++ includes
#include <sstream>
#include <string>
#include <iostream>
using namespace std;


enum EncoderCodecs {
  CODEC_WMV_1 = 0,
  CODEC_WMV_2 = 1,
  CODEC_MPEG1 = 2,
  CODEC_MPEG2 = 3,
  CODEC_MPEG4 = 4
};

enum EncodingQuality {
  QUALITY_LOW   = 0,
  QUALITY_HIGH  = 1
};

const string WINDOWS_VIDEO_ENCODER_EXECUTABLE = "mocaencoder.exe";
const string LINUX_VIDEO_ENCODER_EXECUTABLE   = "mocaencoder";


struct VideoEncoderOptions
{
public:
	VideoEncoderOptions () {
		pathFilename    = "out.avi";
		size [DIM_X]    = 0;
		size [DIM_Y]    = 0;
		size [DIM_BAND] = 3;
		size [DIM_Z]    = 1;
		framerate       = -1.0;		// use default frame rate
		bitrate         = -1;		// use default bit rate
		createGrayscale = false;
		encCodec		= CODEC_MPEG4;
		imgCodec		= CODEC_RGB;
		silent			= true;
		quality         = QUALITY_HIGH;
	}
	
	string pathFilename;
	ImageSize size;
	ieee64 framerate;
	int32 bitrate;			// in kbits/second [4..16000]
	bool createGrayscale;
	EncoderCodecs encCodec;
	VideoCodec imgCodec;
	bool silent;
	EncodingQuality quality;
};



class VideoEncoder 
{
	
public:
	
	// constuctor setting filename
	VideoEncoder (const VideoEncoderOptions &options);
	
	// destructor
	~VideoEncoder ();
	
	// write frame
	void write(const Image<uint8> &src);

	// close decoder
	void close();
	
private:
	void open();
	void writeData(const unsigned char *buf, int size);
	void writeRGB(const SampleBlock<uint8> &img);
	void writeYuv(const SampleBlock<uint8> &img);
	void writeYuvBlock(const SampleBlock<uint8> &img);
	

	inline int Clip(int x) 
	{ 
		if (x<0) return 0; 
		if (x>255) return 255; 
		return x; 
	} // Clip	


	FILE *fh_out;
	VideoEncoderOptions options;

	string cmd;
	int error;
};

#endif
