/*
   This file is part of the MoCA video library.
   Name: Pixel.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef PIXEL_H
#define PIXEL_H

// C++ includes
#include <vector>
#include <iostream>
using namespace std;


// MoCA includes
#include <ImagePosition.h>



/*!

  \brief Class to characterise a Pixel.

  The stored information are at the moment the x and y coordinates of
  a pixel and the value(s) of the pixel over each band.

  Furthermore, a method to compare two pixels based on the euclidian
  distance or something equivalent should be implemented.

  \todo a lot!

 */

template<class T> class Pixel;

template<class T>
class Pixel : public ImagePosition
{
public:

    //! Constructor
    Pixel() : ImagePosition() {}

    //! Constructor
    Pixel(indexType x, indexType y) : ImagePosition(x, y) {}

    //! Constructor
    Pixel(const ImagePosition &pos) : ImagePosition(pos) {}

    //! Destructor
    ~Pixel() {}

    //! operator >
    bool operator> (const Pixel&);
    //! operator >=
    bool operator>= (const Pixel&);
    //! operator <
    bool operator< (const Pixel&);
    //! operator <=
    bool operator<= (const Pixel&);
    //! operator ==
    bool operator== (const Pixel&);

    //! Get x position
    sizeType getX() const
    {
        return position[DIM_X];
    } // getX

    //! Set x position
    void setX(sizeType newX)
    {
        position[DIM_X] = newX;
    } // setX

    //! Get y position
    sizeType getY() const
    {
        return position[DIM_Y];
    } // getY

    //! Set y position
    void setY(sizeType newY)
    {
        position[DIM_Y] = newY;
    } // setY

    //! Get ImagePosition
    indexType *getPos()
    {
        return position;
    } // getPos

    //! Vector of pixelvalues (one for each band)
    vector<T> values;


#ifdef COMPILER_MSVC
    template<typename T>
    friend ostream& operator<< (ostream&, const Pixel<T>&);
#else
    template<typename S>
    friend ostream& operator<< (ostream&, const Pixel<S>&);
#endif

private:
}; // Pixel


template<class T>
inline
bool
Pixel<T>::
operator> (const Pixel<T>& other)
{
  return ((this->values[0]) > (other.values[0]));
} // Pixel<T>::operator>

template<class T>
inline
bool
Pixel<T>::
operator>= (const Pixel<T>& other)
{
  return ((this->values[0]) >= (other.values[0]));
} // Pixel<T>::operator>=

template<class T>
inline
bool
Pixel<T>::
operator< (const Pixel<T>& other)
{
  return ((this->values[0]) < (other.values[0]));
} // Pixel<T>::operator<

template<class T>
inline
bool
Pixel<T>::
operator<= (const Pixel<T>& other)
{
  return ((this->values[0]) <= (other.values[0]));
} // Pixel<T>::operator<=

template<class T>
inline
bool
Pixel<T>::
operator== (const Pixel<T>& other)
{
  return ((this->values[0]) == (other.values[0]));
} // Pixel<T>::operator==

template<class T>
ostream& operator<< (ostream& os, const Pixel<T> &p)
{
    os << "x: " << p.getX() << " y: " << p.getY();
    for (unsigned int i = 0; i < p.values.size(); i++) {
        os << " " << i << ".: " << (int)p.values[i];
    } // for
    return os;
} // operator<<  Pixel<T>

#endif




