/*
   This file is part of the MoCA video library.
   Name: SampleBlockPosition.h
   Copyright (C) 2004 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kuehne@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/* -*- C++ -*-

   file.h  -- short description

   Author:    Gerald Kuehne
              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim, Germany
              email: kuehne@pi4.informatik.uni-mannheim.de

   $Id: SampleBlockPosition.h,v 1.1 2004/02/17 12:00:58 kopf Exp $
*/

#ifndef SAMPLE_BLOCK_POSITION_H
#define SAMPLE_BLOCK_POSITION_H

// MoCA includes
#include <MocaTypes.h>
#include <MocaEnum.h>
#include <MocaMacros.h>

class SampleBlockPosition
{
public:
    SampleBlockPosition ();
    SampleBlockPosition (indexType x, indexType y, indexType z, indexType band);

    indexType& operator[](sizeType i);
    const indexType& operator[](sizeType i) const;

    int operator== (const SampleBlockPosition&) const;
    int operator!= (const SampleBlockPosition&) const;
	SampleBlockPosition& operator= (const SampleBlockPosition&);

protected:
    indexType position[DIM_N];
};

inline
SampleBlockPosition::
SampleBlockPosition ()
{
  position[DIM_X]    = 0;
  position[DIM_Y]    = 0;
  position[DIM_Z]    = 0;
  position[DIM_BAND] = 0;
}

inline
SampleBlockPosition::
SampleBlockPosition (indexType x, indexType y, indexType z, indexType band)
{
  position[DIM_X]    = x;
  position[DIM_Y]    = y;
  position[DIM_Z]    = z;
  position[DIM_BAND] = band;
}

inline
indexType&
SampleBlockPosition::
operator[] (sizeType i)
{
  return position[i];
}

inline
const indexType&
SampleBlockPosition::
operator[] (sizeType i) const
{
  return position[i];
}

inline
int
SampleBlockPosition::
operator== (const SampleBlockPosition& other) const
{
  return(position[DIM_X]    == other.position[DIM_X] &&
         position[DIM_Y]    == other.position[DIM_Y] &&
         position[DIM_Z]    == other.position[DIM_Z] &&
         position[DIM_BAND] == other.position[DIM_BAND]);
}

inline
int
SampleBlockPosition::
operator!= (const SampleBlockPosition& other) const
{
  return !(*this == other);
}

inline
SampleBlockPosition&
SampleBlockPosition::
operator= (const SampleBlockPosition& other)
{
  if (this != &other) {
	  position[DIM_X]    = other.position[DIM_X];
	  position[DIM_Y]    = other.position[DIM_Y];
	  position[DIM_Z]    = other.position[DIM_Z];
	  position[DIM_BAND] = other.position[DIM_BAND];
  }
  return *this;
}

#endif



