/*
   This file is part of the MoCA video library.
   Name: LUsolve.h
   Copyright (C) 2004 Markus

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: beier@informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*! \file

   Solution of a linear equation system using LU decomposition

   \author    Markus Beier

              Lehrstuhl fuer Praktische Informatik IV
              Universitaet Mannheim
              68131 Mannheim
              Germany
              email: beier@informatik.uni-mannheim.de

   (c) Lehrstuhl fuer Praktische Informatik IV, details in COPYRIGHT.txt

   $Id: LUsolve.h,v 1.1 2004/02/17 12:01:05 kopf Exp $

*/

#ifndef LU_SOLVE_H
#define LU_SOLVE_H

// C++ includes
#include <math.h>

// MoCA includes
#include <MocaTypes.h>
#include <MocaExceptions.h>
#include <Vector.h>
#include <Matrix.h>

class LUsolve
{
public:
    void solve(Matrix<ieee64>&, Vector<ieee64>&, Vector<ieee64>&) throw (MocaException);

private:
    void ludcmp(Matrix<ieee64>&, int&, Vector<int>&, ieee64& ) throw (MocaException);
    void lubksb(Matrix<ieee64>&, int&, Vector<int>&, Vector<ieee64>&);
};

#endif



