/*
   This file is part of the MoCA video library.
   Name: MocaTools.h
   Copyright (C) 2006 Stephan Kopf

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: kopf@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef MOCA_TOOLS_H
#define MOCA_TOOLS_H

//
// just include this file, if you have to include many different tools (headers) in your file
//
// include all header files in directory "tools"
#include <LUsolve.h>
#include <Matrix.h>
#include <MatrixSize.h>
#include <mmx.h>
#include <SadFeature_mmx.h>
#include <SvdFit.h>
#include <Vector.h>

void testTools ()
{
	// test LUsolve (solution of a linear equation system using LU decomposition)
	{
		const int ma = 8;
		const int ndata = 8;
		Matrix<ieee64> A(MatrixSize(ndata, ma));
		Vector<ieee64> b(8);
		ieee64 input[8][8] = {
			{-15, 10, 1, 0, 0, 0, -75, 50},
			{0, 0, 0, -15, 10, 1, 150, -100},
			{-14, -10, 1, 0, 0, 0, -56, -40},
			{0, 0, 0, -14, -10, 1, -140, -100},
			{16, 9, 1, 0, 0, 0, -416, -234},
			{0, 0, 0, 16, 9, 1, -144, -81},
			{16, -16, 1, 0, 0, 0, -416, 416},
			{0, 0, 0, 16, -16, 1, 256, -256}
		};
		for (int i = 0; i < 8; i++) {
			for (int j = 0; j < 8; j++) {
				A[i][j] = input[i][j];
				//cout << A[i][j] << " ";
			}
			//cout << endl;
		}
		b[0] = -5;
		b[1] = 10;
		b[2] = -4;
		b[3] = -10;
		b[4] = 26;
		b[5] =  9;
		b[6] = 26;
		b[7] = -16;
		Vector<ieee64> a(8);

		// solve equation system
		LUsolve lu;
		lu.solve(A,a,b);
		//for (int i = 0; i < a.getSize(); i++)
        //cout << ((fabs(a[i]) > 0.000001) ? a[i] : 0)  << endl;
	}


	// test processSAD2
	{
		ImageSize s(100,100);
		Image<uint8> img0(s);
		Image<uint8> img1(s);
		img0.clear(0);
		img1.clear(0);
		int32 px=24;
		int32 py=48;
		const uint8 *img0Ptr=img0.getSamplePtr(px-7,py-7);
		const uint8 *img1Ptr=img1.getSamplePtr(px-7,py-7);
		long int yOffset0 = img0.getSampleOffsetY(py-7);
		long int yOffset1 = img1.getSampleOffsetY(py-7);
		processSAD2 (img0Ptr, img1Ptr, yOffset0, yOffset1);
	}
	
	

	// test SvdFit (least squares fit by singular value decomposition)
	{
		const int ma = 8;
		const int ndata = 4*4 * 2;
		int x0[ndata/2] = {
			0, 8, 16, 24,
				0, 8, 16, 24,
				0, 8, 16, 24,
				0, 8, 16, 24
		};
		int y0[ndata/2] = {
			0, 0, 0, 0,
				8, 8, 8, 8,
				16, 16, 16, 16,
				24, 24, 24, 24
		};
		int x1[ndata/2] = {
			4, 12, 20, 28,
				4, 12, 20, 28,
				4, 12, 20, 28,
				4, 12, 20, 28
		};
		int y1[ndata/2] = {
			0, 0, 0, 0,
				8, 8, 8, 8,
				16, 16, 16, 16,
				24, 24, 24, 24
		};
		Matrix<ieee64> A(MatrixSize(ndata, ma));
		Vector<ieee64> b(ndata);
		int centerX = 16;
		int centerY = 16;
		int rowToFill = 0;
		for (int f = 0; f < ndata/2; f++) {
			int x    = x0[f] - centerX;
			int y    = centerY - y0[f];
			int xres = x1[f] - centerX;
			int yres = centerY - y1[f];
			
			// filling line for x coordinate
			A[rowToFill][0] = x;
			A[rowToFill][1] = y;
			A[rowToFill][2] = 1;
			A[rowToFill][3] = 0;
			A[rowToFill][4] = 0;
			A[rowToFill][5] = 0;
			A[rowToFill][6] = -x * xres;
			A[rowToFill][7] = -y * xres;
			b[rowToFill] = xres;
			rowToFill++;
			
			// filling line for y coordinate
			A[rowToFill][0] = 0;
			A[rowToFill][1] = 0;
			A[rowToFill][2] = 0;
			A[rowToFill][3] = x;
			A[rowToFill][4] = y;
			A[rowToFill][5] = 1;
			A[rowToFill][6] = -yres * x;
			A[rowToFill][7] = -y * yres;
			b[rowToFill] = yres;
			rowToFill++;
		}
		Vector<ieee64> a(ma);
		SvdFit svd;
		ieee64 chisq = svd.solve (A, a, b);
		//for (int i = 0; i < a.getSize(); i++)
		//    cout << ((fabs(a[i]) > 0.000001) ? a[i] : 0)  << endl;
		//cout << "error : " << sqrt(chisq) << endl;
	}
	
	return;
}

#endif

